dnl
dnl === list management of paths or options === 
dnl

	dnl ADD_LIST(listname, value, badvalue)
	dnl
AC_DEFUN(ADD_LIST, [
	if test X"$2" != X"$3"
	then
		if test X"[$]$1" = X"NONE"
		then
			$1=$2
		else
			$1="[$]$1 $2"
		fi
	fi
])

	dnl SUBST_UNLESS_NONE(makemacroname, valuename)
	dnl
AC_DEFUN(SUBST_UNLESS_NONE, [
	if test X"[$]$2" = X"NONE"
	then
		$1=
	else
		$1=[$]$2
	fi
	AC_SUBST($1)
])

	dnl FIND_LIB(cachename, libname, candidate)
AC_DEFUN(FIND_LIB, [
	AC_CACHE_CHECK(lib$2 library, $1, [
		for dir in $3
		do
			$1=[$]dir
			test -f [$]dir/lib$2.a && break
			test -f [$]dir/lib$2.so && break
			test -f [$]dir/lib$2.sl && break
			$1=NONE
		done
		if test [$]$1 = NONE
		then
			AC_MSG_ERROR(lib$2 library not found)
		fi
	])
])

	dnl FIND_INC(cachename, header, candidate)
AC_DEFUN(FIND_INC, [
	AC_CACHE_CHECK(header $2, $1, [
		for dir in $3
		do
			$1=[$]dir
			test -f [$]dir/$2 && break
			$1=NONE
		done
		if test [$]$1 = NONE
		then
			AC_MSG_ERROR(header $2 not found)
		fi
	])
])

#
# === charset management ===
#

AC_DEFUN(HOST_CHARSET, [
	AC_CHECK_PROG(SJISTOEUC, nkf, nkf -e)
	AC_CHECK_PROG(EUCTOSJIS, nkf, nkf -s)
	AC_ARG_WITH(charset,
		[charset: of source and resource (shift_jis or euc-jp)], [
		ac_cv_sys_charset=$withval
	], [
		AC_CACHE_CHECK([charset for source/resource],
			ac_cv_sys_charset, [
			case $host in
			*-*-*bsd*|*-*-linux*|*-*-sunos4.*)
				ac_cv_sys_charset=euc-jp
				;;
			*)
				ac_cv_sys_charset=shift_jis
				;;
			esac
		])
	])
	CHARSET=$ac_cv_sys_charset
	AC_SUBST(CHARSET)
])

dnl
dnl === fortran compilers ===
dnl

AC_DEFUN(AC_PROG_F90, [
  AC_ARG_WITH(f90, [f90: compiler command], [
    if test X"$withval" = X"yes"; then
      for command in f95 f90 lf95 frt
      do
        AC_CHECK_PROG(F90, $command, $command)
      done
    else
      ac_cv_prog_f90=$withval
    fi
  ], [
    AC_CACHE_CHECK([Fortran 90 compiler], ac_cv_prog_f90, ac_cv_prog_f90=NONE)
  ])
  AC_CHECK_PROG(F90, $ac_cv_prog_f90, $ac_cv_prog_f90)
  : ${F90:=NONE}
  AC_SUBST(F90)
])

AC_DEFUN(AC_PROG_F90_CFLINK, [
  if test X"$CFLINK" != X""; then
    ac_cv_prog_f90_clink=$CFLINK
  fi
  AC_CACHE_CHECK([Fortran and C linkage], ac_cv_prog_f90_clink, [
    if test X"$F90" = X"NONE"; then
      x=fortran-not-found
    else
      cat > xcfg1.c <<END_OF_HEREDOC[
sub(int *p) { *p = 101; }
sub_(int *p) { *p = 102; }
SUB(int *p) { *p = 201; }
]END_OF_HEREDOC
      ${CC} -c xcfg1.c
      cat > xcfg2.f90 <<END_OF_HEREDOC[
integer:: i
i = 0
call sub(i)
print '(i3)', i
end
]END_OF_HEREDOC
      $F90 -o xcfg ${FFLAGS} xcfg1.o xcfg2.f90
      if test -x ./xcfg; then
        x=`./xcfg`
      else
        x=compile-or-run-error
      fi
    fi
    if test $x = 101; then
      ac_cv_prog_f90_clink=LOWER
    elif test $x = 102; then
      ac_cv_prog_f90_clink=LOWER_
    elif test $x = 201; then
      ac_cv_prog_f90_clink=UPPER
    else
      ac_cv_prog_f90_clink=NONE
    fi
    rm -f xcfg*
  ])
  CFLINK=$ac_cv_prog_f90_clink
  AC_SUBST(CFLINK)
])

AC_DEFUN(AC_PROG_F90_AVAILABLE, [
  if test X"$F90" = X"NONE"; then
    F90_AVAILABLE=no
  else
    F90_AVAILABLE=yes
  fi
  AC_SUBST(F90_AVAILABLE)
])

#
# === endian ===
#

AC_DEFUN(HOST_BYTEORDER, [
  AC_CACHE_CHECK([system endian], ac_cv_host_endian, [
    AC_TRY_RUN([
#include <stdio.h>
#include <string.h>
int
main() {
 char data[16] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
 int *p;
 FILE *f;
 char *msg;
 p = (int *)data;
 if (*p == 0x01020304) {
  msg = "BIG_ENDIAN\n";
 } else if (*p == 0x04030201) {
  msg = "LITTLE_ENDIAN\n";
 } else if ((*p & 0xffffffff) == 0x05060708) {
  msg = "LITTLE_ENDIAN_64\n";
 } else if ((*p & 0xffffffff) == 0x04030201) {
  msg = "LITTLE_ENDIAN_64\n";
 } else {
  msg = "UNKNOWN_ENDIAN\n";
 }
 f = fopen("endian.tmp", "wt");
 fwrite(msg, 1, strlen(msg), f);
 fclose(f);
 return *p;
}
    ], [:], [:], [:])
    ac_cv_host_endian=`cat endian.tmp`
    rm -f endian.tmp
  ])
  AX_HOST_BYTEORDER=$ac_cv_host_endian
  AC_SUBST(AX_HOST_BYTEORDER)
])
