=begin
= NuSDaS Element Table

$Date: 2003/03/07 08:42:43 $

TOYODA Eizi ((<URL:mailto:toyoda@npd.kishou.go.jp>))
Numerical Prediction Division, Japan Meteorological Agency

This document describes predefined element table in NuSDaS.

* Symbol ? in table indicates "undefined" or "unknown to the author".
* GRIB1 field shows corresponding GRIB (edition 1) parameter indicator.
  This can be used in conversion ((*to*)) GRIB 1.
  Asterisk '*' before the code indicates that the NuSDaS element name is
  NOT used in case of conversion ((*from*)) GRIB 1, 
  generally because the units is different from that in GRIB.
* GRIB parameter indicator more than 127 is local assignment of Japan
  Meteorological Agency.
* When you are handling data integrated over time
  (i.e. the time attribute of type2 is (({AV}))),
  multiply (({s})) to the units in this table.
* Element names beginning with underline character '(({_}))'
  are used when the time attribute of type2 is (({SV})).
  They indicates that the element is time average (or integration).
  In case of time-integrated data,
  you have also to multiply (({s})) to the units in this table.

=end
=begin RT
delimiter = ;

NuSDaS;	units;		GRIB1;	description
P;	hPa;		*1;	Pressure
Pres;	Pa;		1;	||
PAI;	-;		*1;	Log pressure
PSEA;	hPa;		*2;	Pressure reduced to MSL
Pmsl;	Pa;		2;	||
Ptend;	Pa.s-1;		3;	Pressure tendency
pVOR;	K.m2.kg-1.s-1;	4;	Potential vorticity
sarH;	m;		5;	ICAO Standard Atmosphere reference height
PHI;	m2.s-2;		*6;	Geopotential
gpH;	m2.s-2;		6;	||
Z;	m;		*7;	Geopotential height
gpH;	gpm;		7;	||
gmH;	m;		8;	Geometrical height
sdH;	m;		9;	Standard deviation of height
tOZON;	Dobson;		10;	Total ozone
T;	K;		11;	Temperature
vT;	K;		12;	Virtual temperature
pT;	K;		13;	Potential temperature
papT;	K;		14;	Pseudo-adiabatic potential temperature
maxT;	K;		15;	Maximum temperature
minT;	K;		16;	Minimum temperature
dT;	K;		17;	Dew-point temperature
TTD;	K;		18;	Dew-point depression (or deficit)
TRate;	K.m-1;		19;	Lapse rate
VIS;	m;		20;	Visibility
Radr1;	-;		21;	Radar spectra (1)
Radr2;	-;		22;	Radar spectra (2)
Radr3;	-;		23;	Radar spectra (3)
PLI50;	K;		24;	Parcel lifted index (to 500 hPa)
Tano;	K;		25;	Temperature anomaly
Pano;	Pa;		26;	Pressure anomaly
gpHan;	gpm;		27;	Geopotential height anomaly
Wave1;	-;		28;	Wave spectra (1)
Wave2;	-;		29;	Wave spectra (2)
Wave3;	-;		30;	Wave spectra (3)
WindD;	Degree_true;	31;	Wind direction
WindS;	m.s-1;		32;	Wind speed
U;	m.s-1;		33;	u-component of wind [X direction]
WindX;	m.s-1;		*33;	||
UU;	m.s-1;		33;	u-component of wind [eastward]
V;	m.s-1;		34;	v-component of wind [Y direction]
WindY;	m.s-1;		*34;	||
VV;	m.s-1;		34;	v-component of wind [northward]
PSI;	m2.s-1;		35;	Stream function
CHI;	m2.s-1;		36;	Velocity potential
mPSI;	m2.s-2;		37;	Montgomery stream function
sVV;	s-1;		38;	Sigma coordinate vertical velocity
OMG;	hPa.h-1;	*39;	Vertical velocity
VVPa;	Pa.s-1;		39;	||
VVm;	m.s-1;		40;	Vertical velocity
aVOR;	s-1;		41;	Absolute vorticity
aDIV;	s-1;		42;	Absolute divergence
VOR;	10-6.s-1;	*43;	Relative vorticity
rVOR;	s-1;		43;	||
DIV;	10-6.s-1;	*44;	Relative divergence
rDIV;	s-1;		44;	||
vUS;	s-1;		45;	Vertical u-component shear
vVS;	s-1;		46;	Vertical v-component shear
CrntD;	Degree_true;	47;	Direction of current
CrntS;	m.s-1;		48;	Speed of current
CrntU;	m.s-1;		49;	u-component of current
CrntV;	m.s-1;		50;	v-component of current
Q;	kg.kg-1;	51;	Specific humidity
RH;	%;		52;	Relative humidity
HMR;	kg.kg-1;	53;	Humidity mixing ratio
TPW;	kg.m-2;		54;	Precipitable water
VP;	Pa;		55;	Vapour pressure
VPVPD;	Pa;		56;	Saturation deficit
Evap;	kg.m-2;		57;	Evaporation
CIC;	kg.m-2;		58;	Cloud ice
RRate;	kg.m-2.s-1;	59;	Precipitation rate
ThndP;	%;		60;	Thunderstorm probability
RAIN;	kg.m-2;		61;	Total precipitation
RR10;	0.1 mm.min-1;	*61;	||
RR60;	mm.h-1;		*61;	||
RR3H;	8 mm.day-1;	*61;	||
RR6H;	4 mm.day-1;	*61;	||
RR1D;	mm.day-1;	*61;	||
RR1M;	mm.mon-1;	*61;	||
RRfr0;	mm;		*61;	Precipitation from last hour 00min
RRL;	kg.m-2;		62;	Large scale precipitation
RRLpD;	mm.day-1;	*62;	||
RRC;	kg.m-2;		63;	Convective precipitation
RRCpD;	mm.day-1;	*63;	||
SnRWe;	kg.m-2.s-1;	64;	Snawfall rate water equivalent
SnWe;	kg.m-2;		65;	Water equivalent of accumulated snow depth
SnowD;	m;		66;	Snow depth
MLD;	m;		67;	Mixed layer depth
tTcD;	m;		68;	Transient thermocline depth
mTcD;	m;		69;	Main thermocline depth
mTcan;	m;		70;	Main thermocline anomaly
CLA;	%;		71;	Total cloud cover
CLC;	%;		72;	Convective cloud cover
CLL;	%;		73;	Low cloud cover
CLM;	%;		74;	Medium cloud cover
CLH;	%;		75;	High cloud cover
CWC;	kg.m-2;		76;	Cloud water
TCWC;	kg.m-2;		*76;	||
BLI50;	K;		77;	Best lifted index (to 500 hPa)
SnC;	kg.m-2;		78;	Convective snow
SnL;	kg.m-2;		79;	Large scale snow
WatrT;	K;		80;	Water temperature
SST;	K;		*80;	Water temperature [sea surface]
Land;	Proportion;	81;	Land cover (1 = land, 0 = sea)
Sldev;	m;		82;	Deviation of sea level from mean
Z0;	m;		83;	Surface roughness
Albed;	%;		84;	Albedo
SoilT;	K;		85;	Soil temperature
SoilW;	kg.m-2;		86;	Soil moisture content
Veget;	%;		87;	Vegetation
Sali;	kg.kg-1;	88;	Salinity
Dens;	kg.m-3;		89;	Density
Runof;	kg.m-2;		90;	Water run-off
ROF;	mm.day-1;	*90;	||
ROFS;	||;		*90;	Water run-off [surface]
ROFD;	||;		*90;	Water run-off [gravity drag]
IceC;	Proportion;	91;	Ice cover (1 = ice, 0 = no ice)
ICE;	||;		||;	||
IceD;	m;		92;	Ice thickness
IceMD;	Degree true;	93;	Direction of ice drift
IceMS;	m.s-1;		94;	Speed of ice drift
IceMU;	m.s-1;		95;	u-component of ice drift
IceMV;	m.s-1;		96;	v-component of ice drift
IceGR;	m.s-1;		97;	Ice growth rate
IceDV;	s-1;		98;	Ice divergence
SNMlt;	kg.m-2;		99;	Snow melt
CWSSH;	m;		100;	Significant height of combined wind waves and swell
WWvD;	Degree true;	101;	Direction of wind waves
WWvSH;	m;		102;	Significant height of wind waves
WWvMP;	s;		103;	Mean period of wind waves
SwvD;	Degree true;	104;	Direction of swell waves
SwvSH;	m;		105;	Significant height of swell waves
SwvMP;	s;		106;	Mean period of swell waves
PWvD;	Degree ture;	107;	Primary wave direction
PWvMP;	s;		108;	Primary wave mean period
2WvD;	Degree ture;	109;	Secondary wave direction
2WvMP;	s;		110;	Secondary wave mean period
RSNB;	W.m-2;		111;	Net short-wave radiation flux (surface)
RLNB;	W.m-2;		112;	Net long-wave radiation flux (surface)
RSNT;	W.m-2;		113;	Net short-wave radiation flux (top of atmosphere)
RLNT;	W.m-2;		114;	Net long-wave radiation flux (top of atmosphere)
RL;	W.m-2;		115;	Long-wave radiation flux
RLUB;	||;		*115;	ibid. [upward, surface]
RLDB;	||;		*115;	ibid. [downward, surface]
RLUT;	||;		*115;	ibid. [upward, top of atm]
RLDT;	||;		*115;	ibid. [downward, top of atm]
RLUTc;	||;		*115;	ibid. [upward, top of atm, clear sky]
RLDBc;	||;		*115;	ibid. [downward, surface, clear sky]
RS;	W.m-2;		116;	Short-wave radiation flux
RSUB;	||;		*116;	ibid. [upward, surface]
RSDB;	||;		*116;	ibid. [downward, surface]
RSUT;	||;		*116;	ibid. [upward, top of atm]
RSDT;	||;		*116;	ibid. [downward, top of atm]
RSUBc;	||;		*116;	ibid. [upward, surface, clear sky]
RSDBc;	||;		*116;	ibid. [downward, surface, clear sky]
RSUTc;	||;		*116;	ibid. [upward, top of atm, clear sky]
RSDSn;	||;		*116;	ibid. [downward, in accumulated snow]
GlRad;	W.m-2;		117;	Global radiation flux
BrT;	K;		118;	Brightness temperature
WNRad;	W.m-1.sr-1;	119;	Radiance (with respect to wave number)
WLRad;	W.m-3.sr-1;	120;	Radiance (with respect to wave length)
FLLH;	W.m-2;		121;	Latent heat flux
FLSH;	W.m-2;		122;	Sensible heat flux
BLDsp;	W.m-2;		123;	Boundary layer dissipation
FLMU;	N.m-2;		124;	Momentum flux, u-component
FLMV;	N.m-2;		125;	Momentum flux, v-component
WMixE;	J;		126;	Wind mixing energy
Image;	Brightness Level; 127;	Image data
WatrT;	K;		128;	Water Temperature
CLC2;	%;		129;	Cloud cover
AvTBB;	K;		130;	Averaged blackbody temperature
MnTBB;	K;		131;	Minimum blackbody temperature
SDTBB;	K;		132;	Standard deviation of blackbody temperature
SNCov;	%;		133;	Snow cover
Tsun;	J.m-2;		134;	Global solar irradiation
HZanP;	-;		140;	Probability of high geopotential height anomaly 
PSprd;	-;		141;	Ensemble spread of pressure
ZSprd;	-;		142;	Ensemble spread of geopotential height
TSprd;	-;		143;	Ensemble spread of temperature
EAvSLP;	Pa;		200;	Ensemble mean sea-level pressue
EAvZ;	gpm;		201;	Ensemble mean geopotential height
EAvT;	K;		202;	Ensemble mean temperature
EAvU;	m.s-1;		203;	Ensemble mean u-component of wind
EAvV;	m.s-1;		204;	Ensemble mean v-component of wind
ESDSLP;	Pa;		210;	Ensemble standard deviation of pressure
ESDZ;	gpm;		211;	Ensemble standard deviation of geopotential height
ESDT;	K;		212;	Ensemble standard deviation of temperature
ESDU;	m.s-1;		213;	Ensemble standard deviation of u-component of wind
ESDV;	m.s-1;		214;	Ensemble standard deviation of v-component of wind
FGSU;	N.m-2;		;	x-component of momentum flux due to short gravity wave
FGSV;	N.m-2;		;	y-component of momentum flux due to short gravity wave
FGLU;	N.m-2;		;	x-component of momentum flux due to long gravity wave
FGLV;	N.m-2;		;	y-component of momentum flux due to long gravity wave
LTRS;	W.m-2.s-1;	;	Evaporation	
LINT;	W.m-2.s-1;	;	Interception (?)
MSC;	m;		;	Moisture in canopy
MSG;	m;		;	Moisture in ground or grass
TSC;	K;		;	Temperature in canopy
TSG;	K;		;	Temperature in ground or grass
ISC;	m;		;	Ice or frost on canopy
ISG;	m;		;	Frost on grass
SoilI;	?;		;	Soil ice content
SoilQ;	?;		;	Converged heat in soil
TSN;	K;		;	Temperature of accumulated snow surface
SnTmp;	K;		;	Temperature in accumulated snow
SnQ;	?;		;	Converged heat in accumulated snow
SnW;	?;		;	Moisture in accumulated snow
SnDen;	?;		;	Density  of accumulated snow
SnFr;	Proportion;	*133;	Snow cover
KIND;	-;		;	Land surface status code
U1;	m.s-1;		*33;	x-component of wind at the lowest model level
V1;	m.s-1;		*34;	y-component of wind at the lowest model level
T1;	K;		*11;	Temperature at the lowest model level
Q1;	kg.kg-1;	*51;	Specific humidity the lowest model level
WET;	-;		;	Wetness
UWV;	kg.m-1.s-1;	;	Water vapor flux, x-component
VWV;	kg.m-1.s-1;	;	Water vapor flux, y-component
RCST;	?;		;	Radiation forcing (short wave, top of atm)
RCSB;	||;		;	Radiation forcing (short wave, surfce)
RCLT;	||;		;	Radiation forcing (long wave, top of atm)
RCLB;	||;		;	Radiation forcing (long wave, surfce)
PBLH;	m;		;	Height of planetary boundary layer
CVR;	Proportion;	*71;	Cloud cover
HRRS;	K.day-1;	;	Heating rate due to short-wave radiation
HRRL;	||;		;	Heating rate due to long-wave radiation
HRCV;	||;		;	Heating rate due to convection
HRLC;	||;		;	Heating rate due to large-scale condensation
HRVD;	||;		;	Heating rate due to vertical diffusion
HRAD;	||;		;	Heating rate due to adiabatic motion
QRCV;	kg.kg-1.day-1;	;	Moistening rate due to convection
QRLC;	||;		;	Moistening rate due to large-scale condensation
QRVD;	||;		;	Moistening rate due to vertical diffusion
QRAD;	||;		;	Moistening rate due to adiabatic motion
URCV;	m.s-1.day-1;	;	u-component acceleration due to convection
URLC;	||;		;	u-component acceleration due to large-scale condensation
URVD;	||;		;	u-component acceleration due to vertical diffusion
URAD;	||;		;	u-component acceleration due to adiabatic motion
VRCV;	m.s-1.day-1;	;	v-component acceleration due to convection
VRLC;	||;		;	v-component acceleration due to large-scale condensation
VRVD;	||;		;	v-component acceleration due to vertical diffusion
VRAD;	||;		;	v-component acceleration due to adiabatic motion
UMF;	kg.m-2.s-1;	;	Upward mass flux
UMB;	||;		;	Upward mass flux at bottom of cloud
CWF;	J.kg-1;		;	Cloud work function
MXWIN;	m;		;	Height of maximum wind speed
TROP1;	||;		;	Height of first tropopause
TROP2;	||;		;	Height of second tropopause
CBTOP;	||;		;	Height of top of cumulonimbus
NUM;	-;		;	Station ID number
LAT;	degree;		;	Latitude
MLAT;	degree;		;	||
LON;	degree;		;	Longitude
MLON;	degree;		;	||
HIGH;	m;		;	Height
AQC;	-;		;	AQC code in AMeDAS
Sunsh;	s;		;	Duration of sunshine
SSfr0;	min;		;	Duration of sunshine
SEC;	s;		;	Time
CSEC;	||;		;	||
TDDKK;	-;		;	Thunder multiplicity and discharge code
TEC;	kA;		;	Thunder current
SM;	-;		;	Map factor
PI10LV;	-;		;	Radar 10 minutes precipitation index
RR60LV;	-;		;	Radar 60 minutes precipitation index
HIGHLV;	-;		;	Radar echo top height index
=end
