require "nusdas/nusfile"
require "nusdas/nuspath"
require 'nwpl/gridsystem'
require "find"

=begin
= class Nusdas::NusDef ƓNX
=end

module Nusdas

class NusDef

    KEYWORDS = %w(nusdas path filename creator type1 type2 type3 member
	memberlist basetime validtime validtime1 validtime2 plane
	plane1 element elementmap size basepoint distance standard
	others value packing missing information subcntl forcedlen)
    IS_KEYWORD = Hash::new
    KEYWORDS.each { |k|
	IS_KEYWORD[k] = true
    }

    PATH_TEMPLATE = {
	 'nwp_path_s' => '/_3d_name',
	 'nwp_path_vm' => '/_3d_name',
	 'nwp_path_m' => '/_3d_name/_member',
	 'nwp_path_bs' => '/_3d_name/_basetime',
    }

    class SyntaxError < StandardError; end
    class TodoError < StandardError; end
    class IncompletePathError < StandardError; end

=begin
== class Nusdas::NusDef::ElementMap
=end

    class ElementMap

=begin
=== NX\bh
--- self::new stmt, npl, nvt
    elementmap  ((|stmt|)), ʂ̐ ((|npl|)), Ώێ̐ ((|nvt|))
    ^č\zB
=end

	def initialize stmt, npl, nvt
	    stmt.shift
	    @elname = stmt.shift
	    @elclass = stmt.shift.to_i
	    case @elclass
	    when 0
		@elmap = nil
	    when 1
		@elmap = stmt
	    when 2
		@elmap = Array::new(nvt)
		ip = 0
		until (count = stmt.shift).nil? do
		     sliced = stmt.slice!(0, npl)
		     count.to_i.times do
		         @elmap[ip] = sliced
			 ip += 1
		     end
		end
	    end
	end

=begin
=== CX^X\bh
--- to_s
--- elname
    vfԂ
=end

	def to_s; @elname; end
	def elname; @elname; end

=begin
--- allowed? (ip, iv = nil, im = nil)
    iqԍ (ip, iv, im)@̊iq邩ۂԂB
=end

	def allowed? (ip, iv = nil, im = nil)
	    case @elclass
	    when 0
		true
	    when 1
		@elmap[ip] == "1"
	    when 2
		if (iv.nil?) then
		    @elmap.each do |ivslice|
			next if ivslice.nil?
			return true if ivslice[ip] == "1"
		    end
		    return false
		else
		    @elmap[iv][ip] == "1"
		end
	    else
		fail("unimplemented elementmap class = #{@elclass}")
	    end
	end

    end

=begin
== class Nusdas::NusDef

=== NX\bh

--- self::new filename, rootdir
    t@C ((|filename|)) ƃiX_X[g ((|rootdir|))
    w肵č\zB
=end

    def parse_vt1 inout, unit
	return nil unless (/^IN|OUT$/i =~ inout)
	@vt_infile = (/^IN$/i =~ inout)
	return nil unless (/^MIN|HOUR|DAY|PEN|MON|WEEK|JUN$/i =~ unit)
	@vt_unit = unit
    end
    private :parse_vt1

    def decode stmt
	case stmt.first
	when "type1"
	    @type1 = stmt[1, 3].pack('A4A2A2')
	when "type2"
	    @type2 = stmt[1, 2].pack('A2A2')
	when "type3"
	    @type3 = stmt[1, 1].pack('A4')
	when "validtime"
	    @vt_size = Integer(stmt[1])
	    parse_vt1(stmt[2], stmt[3]) || parse_vt1(stmt[3], stmt[2]) ||
		raise(SyntaxError, "<#{stmt.join(' ')}>")
        when "validtime1"
	    @vt1_style = stmt[1].upcase
	    if (@vt1_style == 'ARITHMETIC') then
		@vt1 = Array::new;
		0.upto(@vt_size.to_i - 1) do |iv|
		    @vt1.push(Integer(stmt[2]) + Integer(stmt[3]) * iv)
		end
	    elsif (@vt1_style == 'ALL_LIST') then
		fail(SyntaxError, "validtime1 before validtime") \
		    unless @vt_size
                @vt1 = stmt[2, @vt_size].collect{|x| x.to_i}
	    else
		fail SyntaxError, 'after validtime1'
	    end
        when "validtime2"
            @vt2 = stmt
	when "plane"
	    @pl_size = Integer(stmt[1])
	when "plane1"
	    fail(SyntaxError, "plane1 before plane") \
		unless @pl_size
	    @pl1 = stmt[1, @pl_size]
	when "plane2"
	    fail(SyntaxError, "plane2 before plane") \
		unless @pl_size
	    @pl2 = stmt[1, @pl_size]
	when "element"
	    @el_size = Integer(stmt[1])
	when "elementmap"
	    @el.push(ElementMap::new(stmt, @pl_size, @vt_size))
	when "path"
	    if stmt[1] == 'relative_path'
		@path = stmt[2]
            else
		@path = PATH_TEMPLATE[stmt[1]]
		@filename = '_validtime'
	    end
	when "filename"
	    @filename = stmt[1]
	when "member"
	    @me_size = stmt[1].to_i
	    @me_infile = /in/i.match stmt[2]
        when "size"
            @size = [stmt[1].to_i, stmt[2].to_i]
        when "basepoint"
            @basegrid = [stmt[1].to_f, stmt[2].to_f]
            @basepoint = NWPL::LonLat::new(stmt[3], stmt[4])
	when "distance"
	    @distance = [stmt[1].to_f, stmt[2].to_f]
	when "packing"
	    @packing = stmt[1].strip
	when "standard"
	    @standard = [NWPL::LonLat::new(stmt[1], stmt[2]),
		NWPL::LonLat::new(stmt[3], stmt[4])]
	when "others"
	    @others = [NWPL::LonLat::new(stmt[1], stmt[2]),
		NWPL::LonLat::new(stmt[3], stmt[4])]
	when "memberlist"
	    @member = stmt[1, @me_size]
	else
	    STDERR.print "NusDef: #{stmt.join(' ')} not supported\n"
	end
    end
    private :decode

    def initialize filename, rootdir
	@deffile = filename
	@rootdir = rootdir
	@type1 = @type2 = @type3 = nil
	@vt_size = @vt_infile = @vt_unit = @vt1_style = @vt1 = @vt2 = nil
	@pl_size = @pl1 = @pl2 = @el_size = @me_size = @me_infile = nil
	@member = ["none"]
	@path = '/_model/_attribute/_space/_time/_name'
	@filename = '_basetime'
        @size = @basegrid = @basepoint = nil
	@standard = @others = []
        @packing = "2PAC"
	@el = Array::new
	fail Errno::ENOENT, "#{filename} not a file" \
            unless File::file? filename
        input = open(filename, "r")
	a = Array::new
	begin
	    input.each_line do |line|
		w = line.gsub(/#.*/, '').split
		next if w.empty?
		if IS_KEYWORD[w.first.downcase] then
		    a.push w
		elsif (a.size > 0) then
		    a[a.size - 1].concat w
		else
		    a.push w
		end
	    end
	ensure
	    input.close
	end
	a.each do |stmt|
	    decode stmt
	end
    end

=begin
=== 
--- vt_unit
    Ώێ̒P
--- vt_infile
    Ώێɂăt@C邩ۂ
--- vt_size
    Ώێ̐
--- vt1
    Ώێ1 ̔z
--- vt2
    Ώێ2 ̔zA` nil
--- pl_size
    ʂ̐
--- pl1
    1 ̔z
--- pl2
    2 ̔zA` nil
--- el
    Gg}bv̔z
--- el_size
    vf̐
--- nustype
    ʕ
--- deffile
    `t@C̃pX
--- rootdir
    iX_X[gfBNg̃pX
--- path
    `t@C path Ō܂ĂApXev[g
--- filename
    `t@C filename Ō܂At@Cev[g
--- basegrid
--- basepoint
--- size
--- packing
--- distance
--- standard
--- others
=end

    attr_reader(:type1, :type2, :type3)
    attr_reader(:vt_unit, :vt_infile, :vt_size, :vt1, :vt2)
    attr_reader(:pl_size, :pl1, :pl2, :el, :el_size)
    def nustype; sprintf('%8s.%4s.%4s', @type1, @type2, @type3); end
    attr_reader(:deffile, :rootdir, :path, :filename)
    attr_reader(:basegrid, :basepoint, :size, :packing, :distance)
    attr_reader(:standard, :others)

=begin
=== CX^X\bh
--- elname
    vf̔zԂ
=end

    def elname
	@el.collect {|e| e.elname }
    end

=begin
--- el_allowed? (ie, ip, iv = nil, im = nil)
    vf ((|ie|)), ʂ̏ ((|ip|)) ȂтɕKvȂ
    Ώێ̏ ((|iv|)), ȍ ((|im|))
    w肵āAGg}bvŋeĂ邩ۂԂB
=end

    def el_allowed? (ie, ip, iv = nil, im = nil)
	@el[ie].allowed?(ip, iv, im)
    end

=begin
--- nusfilename (basetime, member = nil, validtime = nil, strip = nil)
    ((|basetime|)), ((|member|)), ((|validtime|)) Ō܂t@CԂB
     nil ^ƁAsł邱ƂwB
    t@CsƂȂꍇAftHgł IncompletePathError ƂȂB
    ((|strip|))  true łꍇ͕s̃t@C
    ; ̓fBNgT̋N_ƂĎgB
    ((|strip|))  false łꍇ͐K\ƂȂB
=end

    def nusfilename (basetime, member = nil, validtime = nil, strip = nil) 
	fnam = [@rootdir, @path, @filename].join('/')
	fnam = Regexp.quote(fnam) if strip == false
	fnam.sub!(/_model/i, @type1[0, 4])
	fnam.sub!(/_2d/i, @type1[4, 2])
	fnam.sub!(/_3d/i, @type1[6, 2])
	fnam.sub!(/_space/i, @type1[4, 4])
	fnam.sub!(/_attribute/i, @type2[0, 2])
	fnam.sub!(/_time/i, @type2[2, 2])
	fnam.sub!(/_name/i, @type3)
	if (basetime.nil?) then
	    fnam.sub!(/_basetime/i, '\d{12}')
	else
	    fnam.sub!(/_basetime/i, basetime.narrow)
	end
	if (member.nil?) then
	    fnam.sub!(/_member.*/i, '\w{4}')
	else
	    fnam.sub!(/_member/i, member.to_s)
	end
	if (validtime.nil?) then
	    fnam.sub!(/_validtime.*/i, '\d{12}')
	else
	    fnam.sub!(/_validtime/i, validtime.narrow)
	end
	fnam.gsub!(/\s/, '_')
	if strip != false then
	    segms = fnam.split(/\//)
	    while /\\(d\{12\}|w\{4\})/.match(segms.last)
		segms.pop
		raise IncompletePathError if strip.nil?
	    end
	    segms.join('/')
	else
	    fnam
	end
    end

=begin
--- to_s
    Ƃ肠A`t@C̃pXԂ܂ǁA
    ܂肠ĂɂȂłB
=end

    def to_s; @deffile; end

=begin
--- validlist (basetime)
    w肳ꂽ ((|basetime|)) ɑ΂đΏێ (\񎞊Ԃł͂Ȃ)
    ̃XgԂB
=end

    def validlist(basetime)
	tu = NWPL::NwpTimeSpan::new(1, vt_unit)
	if vt2.nil? then
	    vt1.collect { |f| basetime + tu * f }
	else
	    (0 ... vt1.size).collect { |i|
		a = basetime + tu * vt1[i]
		b = basetime + tu * vt2[i]
		"#{a},#{b}"
	    }
	end
    end

=begin
--- planelist
    ʂ̃XgԂB2ꍇ̓R}؁B
=end

    def planelist
	if pl2.nil? then
	    pl1
	else
	    (0 ... pl1.size).collect { |i| "#{pl1[i]},#{pl2[i]}" }
	end
    end

=begin
--- pvmindex (plane, validtime, member, basetime)
    ʔԍ ip, Ώێԍ iv, oԍ im ߂ĔzɂĕԂB
=end

    def pvmindex(p, v, m, b)
	[planelist.index(p), validlist(b).index(v), @member.index(m)]
    end

=begin
--- elemlist(ip, iv, im)
    ʔԍ ip, Ώێԍ iv, oԍ im ɂ
    vf̃XgԂB
--- elemlist(nuspath)
    NusPath Ŏw肳ꂽ member, validtime, plane ɂ
    vf̃XgԂB
=end

    def elemlist (ip, iv = nil, im = nil)
	if ip.is_a? NusPath
	    return elemlist(*pvmindex(ip.plane, ip.validtime, ip.member, \
		ip.basetime))
	end
	ans = []
	@el.each {|e|
	    ans.push e.elname if e.allowed?(ip, iv, im)
	}
	return ans
    end

=begin
--- find_rec (nuspath)
=end

    def find_rec (np)
        fp = NusFile::open(nusfilename(np.basetime, np.member, np.validtime))
	r = fp.read_record(np.member, np.validtime, np.plane, np.element)
	fp.close
	return r
    end

=begin
--- each_file { |filename| ... }
    `t@Cwt@C ((|filename|))
    (iX_X[gn܂tpX)
    ЂƂЂƂT yield B
=end

    def each_file
	pattern = Regexp::new(nusfilename(nil, nil, nil, false))
        Find::find(nusfilename(nil, nil, nil, true)) { |fnam|
	    next if not pattern.match fnam
	    next if File::directory? fnam
	    yield fnam
	}
    end

=begin
--- list_basetime (long)
    `t@Cɑ΂̃Xg
=end

    def list_basetime (long)
	ans = []
	each_file { |fnam|
	    fp = NusFile::open(fnam)
	    ans.push fp.basetime.to_s
	    fp.close
	}
	ans.sort.uniq
    end

=begin
--- list_schema (nuspath, long)
    ((<Nusdas::NusPath|URL:nuspath.html>)) ((|nuspath|)) 
    w肳鉽炩̎ɂĈꗗ쐬B
    ʂ String ̔złB
    `t@CQƂĈꗗ쐬邽ߍA
    ܂܂ĂȂpXꗗɊ܂܂B
    Abasetime ̈ꗗ͎̂QƂčsB
=end

    def list_schema (nuspath, long)
	return list_basetime (long) if nuspath.basetime.nil?
	return @member if nuspath.member.nil?
	# { elementmap QƂ̕Kv邩
	return validlist(nuspath.basetime) if nuspath.validtime.nil?
	# { elementmap QƂ̕Kv邩
	return planelist if nuspath.plane.nil?
	return elemlist(nuspath) if nuspath.element.nil?
	[]
    end

=begin
--- list_real (nuspath, long)
    ((<Nusdas::NusPath|URL:nuspath.html>)) ((|nuspath|)) 
    w肳鉽炩̎ɂĈꗗ쐬B
    ʂ String ̔złB
    `t@Cł͂ȂAt@CQƂĎ݂pXvf
    悤ƂB
=end

    def list_real (nuspath, long)
	return list_basetime(long) if nuspath.basetime.nil?
	ans = []
	each_file { |fnam|
	    fp = NusFile::open(fnam)
	    next if fp.basetime != nuspath.basetime
	    list = fp.list(nuspath, long)
	    ans.push(*list) if not list.empty?
	    fp.close
	}
	ans.uniq
    end

=begin
--- proj
    e@ ((<class NWPL::Projection|URL:../nwpl/gridsystem.html>)) 
    \zĕԂB
--- gridsystem
    iqn ((<class NWPL::GridSystem|URL:../nwpl/gridsystem.html>)) 
    \zĕԂB
=end

    def proj
	params = standard + others
	NWPL::Projection::new(@type1[4, 2], params)
    end

    def gridsystem
	ofs = [@basegrid[0], @basegrid[1]]
	gridsize = [@distance[0], -@distance[1]]
	NWPL::GridSystem::new(proj, @basepoint, ofs, gridsize)
    end

end
end
# $Id: nusdef.rb,v 1.21 2003/01/16 02:16:44 suuchi43 Exp $
