#!/usr/bin/ruby
# vi: set ts=8 sw=4:

require "nwpl/nwptime"
require "nusdas/nusdecode"

=begin
= nusdas/nusrecord.rb
R[hɊւNXQB
* ((<class Nusdas::NusRecord>))  ܂݂͂Ă
* ((<"class Nusdas::NusRecordNUSD">))
* ((<class Nusdas::NusRecordCNTL>))
* ((<class Nusdas::NusRecordINDX>))
* ((<class Nusdas::NusRecordDATA>))
=end

module Nusdas

class NusRecordNUSD

=begin
== class Nusdas::NusRecordNUSD
NUSD R[h̃fR[h
--- self::new string
    oCg񂩂烌R[h\
--- to_s
    fobOp\
=end

    def initialize val
	fragm = val[0..79];
	@source, = fragm.unpack('80a');
	fragm = val[80..99];
	@version, @t, @nrec, @ninfo, @nsubc = fragm.unpack("NNNNN")
    end

    def to_s
	sprintf("source '%s' ver %d Nrec %d Ninfo %d Nsubc %d", \
	    @source, @version, @nrec, @ninfo, @nsubc)
    end
    
end

class NusRecordCNTL

=begin
== class Nusdas::NusRecordCNTL
ǗR[hɊւfR[h
--- self::new string
    oCg񂩂烌R[h\
=end

    def initialize val
	@body = val
	@n_member = @body.unpack('x36N')[0]
	@n_valid = @body.unpack('x40N')[0]
	@n_plane = @body.unpack('x44N')[0]
	@n_element = @body.unpack('x48N')[0]
    end

=begin
=== E\
--- type1
--- type2
--- type3
--- nustype
    sIhȂB
--- basetime
    ((<NWPL::NwpTime|URL:../nwpl/nwptime.html>)) IuWFNgԂB
--- n_member
--- n_valid
--- n_plane
--- n_element
=end

    def type1; @body.unpack('a8')[0]; end
    def type2; @body.unpack('x8a4')[0]; end
    def type3; @body.unpack('x12a4')[0]; end
    def nustype; sprintf('%s.%s.%s', type1, type2, type3); end
    def basetime; NWPL::NwpTime::new(@body.unpack('x28N')[0]); end
    def tunit; @body.unpack('x32a4')[0]; end
    def n_member; @n_member; end
    def n_valid; @n_valid; end
    def n_plane; @n_plane; end
    def n_element; @n_element; end

=begin
--- member
    o̔z
=end

    def member
	ofs = 156
	wid = 4 * @n_member
	block = @body[ofs, wid]
	block.unpack('a4' * @n_member)
    end

=begin
--- validtime1
--- validtime2
    ((<NWPL::NwpTime|URL:../nwpl/nwptime.html>)) IuWFNg̔z񂪕ԂB
=end

    def validtime1
	ofs = 156 + 4 * @n_member
	wid = 4 * @n_valid
	block = @body[ofs, wid]
	r = []
	block.unpack('N*').each do |x|
	    r << NWPL::NwpTime.new(x)
	end
	return r
    end

    def validtime2
	ofs = 156 + 4 * (@n_member + @n_valid)
	wid = 4 * @n_valid
	block = @body[ofs, wid]
	r = []
	block.unpack('N*').each do |x|
	    r << NWPL::NwpTime.new(x)
	end
	return r
    end

    def validtime
	ofs1 = 156 + 4 * @n_member
	ofs2 = 156 + 4 * (@n_member + @n_valid)
	wid = 4 * @n_valid
	block1 = @body[ofs1, wid].unpack('N*')
	block2 = @body[ofs2, wid].unpack('N*')
	return (0 ... @n_valid).collect { |i|
	    if (block2[i] == NWPL::NwpTime::UNDEF) then
	         NWPL::NwpTime.new(block1[i])
	    else
	         [NWPL::NwpTime.new(block1[i]), NWPL::NwpTime.new(block2[i])]
	    end
	}
    end

    def plane1
	ofs = 156 + 4 * (@n_member + @n_valid * 2)
	wid = 6 * @n_plane
	block = @body[ofs, wid]
	block.unpack('a6' * @n_plane)
    end

    def plane2
	ofs = 156 + 4 * (@n_member + @n_valid * 2) + 6 * @n_plane
	wid = 6 * @n_plane
	block = @body[ofs, wid]
	block.unpack('a6' * @n_plane)
    end

    def plane
	ofs1 = 156 + 4 * (@n_member + @n_valid * 2)
	ofs2 = 156 + 4 * (@n_member + @n_valid * 2) + 6 * @n_plane
	wid = 6 * @n_plane
	block1 = @body[ofs1, wid].unpack('a6' * @n_plane)
	block2 = @body[ofs2, wid].unpack('a6' * @n_plane)
	return (0 ... @n_plane).collect { |i|
	    if (block1[i] == block2[i]) then
	         block1[i].strip
	    else
	         block1[i].strip + "," + block2[i].strip
	    end
	}
    end

    def element
	ofs = 156 + 4 * (@n_member + @n_valid * 2) + 6 * 2 * @n_plane
	wid = 6 * @n_element
	block = @body[ofs, wid]
	block.unpack('a6' * @n_element).collect{ |s| s.strip }
    end

=begin
=== WIg
--- proj
--- grid
--- basetpoint
--- baseloc
--- gridsize
--- projparam
--- repres
=end

    def proj; @body.unpack('x52a4')[0]; end
    def grid; @body.unpack('x56N2'); end
    def basepoint; @body.unpack('x64g2'); end
    def baseloc; @body.unpack('x72g2'); end
    def gridsize; @body.unpack('x80g2'); end
    def projparm; @body.unpack('x88g8'); end
    def repres; @body.unpack('x120a4'); end

=begin
=== foO
--- to_s
=end

    def to_s
	sp = ' '
	qsp = "' '"
	return (<<EOF).chop
	IDEN type '#{nustype}' basetime #{basetime}
	PROJ '#{proj}' #{projparm.join(sp)} base #{baseloc.join(sp)}
	GRID #{grid.join(sp)} size #{gridsize.join(sp)} \
base #{basepoint.join(sp)} time '#{repres}'
	MEMBER #{n_member}
	'#{member.join qsp}'
	VALID #{n_valid} '#{tunit}'
	#{validtime1.join sp}
	#{validtime2.join sp}
	PLANE #{n_plane}
	'#{plane.join qsp}'
	'#{plane2.join qsp}'
	ELEMENT #{n_element}
	'#{element.join qsp}'
EOF
    end

end

class NusRecordINDX

=begin
== class Nusdas::NusRecordINDX
INDX R[h̃fR[h
=== NX\bh
--- self::new string
=end

    def initialize val
	@val = val.unpack("N*")
    end

=begin
--- self::ofs_str (ofs)
    l ((|ofs|)) 6̏\Zi\LBA-1 ̏ꍇɌ "-1" ƂB
=end

    def self::ofs_str(ofs)
	if (ofs == 0xffffffff) then "    -1" else sprintf('%6x', ofs) end
    end

=begin
=== CX^X\bh
--- val
    l̔zԂB
=end

    def value; @val; end

=begin
--- to_s
    foO
=end

    def to_s
	r = ""
	i = 0
	@val.each do |x|
	    if ((i += 1) == 10) then
	        r = r + "\n " + self.class.ofs_str(x)
		i = 0
	    else
	        r = r + " " + self.class.ofs_str(x)
	    end
	end
	r;
    end

end

class NusRecordDATA

=begin
== class Nusdas::NusRecordDATA
DATA R[h̃fR[hB
=== NX\bh
--- self::new string
    oCg񂩂\z
=end

    def initialize val
	@member, @valid1, @valid2, @plane1, @plane2, @element, \
	    @nx, @ny, @pack, @miss \
	    = val.unpack('a4N2a6a6a6x2N2a4a4');
	@valid1 = NWPL::NwpTime::new(@valid1)
	@valid2 = NWPL::NwpTime::new(@valid2)
	@payload = val[48 .. -1]
    end

=begin
=== CX^X\bh
==== wb_
--- member
--- validtime1
--- validtime2
--- plane1
--- plane2
--- element
--- nx
--- ny
--- nustype
--- miss
=end

    attr_reader(:valid1, :valid2, :plane1, :plane2)
    attr_reader(:nx, :ny, :pack, :miss)

    def element; @element.strip; end
    def member; @member == "    " ? "none" : @member; end

    def validtime
	if @valid2.undef? then
	    @valid1
	else
	    [@valid1, @valid2]
	end
    end

    def validtime_str
	validtime.to_a.join(',')
    end

    def plane
	if @plane1 == @plane2 then
	    @plane1.strip
	else
	    @plane1.strip + "," + @plane2.strip
	end
    end

    def payload
	NusDecode::new @nx, @ny, @pack, @miss, @payload
    end

=begin
==== foO
--- to_s
=end

    def to_s
	sprintf("[%s/%s/%s/%s] %d %d %s %s", \
            member, validtime_str, plane, element, @nx, @ny, @pack, @miss)
    end
    
end


class NusRecord

=begin
== class Nusdas::NusRecord
=end

=begin
=== NX\bh
--- self::new (head, body)
    R[h̐擪 12 oCg(ׂẴR[hɋʂ) ((|head|)) 
    c ((|body|)) ^č\zB
    ((<Nusdas::NusRecStream|URL:nusrecstream.html>)) ̓sɂB
=end

    def initialize(_head, _body)
	@recl, @record_type, @contentlen, @rectime = _head.unpack('Na4NN')
	@body = _body
	@contentlen -= 12
    end

=begin
=== CX^X\bh
--- body
    \z̃oCg ((|body|)) ̂
--- recl
    L^̒łByC[h̒ + 16 ł邩ǂ
    ۏ؂̌ł͂Ȃ
--- record_type
    "NUSD", "CNTL", "INDX", "DATA" ̂ꂩ̕B
--- rectime
--- contentlen
    yC[h̒łAL^擪ɏĂl 12 Ȃ
=end

    def body; @body; end
    def recl; @recl; end
    def record_type; @record_type; end
    def rectime; NWPL::NwpTime::new(Time::at(@rectime)); end
    def contentlen; @contentlen; end

=begin
--- payload
    yC[h߂Ĉȉ̂ǂꂩ̃NX\zĕԂB
    * ((<"class Nusdas::NusRecordNUSD">))
    * ((<class Nusdas::NusRecordCNTL>))
    * ((<class Nusdas::NusRecordINDX>))
    * ((<class Nusdas::NusRecordDATA>))
--- record
    payload ̓`
=end

    def payload
	if (@record_type == 'NUSD') then
	    NusRecordNUSD.new(@body)
	elsif (@record_type == 'CNTL') then
	    NusRecordCNTL.new(@body)
	elsif (@record_type == 'INDX') then
	    NusRecordINDX.new(@body)
	elsif (@record_type == 'DATA') then
	    NusRecordDATA.new(@body)
	else
	    nil
	end
    end
    alias record payload

=begin
--- to_s
    foOpo
	sprintf("record %4s %6d [%s]", record_type, contentlen, rectime.to_s)
=end

    def to_s
	sprintf("record %4s %6d [%s]", record_type, contentlen, rectime.to_s)
    end

=begin
--- inspect
    foOpóB
    to_s ɉApayload.to_s \ȂB
=end

    def inspect
	if ((p = payload) != nil) then
	    return to_s + "\n" + p.to_s
	else
	    return to_s
	end
    end

end
end
# $Id: nusrecord.rb,v 1.13 2002/08/01 10:38:27 suuchi43 Exp $
