#!/usr/bin/ruby

=begin
= nusdir
== NAME
nusdir - list possible path segments
=end

require "nusdas"

long = false
schema = false
recursive = false

=begin
== SYNOPSIS
((*nusdir*)) [((*-R*))] [((*-l*))] [((*-s*))]
[((*-r*)) ((|rootdir|))] ((|path|))
== DESCRIPTION
((*nusdir*)) lists possible path segments after NuSDaS ((|path|)).
If ((|path|)) consisits of only data ((*type*)) component and it contains
shell metacharacters ('((***))' or '((*?*))'),
it lists possible data ((*type*))s matching to ((|path|)).
One segment is listed per one line.
== OPTIONS
:-l
  When listing data ((*type*)), path to definition file is also shown.
:-r ((|rootdir|))
  Equivalent to setting environment variable NUSDAS99 to ((|rootdir|)).
:-s
  (use Schema)
  Show a list in definition file, instead of searching datafiles.
  With this option ((*nusdir*)) runs much faster,
  but some of listed paths may be unavailable.
  By default, ((*nusdir*)) checks all datafiles, and all paths
  listed is already written.
:-R
  Recursive.
== ENVIRONMENT
:NUSDAS((|nn|)) (((|nn|)) is an integer between 00 and 99)
  NuSDaS root directory
=end

until ARGV.first.nil?
    break unless /^-/.match ARGV.first
    arg = ARGV.shift
    case arg
    when '--'
	break
    when /^-l/
	long = true
    when /^-s/
	schema = true
    when /^-r/
	ENV['NUSDAS99'] = if $'.empty? then ARGV.shift else $' end
    when /^-R/
	recursive = true
    when /^-alpha/
        alpha = true
    else
	raise "unknown option '#{arg}'"
    end
end

def listing path, long, schema, alpha
  nuspath = Nusdas::NusPath::new(path)
  if (alpha) then
    if nuspath.valid2.nil? and not nuspath.valid1.nil? then
      print("min1\n")
      next
    elsif nuspath.plane2.nil? and not nuspath.plane1.nil? then
      print(nuspath.plane1, "\n")
      next
    end
  end
  nuspath.list(long, schema)
end

def rlist path, long, schema, alpha
  list = listing(path, long, schema, alpha)
  if (list.empty?) then
    print path, "\n"
  else
    list.each { |seg|
      xpath = if (/[*?]/.match path) then seg.dup else (path + "/" + seg.to_s) end
      rlist(xpath, long, schema, alpha)
    }
  end
end

args = ['/*']
args = ARGV unless ARGV.empty?
args.each { |arg|
  if recursive then
    rlist(arg, long, schema, alpha)
  else
    print(listing(arg, long, schema, alpha).join("\n"), "\n")
  end
}

exit 0
# $Id: nusdir.rb,v 1.13 2002/08/01 10:38:27 suuchi43 Exp $
