#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include "nwplraw.h"

#define CTOI(c)		((c) - '0')

#ifdef USEMAIN
# define	M_PRINT(args)	printf args
#else
# define	M_PRINT(args)
#endif

#define DAY		1440
#define	WEEK		(7 * 1440)
#define	YEAR		(365 * 1440)
#define OLYMPIAD	((4 * 365 + 1) * 1440)
#define CENTURY_DAYS	36524
#define CENTURY		(36524 * 1440)
#define QCENTURY_DAYS	(36524 * 4 + 1)
#define QCENTURY	(QCENTURY_DAYS * 1440)
#define SEQ_01MAR1600	(((31 + 28) - 365 - 36524 * 2) * 1440)
#define SEQ_01MAR5200	(SEQ_01MAR1600 + QCENTURY * 9)
#define SEQ_01MARBC2001	(SEQ_01MAR1600 - QCENTURY * 9)

void
nwpl_seq2tm(long seq, struct tm *tp)
{
	int	era, qcentury, century, olympiad, year;
	if (seq >= SEQ_01MAR5200) {
		seq -= SEQ_01MAR5200;
		era = 5200;
	} else if (seq >= SEQ_01MAR1600) {
		seq -= SEQ_01MAR1600;
		era = 1600;
	} else {
		seq -= SEQ_01MARBC2001;
		era = -2000;
		while (seq < 0) {
			M_PRINT(("seq=%ld era=%d\n", seq, era));
			seq += QCENTURY;
			era -= 400;
		}
	}
	/* Note that QCENTURY_DAYS is multiple of 7. */
	tp->tm_wday = (seq % WEEK / DAY + 3) % 7;

	/* YEAR */
	M_PRINT(("seq=%ld era=%d wday=%d\n", seq, era, tp->tm_wday));
	qcentury = seq / QCENTURY;
	seq -= qcentury * QCENTURY;
	M_PRINT(("seq=%ld qcentury=%d\n", seq, qcentury));
	century = seq / CENTURY;
	if (century == 4) century = 3;
	seq -= century * CENTURY;
	M_PRINT(("seq=%ld century=%d\n", seq, century));
	olympiad = seq / OLYMPIAD;
	seq -= olympiad * OLYMPIAD;
	M_PRINT(("seq=%ld olympiad=%d\n", seq, olympiad));
	year = seq / YEAR;
	if (year == 4) year = 3;
	seq -= year * YEAR;
	M_PRINT(("seq=%ld year=%d\n", seq, year));
	tp->tm_year = era + qcentury * 400 + century * 100
		+ olympiad * 4 + year - 1900;

	/* MONTH-DAY */
	tp->tm_yday = seq / DAY;
	seq -= tp->tm_yday * DAY;
	tp->tm_mon = (tp->tm_yday * 10 + 5) / 306;
	M_PRINT(("yday=%d mon=%d ofs=%d\n", tp->tm_yday, tp->tm_mon,
	 ((tp->tm_mon * 306 + 5) / 10)));
	tp->tm_mday = tp->tm_yday - ((tp->tm_mon * 306 + 5) / 10) + 1;
	/* !!! January is 0, UNLIKE CHINESE/JAPANESE NAMES !!!! */
	if (tp->tm_mon >= 10) {
		tp->tm_mon -= 10;
		tp->tm_year++;
	} else {
		tp->tm_mon += 2;
	}

	/* HOUR-MIN-SEC */
	tp->tm_hour = seq / 60;
	tp->tm_min = seq % 60;
	tp->tm_sec = 0;
	tp->tm_isdst = 0;
}

long
nwpl_ymdhm2seq(long year, long mon, long day, long hour, long min)
{
	int	shift;
	long	iday;
	if (mon <= 2) {
		shift = (14 - mon) / 12;
		year -= shift;
		mon += shift * 12;
	}
	if (mon > 14) {
		shift = (mon - 3) / 12;
		year += shift;
		mon -= shift * 12;
	}
	iday = ((365 * 4 + 1) * (year + 2400)) / 4;
	iday -= (QCENTURY_DAYS * 10 + CENTURY_DAYS * 2 + 306);
	iday -= (((year + 2500) / 100) * 3 / 4 + 1);
	iday += ((mon - 3) * 306 + 5) / 10 + day;
	return (iday * 24 + hour) * 60 + min;
}

#define NUL '\0'

static int
strscan_c(const char **textp, const char **fmtp, long *lp)
{
    int	 convname, width = -1;
    if (*(*fmtp)++ == NUL) return -1;
    if (isdigit(**fmtp)) {
	width = CTOI(**fmtp);
        M_PRINT(("  widthchar=%d\n", **fmtp));
	if (*(*fmtp)++ == NUL) return -1;
    }
    convname = *(*fmtp)++;
    M_PRINT(("  strscan_c conv=%c width=%d text=<%s>\n", convname, width, *textp));
    if (convname == 'u') {
	*lp = 0;
	while (width--) {
	    M_PRINT(("   %%u w=%d val=%ld c=%c\n", width, *lp, **textp));
	    if (!isdigit(**textp)) return ((width > 0) ? -1 : 0);
	    *lp *= 10;
	    *lp += CTOI(**textp);
	    (*textp)++;
	}
	return 0;
    } else if (convname == 'w') {
        const char *test = "sun|mon|tue|wed|thu|fri|sat|";
	int	i;
	for (i = 0; i < 7; i++, test += 4) {
	    if (strncasecmp(*textp, test, 3) == 0) {
		*textp += 3;
		*lp = i;
		return 1;
	    }
	}
	return -1;
    } else if (convname == 'm') {
        const char *test = "jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|";
	int	i;
	for (i = 1; i <= 12; i++, test += 4) {
	    if (strncasecmp(*textp, test, 3) == 0) {
		*textp += 3;
		*lp = i;
		return 1;
	    }
	}
	return -1;
    } else {
	return -1;
    }
}

static int
strscan_r(const int c, const char **fmtp)
{
    int	k;
    while (1) {
	k = *(*fmtp)++;
	if (k == NUL || k == ']') return -1;
	if (k == c) break;
    }
    while (1) {
	k = *(*fmtp)++;
	if (k == NUL) return -1;
	if (k == ']') return 0;
    }
}

static int
strscan(const char *text, const char *fmt, long *lp)
{
    int	convcount = 0, r = 0;
    M_PRINT(("strscan fmt=<%s> text=<%s>\n", fmt, text));
    while (*fmt) {
        M_PRINT((" conv=%c\n", *fmt));
	switch (*fmt) {
	    case '!':
		convcount++;
		fmt++;
		break;
	    case '%':
		if (*text == NUL) { r = -1; goto last; }
		r = strscan_c(&text, &fmt, lp++);
		break;
	    case '[':
		if (*text == NUL) { r = -1; goto last; }
		r = strscan_r(*text++, &fmt);
		break;
	    default:
		if (*text == NUL) { r = -1; goto last; }
                M_PRINT((" simple '%c' '%c'\n", *fmt, *text));
		if (*text++ != *fmt++) r = -1;
		break;
	}
last:
	if (r == -1) break;
    }
    if ((r == 0) && (*fmt == NUL)) convcount++;
    M_PRINT((" convcount=%d r=%d fmt=<%s> text=<%s>\n", convcount, r, fmt, text));
    if ((r == -1) && (convcount == 0)) return -1;
    return convcount;
}

long
nwpl_str2mseq(const char *text)
{
    int	r;
    long	buf[7], y, m, d, h, mi;

    if ((r = strscan(text, "min%u", buf)) == 1) {
	return buf[0];
    }

    if (strscan(text, "%4u-%2u-%2u-%2u-%2u", buf) == 1) {
	goto YMDHM;
    }

    if (strscan(text, "%4u-%2u-%2u[-Tt ]%2u:%2u", buf) == 1)
	goto YMDHM;
    if ((r = strscan(text, "%4u-%2u-%2u![-Tt ]%2u!%2u", buf)) >= 1)
	goto YMD_optHM;
    if ((r = strscan(text, "%4u%2u%2u!%2u!%2u", buf)) >= 1) {
YMD_optHM:
	if (r < 3) buf[4] = 0;
	if (r < 2) buf[3] = 0;
YMDHM:
	y = buf[0], m = buf[1], d = buf[2], h = buf[3], mi = buf[4];
	goto Convert;
    }

    if ((r = strscan(text, "-%4u-%2u-%2u![-Tt ]%2u!%2u", buf)) >= 1) {
	if (r < 3) buf[4] = 0;
	if (r < 2) buf[3] = 0;
	y = -(long)buf[0], m = buf[1], d = buf[2], h = buf[3], mi = buf[4];
	goto Convert;
    }

    if ((r = strscan(text, "BC%4u-%2u-%2u![-Tt ]%2u!%2u", buf)) >= 1) {
	if (r < 3) buf[4] = 0;
	if (r < 2) buf[3] = 0;
	y = 1-(long)buf[0], m = buf[1], d = buf[2], h = buf[3], mi = buf[4];
	goto Convert;
    }

    /* ctime (3) format */
    if ((strscan(text, "%w %m %2u %2u:%2u:%2u UTC %4u", buf) == 1)
	|| (strscan(text, "%w %m %2u %2u:%2u:%2u JST %4u", buf) == 1)) {
	y = buf[6], m = buf[1], d = buf[2], h = buf[3], mi = buf[4];
	goto Convert;
    }

    /* GrADS format */
    if (strscan(text, "%2u%m%4u", buf) == 1) {
	y = buf[2], m = buf[1], d = buf[0], h = mi = 0;
	goto Convert;
    }
    if (strscan(text, "%2u[zZ]%2u%m%4u", buf) == 1) {
	y = buf[3], m = buf[2], d = buf[1], h = buf[0],  mi = 0;
	goto Convert;
    }
    if (strscan(text, "%2u:%2u[zZ]%2u%m%4u", buf) == 1) {
	y = buf[4], m = buf[3], d = buf[2], h = buf[1],  mi = buf[0];
	goto Convert;
    }
    return -1;

Convert:
#ifdef USEMAIN
    printf("parsed: %d %d %d %d %d\n", y, m, d, h, mi);
#endif
    return nwpl_ymdhm2seq(y, m, d, h, mi);
}

#ifdef USEMAIN
int
main(int argc, char **argv)
{
	int	argind;
	long	seq;
	struct tm	date;
	if (argc <= 2) {
		printf("usage: s <str>\n");
		return 1;
	}
	printf("mseq=%ld\n", seq = nwpl_str2mseq(argv[2]));
	nwpl_seq2tm(seq, &date);
	printf("retr=%04d-%02d-%02dT%02d:%02d (%d)\n", date.tm_year + 1900,
		date.tm_mon + 1, date.tm_mday, date.tm_hour, date.tm_min,
		date.tm_wday);
	return 0;
}
#endif
