require 'nwplraw.so'

print <<EOF
--- test 4: demonstration of all methods ---
EOF

def check(title, value, ref)
	if (value == ref)
		print "\t#{title}: #{ref}\n"
	else
		raise "#{title}: #{value} != #{ref}"
	end
end

def test(obj, method, ref)
	value = obj.send(method)
	check(method, value, ref)
end

def testc(klass, name, ref)
	value = klass.const_get(name)
	check(name, value, ref)
end

print <<EOF
* NWPL::NwpTimeSpan constants ...
EOF

testc NWPL::NwpTimeSpan, 'MIN', NWPL::NwpTimeSpan::new([1, 0])
testc NWPL::NwpTimeSpan, 'HOUR', NWPL::NwpTimeSpan::new([60, 0])
testc NWPL::NwpTimeSpan, 'DAY', NWPL::NwpTimeSpan::new([1440, 0])
testc NWPL::NwpTimeSpan, 'WEEK', week = NWPL::NwpTimeSpan::new([1440 * 7, 0])
testc NWPL::NwpTimeSpan, 'PENT', pent = NWPL::NwpTimeSpan::new([0, 1])
testc NWPL::NwpTimeSpan, 'MON', NWPL::NwpTimeSpan::new([0, 6])
testc NWPL::NwpTimeSpan, 'YEAR', NWPL::NwpTimeSpan::new([0, 72])

print <<EOF
* NWPL::NwpTimeSpan methods ...
EOF

print " * week\n"
test week, 'to_s', '1 WEEK'
test week, 'min', 1440 * 7
test week, 'mon', 0.0
test week, 'pentad', 0

print " * pentad\n"
test pent, 'to_s', '1 PENT'
test pent, 'min', 0
test pent, 'mon', 1.0/6.0
test pent, 'pentad', 1

print <<EOF
* NWPL::NwpTime constants ...
EOF

testc NWPL::NwpTime, "UNDEF", 1
testc NWPL::NwpTime, "UNIX_ERA", 88_885_440

print <<EOF
* NWPL::NwpTime methods ...
EOF

base = NWPL::NwpTime::new("2002-08-04t23:58")

test base, "to_s", "2002-08-04t2358"
test base, "narrow", "200208042358"
test base, "to_i", 106027198
test base, "undef?", false
test base, "oob?", false
test base, "year", 2002
test base, "mon", 8
test base, "mday", 4
test base, "wday", 0
test base, "yday", 156
test base, "hour", 23
test base, "min", 58
test base, "time", basetime = Time::at(1028505480)
test base, "unix", 1028505480

print <<EOF
* NWPL::NwpTimeSpan operations ...
EOF

check('== self.class', week, week)
check('+ self.class', week + pent, NWPL::NwpTimeSpan::new [10080, 1])
check('- self.class', week - pent, NWPL::NwpTimeSpan::new [10080, -1])
check('unary -', - week, NWPL::NwpTimeSpan::new -10080)
check('* Integer', week * 10, NWPL::NwpTimeSpan::new 100800)
check('/ Integer', week / 10, NWPL::NwpTimeSpan::new 1008)
check('<=> self.class', week <=> pent, 1)
check('<=> self.class', pent <=> week, -1)

print <<EOF
* NWPL::NwpTime operations ...
EOF

check('== self.class', base, base)
check('== Integer', base, 106027198)
check('== Time', base, basetime)
check('base + 1', base + 1, NWPL::NwpTime::new('2002-08-04t2359'))
check('base + 1440', base + 1440, NWPL::NwpTime::new('2002-08-05t2358'))
check('base - 10080', base - 10080, NWPL::NwpTime::new('2002-07-28t2358'))
check('base - TimeSpan 0', base - NWPL::NwpTimeSpan::new(0),
	NWPL::NwpTime::new('2002-08-04t2358'))
check('base - pent', base - pent, NWPL::NwpTime::new('2002-07-29t2358'))
check('-pent + base', -pent + base, NWPL::NwpTime::new('2002-07-29t2358'))
check('base - week', base - week, NWPL::NwpTime::new('2002-07-28t2358'))
check('base <=> base', base <=> base, 0)
check('base <=> base + week', base <=> base + week, -1)
check('base <=> base - week', base <=> base - week, 1)
