
class ElementMapEditor

  Editing = 0
  NotModified = 1
  Registered = 2

  def read_current_member
    return if @member.nil?
    read_from_member @member
  end

  def read_from_member im
    return if im.nil?
    @member = im
    if @members.size > 1
      @member_menubutton.configure('text' => @members[im])
      @members.each_index {|i|
	@othermember_menu.entryconfigure(i, 'state'=> 
					 (i == im)? 'disabled' : 'normal')
      }
    end
    sliced = @work_elem.slice_member_at(im)
    return if sliced.nil?
    (0..(@n_vt - 1)).each {|iv|
      (0..(@n_lv - 1)).each {|il|
	@bitmap[iv][il].value = sliced[iv][il]
      }
    }
    @member_menubutton.configure('state'=>'normal'
				 ) unless @member_menubutton.nil? 
    @edit_status = NotModified
    @status_label.configure('text'=>'not modified', 'background'=>'cyan'
			    ) unless @status_label.nil?
    @register_cancel_btn.configure('state' => 'disabled') unless 
      @register_cancel_btn.nil?
  end

  def register_to_member member_index
    im = (member_index.nil?) ? @member : member_index
    @work_elem.fill_member_at(@bitmap.map{ |v| v.map{ |l| l.value } }, im)
    @member_menu.entryconfigure(im, 'foreground' => 'blue') unless
      @member_menu.nil?
    @othermember_menu.entryconfigure(im, 'foreground' => 'blue') unless
      @othermember_menu.nil?
    read_from_member im
    @edit_status = Registered
    @status_label.configure('text'=>'registered', 'background' => 'green'
			    ) unless @status_label.nil?
  end
  
  def delete_element e
    return if e.nil? ||
      Tk.messageBox('icon'=>'question', 'type' => 'okcancel','parent' => @top,
		    'title' => 'Really OK?', 
		    'message' => "Really DELETE  \"#{e.elname}\" ? ") != "ok"
    @nusdef.cell("element", "elementmap").value.delete(e)
    Tk.messageBox('type' => 'ok','parent' => @top, 'title' => 'Deleted', 
		  'message' => "Element \"#{e.elname}\" deleted. \n\n" +
		  "Return to main window.. ")
    $edwin.refresh
    @top.destroy
  end

  def cancel_exit nusdef, element
    return if (edit_status == Editing) && 
      Tk.messageBox('icon'=>'warning', 'type' => 'okcancel',
		    'title' => 'Really OK?', 'parent' => @top, 
		    'message' => "Operations are NOT registered.\n" + 
		    "Really close ? ") != "ok"
    unless (@work_elem.same_as? element)
      return if Tk.messageBox('icon'=>'question', 'type' => 'okcancel',
		    'title' => 'Really OK?', 'parent' => @top, 
		    'message' => "Operations are NOT saved.\n" + 
		    "Really close ? ") != "ok"
    end
    $edwin.refresh
    @top.destroy
  end

  def save_exit nusdef, element
    newname = @newname.value
    elem_array = nusdef.cell("element", "elementmap").value
    if newname.nil? || newname.size == 0
      Tk.messageBox('icon'=>'warning', 'type' => 'ok',
		    'title' => 'input NAME', 'parent' => @top, 
		    'message' => "INPUT element NAME. ")
      return
    end
    unless (f = elem_array.find{|e| e.elname == newname }).nil?
      if f != element
	Tk.messageBox('icon'=>'error', 'type' => 'ok',
		      'title' => 'NAME error', 'parent' => @top, 
		      'message' => "NAME \"#{newname}\" already exists. ")
	return
      end
    end
    if (edit_status != Registered) &&
      Tk.messageBox('icon'=>'question', 'type' => 'okcancel',
		    'title' => 'Not registered', 'parent' => @top, 
		    'message' => "Edit data not registered.\n" +
		    "Regist to current member ?") == "ok"
      register_to_member nil
    end
    if (edit_status != Registered)
      Tk.messageBox('icon'=>'warning', 'type' => 'ok',
		    'title' => 'Not registered', 'parent' => @top, 
		    'message' => "Edit data not registered.\n" + 
		    "No data saved.")
      return
    end
    return if Tk.messageBox('icon'=>'question', 'type' => 'okcancel',
		    'title' => 'Really OK?', 'parent' => @top, 
		    'message' => "Really apply ? ") != "ok"
    return if (! element.nil?) && newname != element.elname &&
      Tk.messageBox('icon'=>'question', 'type' => 'okcancel',
		    'title' => 'Really RENAME ?', 'parent' => @top, 
		    'message' => "Element  \"#{element.elname}\"  renamed " +
		    "to \"#{newname}\".  OK ?") != "ok"
    @work_elem.elname = newname
    if element.nil?
      elem_array.push(@work_elem)
      nusdef.cell("element", "elementmap").value = elem_array
    else
      tgt_idx = elem_array.index(element)
      elem_array[tgt_idx] = @work_elem
    end
    $edwin.refresh
    @top.destroy
  end

  def initialize widget, nusdef, element = nil
    @widget = widget
    @nusdef = nusdef
    @edit_status = nil
    @member_menubutton = nil
    @status_label = nil
    @register_cancel_btn = nil
    
    @top = t = TkToplevel.new
    t.grab

    # prepare data...
    @members = members = nusdef.cell("memberlist", "member").value
    @n_dc = nusdef.cell("member", "n_dc").value.to_i
    @n_vt = nusdef.cell("validtime", "n_vt").value.to_i
    @n_lv = nusdef.cell("plane", "n_lv").value.to_i
    @work_elem = ElementMap::new(nusdef, @n_dc, @n_vt, @n_lv)
    unless element.nil?
      @work_elem.elname = element.elname
      (0..(@n_dc - 1)).each {|im|
	(0..(@n_vt - 1)).each {|iv|
	  (0..(@n_lv - 1)).each {|il|
	    @work_elem.elmap[im][iv][il] = element.elmap[im][iv][il]
	  }
	}
	@work_elem.modified[im] = element.modified[im]
      }
    end

    if nusdef.cell("validtime1", "vt_style").value.upcase == "ARITHMETIC"
      initial_step = nusdef.cell("validtime1", "initial-step").value
      initial = initial_step[0].to_i
      step = initial_step[1].to_i
      vt_names = Array::new
      (0..(@n_vt - 1)).each {|i|
	vt_names.push((initial + i * step).to_s)
      }
    else
      vt_names = nusdef.cell("validtime1", "vt_list").value
    end
    lv_names = nusdef.cell("plane1", "name").value
    @bitmap = Array::new(@n_vt)
    (0..(@n_vt - 1)).each {|iv|
      @bitmap[iv] = Array::new(@n_lv)
      (0..(@n_lv - 1)).each {|il|
	@bitmap[iv][il] = TkVariable.new(1)
      }
    }

    # Name
    f = TkFrame.new(t).pack('fill' => 'x', 'pady' => '5')
    nlf = TkFrame.new(f, 'borderwidth'=>1, 'relief'=>'solid',
		      'background' => 'DarkSeaGreen3'
		      ).pack('padx'=>10, 'pady'=>3, 'side'=> 'left')
    nf = TkFrame.new(f, 'borderwidth'=>2, 'relief'=>'ridge'
		     ).pack('padx'=>3, 'pady'=>3, 'side'=> 'left')
    unless element.nil?
      TkLabel.new(nlf, 'text' => element.elname, 
		  'borderwidth' => '1', 'relief' => 'solid'
		  ).pack('padx'  => '3', 'pady'=>'3', 'side' => 'left',
			 'ipadx' => '3', 'ipady' => '3')
      TkLabel.new(nf, 'text' => ' Rename to:').pack('side' => 'left')
    else
      TkLabel.new(nlf, 'text' => 'New Element', 
		  'borderwidth' => '1', 'relief' => 'solid'
		  ).pack('padx'  => '3', 'pady'=>'3', 'side' => 'left',
			 'ipadx' => '3', 'ipady' => '3')
      TkLabel.new(nf, 'text' => ' Name:').pack('side' => 'left')
    end
    @newname = TkVariable.new
    e = TkEntry.new(nf,'textvariable'=>@newname).pack('side'=>'left','pady'=>5)
    unless element.nil?
      @newname.value = element.elname
      c = TkButton.new(nf, 'text'=>'Cancel').pack('side'=>'left', 'padx'=>5)
      c.command proc{
	@newname.value = element.elname
      }
      b = TkButton.new(f, 'text' => 'DELETE this element'
		       ).pack('side' => 'left', 'padx' => '10')
      b.command proc{
	delete_element(element)
      }
    else
      TkLabel.new(nf, 'text' => ' ').pack('side' => 'left')
    end
        
    # Member
    if members.size > 1
      mf = TkFrame.new(f, 'borderwidth'=>2, 'relief'=>'ridge'
		       ).pack('side'=>'top', 'fill'=>'x', 'side'=>'left',
			      'padx'=>'10')
      TkLabel.new(mf, 'text' => 
		  element.nil? ? ' Member:' : ' Read initial data from Member:'
		  ).pack('side'=>'left', 'padx'=>3, 'pady'=>3)
      mbf = TkFrame.new(mf, 'borderwidth'=>1, 'relief'=>'solid',
			'background' => 'DarkSeaGreen3'
		     ).pack('padx'=>5, 'pady'=>3, 'side'=> 'left')
      @member_menubutton = mb =
	TkMenubutton.new(mbf, 'borderwidth' => 2, 'relief'=>'raised'
			 ).pack('side' => 'left', 'padx'=>3, 'pady'=>3)
      @member_menu = options = TkMenu.new(mb, 'tearoff'=> 0)
      members.each_index{|i|
	case @work_elem.modified[i]
	when nil
	  color = 'red'
	when true
	  color = 'blue'
	else
	  color = 'black'
	end
	options.add 'command', 'label' => members[i],
	  'foreground' => color, 'command' => proc{ read_from_member(i) }
      }
      mb.menu(options)
    end

    # Commands
    f = TkFrame.new(t).pack('fill' => 'x', 'pady' => '5', 'side'=>'bottom')

    st_f = TkFrame.new(f, 'borderwidth'=>2, 'relief'=>'ridge'
		       ).pack('side'=>'left', 'padx'=>'10')
    TkLabel.new(st_f, 'text'=>' Status: ').pack('side'=>'left')
    @status_label = TkLabel.new(st_f).pack('side'=>'left', 'padx'=>5)
    @register_cancel_btn = TkButton.new(st_f, 'text'=>'Cancel')
    @register_cancel_btn.command proc { read_current_member }
    @register_cancel_btn.pack('side'=>'right', 'padx'=>'10', 'pady'=>3)

    if members.size > 1
      regm_menu = TkMenubutton.new(st_f, 'text'=>' Register to member.. ',
				   'borderwidth' => 2, 'relief'=>'raised',
				  'menu' => options
				  ).pack('side'=>'left', 'padx'=>10,'pady'=>3)
      options = TkMenu.new(regm_menu, 'tearoff'=> 0)
      options.add 'command', 'label' => 'THIS member', 'command' => proc {
	if Tk.messageBox('icon'=>'question', 'type' => 'okcancel',
			 'title' => 'Really OK?', 'parent' => t, 'message' => 
			 "Really overwrite to THIS member ? ") == "ok"
	  register_to_member nil 
	end
      }
      options.add 'command', 'label' => 'ALL membermes', 'command' => proc{
	if Tk.messageBox('icon'=>'question', 'type' => 'okcancel',
			 'title' => 'Really OK?', 'parent' => t,
			 'message' => "Really overwrite ALL members ?") == "ok"
	  members.each_index {|i| register_to_member(i) }
	end
      }
      @othermember_menu = oth_mm = TkMenu.new(regm_menu, 'tearoff'=> 0)
      options.add('cascade', 'label' => 'The OTHER member..', 'menu'=>oth_mm)
      members.each_index {|i|
	oth_mm.add 'command', 'label' => members[i], 
	  'foreground' => (@work_elem.modified[i] ? 'blue' : 'black'), 
	  'command' => proc{
	  if Tk.messageBox('icon'=>'question', 'type' => 'okcancel',
			   'title' => 'Really OK?', 'parent' => t, 'message' =>
			   "Really overwrite to the member " +
			   "\"#{members[i]}\" ?") ==  "ok"
	    register_to_member(i)
	  end
	}
      }
      regm_menu.menu(options)
    else
      TkButton.new(st_f, 'text'=>' Register ', 'borderwidth'=>2,
		   'relief'=>'raised', 'command'=> proc {
		     if Tk.messageBox('icon'=>'question', 'type' => 'okcancel',
				      'title' => 'Really OK?', 'parent' => t,
				      'message'=>"Really overwrite ? ") == "ok"
		       register_to_member nil 
		     end
		   }
		   ).pack('side'=>'left', 'padx'=>10,'pady'=>3)
    end

    cl_f = TkFrame.new(f, 'borderwidth'=>2, 'relief'=>'ridge'
		       ).pack('side'=>'right', 'padx'=>'10')
    TkButton.new(cl_f, 'text'=>'Cancel & Close', 'command' => proc { 
		   cancel_exit(nusdef, element) 
		 }).pack('side'=>'right', 'padx'=>'10', 'pady'=>3)
    TkButton.new(cl_f, 'text'=>'Apply & Close', 'command' => proc {
		   save_exit(nusdef, element) 
		 }).pack('side'=>'right', 'padx'=>'10', 'pady'=>3)

    read_from_member(0)

    width  = 35 * @n_vt * 1.1
    height = 20 * @n_lv * 1.1
    width  = 600 if (width  > 600)
    height = 350 if (height > 350)


    # Level-VT plane

    f = TkFrame.new(t, 'borderwidth'=>'2', 'relief'=>'sunken')
    f.pack('side'=>'bottom', 'fill'=>'both', 'expand' => true,
	   'padx'=>5, 'pady'=>5, 'ipadx'=>5, 'ipady'=>5)

    TkGrid.rowconfigure(f, 1, 'weight' => 1, 'minsize' => 0)
    TkGrid.columnconfigure(f, 1, 'weight' => 1, 'minsize' => 0)

    # vt - title
    tf = TkFrame.new(f, 'borderwidth'=>2, 'relief'=>'sunken'
		     ).grid('row'=> 0,'column'=>1, 'sticky'=>'ew', 'pady'=> 5)
    vtc = TkCanvas.new(tf, 'height'=>'1c').pack('fill'=>'x', 'expand'=>true)
    (0..(@n_vt - 1)).each {|iv|
      vtl = TkLabel.new(vtc, 'text'=>vt_names[iv])
      TkcWindow.new(vtc,"#{iv * 1.3}c", '2c','tags'=>'window', 'window'=> vtl,
		    'width' => 35, 'anchor' => 'sw')
    }
    TkLabel.new(f, 'text'=>' (Level)').grid('row'=>2, 'column'=>0)

    # level - title
    tf = TkFrame.new(f, 'borderwidth'=>2, 'relief'=>'sunken'
		     ).grid('row'=> 1,'column'=>0, 'sticky'=>'ns', 'padx'=> 5)
    lvc = TkCanvas.new(tf, 'width' => '2c').pack('fill'=>'y', 'expand'=>true)
    (0..(@n_lv - 1)).each {|il|
      lvl = TkLabel.new(lvc, 'text'=>lv_names[il])
      TkcWindow.new(lvc, '2c', "#{@n_lv - il * 0.7}c", 'tags'=>'window',
		    'window'=> lvl, 'height' => 20, 'anchor' => 'ne')
    }
    TkLabel.new(f, 'text'=>'(vt)', 'justify'=>'left'
		).grid('row'=>0, 'column'=>2)

    # checkbuttons
    bf = TkFrame.new(f, 'borderwidth'=>2, 'relief'=>'sunken'
		     ).grid('row' => 1, 'column' => 1, 'sticky'=>'news')
    btnsc = TkCanvas.new(bf, 'width' => width, 'height' => height
			 ).pack('fill'=>'both', 'expand'=>true)
    (0..(@n_vt - 1)).each {|iv|
      (0..(@n_lv - 1)).each {|il|
	btn = TkCheckButton.new(btnsc,'variable' => @bitmap[iv][il],
				'justify' => 'center',
				'background' => 'DarkSlateGrey',
				'activebackground' => 'DarkSeaGreen4', 
				'selectcolor' => 'GreenYellow'
			  )
	btn.command proc { change }
	TkcWindow.new(btnsc, "#{iv * 1.3}c", "#{@n_lv - il * 0.7}c",
		      'tags'=>'window', 'window'=>btn,
		      'width'=>35,'height'=>20,'anchor'=>'nw')
      }
    }

    # vt_all buttons
    tf = TkFrame.new(f, 'borderwidth'=>2, 'relief'=>'sunken'
		     ).grid('row'=> 1,'column'=>3, 'sticky'=>'ns', 'padx'=> 5)
    vtac = TkCanvas.new(tf, 'width'=>'3.4c').pack('fill'=>'y', 'expand'=>true)
    (0..(@n_lv - 1)).each {|il|
      f1 = TkFrame.new(vtac)
      TkLabel.new(f1, 'text'=>'<').pack('side'=>'left')
      TkButton.new(f1, 'text'=>'on', 'command' => proc{
		     change
		     @bitmap.each{|v| v[il].value = 1 }
		   }).pack('side'=>'left', 'ipadx'=>0, 'ipady'=>0)
      TkButton.new(f1, 'text'=>'off', 'command' => proc{
		     change
		     @bitmap.each{|v| v[il].value = 0 }
		   }).pack('side'=>'left', 'ipadx'=>0, 'ipady'=>0)
      TkcWindow.new(vtac, '0c', "#{@n_lv - il*0.7}c", 'tags'=>'window',
		    'window'=>f1, 'height'=>20, 'anchor'=>'nw')
    }
    
    # level_all buttons
    tf = TkFrame.new(f, 'borderwidth'=>2, 'relief'=>'sunken'
		     ).grid('row'=> 3,'column'=>1, 'sticky'=>'ew', 'pady'=> 5)
    lvac = TkCanvas.new(tf, 'height'=>'2.5c').pack('fill'=>'x', 'expand'=>true)
    (0..(@n_vt - 1)).each {|iv|
      f1 = TkFrame.new(lvac)
      TkLabel.new(f1, 'text'=>'^').pack
      TkButton.new(f1, 'text'=>'on', 'command' => proc{
		     change
		     @bitmap[iv].each {|l| l.value = 1 }
		   }).pack('ipadx'=>0, 'ipady'=>0)
      TkButton.new(f1, 'text'=>'off', 'command' => proc{
		     change
		     @bitmap[iv].each {|l| l.value = 0 }
		   }).pack('ipadx'=>0, 'ipady'=>0)
      TkcWindow.new(lvac, "#{iv * 1.3}c", '1c', 'tags'=>'window', 
		    'window'=>f1, 'width'=>35, 'anchor'=>'nw')
    }

    # VT&level all buttons
    tf = TkFrame.new(f, 'borderwidth'=>2, 'relief'=>'sunken'
		     ).grid('row'=> 3,'column'=>3, 'sticky'=>'news',
			    'padx' => 5, 'pady'=> 5)
    TkLabel.new(tf, 'text'=>'All').pack
    TkLabel.new(tf, 'text'=>'levels & vt').pack
    tbf = TkFrame.new(tf).pack
    TkButton.new(tbf, 'text'=>'on', 'command' => proc{
		   change
		   @bitmap.each{|v| v.each {|l| l.value = 1 }}
		 }).pack('side'=>'left')
    TkButton.new(tbf, 'text'=>'off', 'command' => proc{
		   change
		   @bitmap.each{|v| v.each {|l| l.value = 0 }}
		 }).pack('side'=>'left')
    xscr = TkScrollbar.new(f).grid('row'=>2, 'column'=>1, 'sticky'=>'ew')
    xscr.configure('orient'=>'horiz', 'command' => proc{|*args|
		     [vtc, btnsc, lvac].each {|w| w.xview *args} })
    yscr = TkScrollbar.new(f).grid('row'=>1, 'column'=>2, 'sticky'=>'ns')
    yscr.configure('command' => proc{|*args|
		     [lvc, btnsc, vtac].each {|w| w.yview *args} })
    [vtc, btnsc, lvac].each {|w| w.xscrollcommand(proc{|f, l| xscr.set f, l})}
    [lvc, btnsc, vtac].each {|w| w.yscrollcommand(proc{|f, l| yscr.set f, l})}
    [vtc, lvac].each {|w|
      w.configure('scrollregion'=>['0c','1c', "#{@n_vt * 1.3}c",'1c'])
    }
    [lvc, vtac].each {|w|
      w.configure('scrollregion'=>['0c',"#{(1.0 - 0.7)*(@n_lv-1)+1}c",
		    '0c',"#{@n_lv+1}c"])
    }
    btnsc.configure('scrollregion'=>['0c',"#{(1.0 - 0.7)*(@n_lv-1)+1}c",
		      "#{@n_vt * 1.3}c","#{@n_lv+1}c"])
    canvas_xscroll_bind([vtc, btnsc, lvac])
    canvas_yscroll_bind([lvc, btnsc, vtac])

    t.title "ElementMap <" +
      (element.nil? ? "New ELEMENT" : element.elname) + ">"

  end

  def change
    @edit_status = Editing
    @member_menubutton.configure('state'=>'disabled') unless 
      @member_menubutton.nil? 
    @status_label.configure('text'=>' Editing ', 'background' => 'yellow'
			    ) unless @status_label.nil?
    @register_cancel_btn.configure('state'=>'normal') unless 
      @register_cancel_btn.nil?
  end

  attr_reader(:edit_status)

end
