#!/usr/bin/ruby

require "NusDEF"
require "tk"
require "elemmap_ed"

$root = TkRoot.new

$ev_UP    = TkVirtualEvent.new('Button-4', 'Up',   'Prior')
$ev_DOWN  = TkVirtualEvent.new('Button-5', 'Down', 'Next' )

$ev_LEFT  = TkVirtualEvent.new('Shift-Button-4', 'Left')
$ev_RIGHT = TkVirtualEvent.new('Shift-Button-5', 'Right')

def descendants widget
  return nil if widget.nil?
  a = Array::new
  a.push widget
  TkWinfo.children(widget).each do |w|
    des = descendants(w)
    a << des unless des.nil?
  end
  a.flatten
end

def canvas_xscroll_bind widgets
  widgets.each { |widget|
    descendants(widget).each {|win|
      win.bind($ev_LEFT,  proc{widgets.each{|w| w.xview('scroll',-1,'units')}})
      win.bind($ev_RIGHT, proc{widgets.each{|w| w.xview('scroll', 1,'units')}})
    }
  }
end

def canvas_yscroll_bind widgets
  widgets.each { |widget|
    descendants(widget).each {|win|
      win.bind($ev_UP,   proc{widgets.each{|w| w.yview('scroll',-1,'units')} })
      win.bind($ev_DOWN, proc{widgets.each{|w| w.yview('scroll', 1,'units')} })
    }
  }
end

# IO methods

def new_file
  fname = Tk.getSaveFile
  return if fname == ''
  $original = nil
  $nusdef = NusDef::new
  $edwin.destroy unless $edwin.nil?
  $edwin = EditWindow::new($nusdef)
  open(fname, 'w') {}
  $root.title($current = fname)
end

def load_file filename
  open(filename, 'r') { |f| $original = f.read } unless filename.nil?
  $nusdef = NusDef::new(filename, nil)
  $edwin.destroy unless $edwin.nil?
  $edwin = EditWindow::new($nusdef)
  $root.title($current = filename)
end

$nusdef_ftypes = [
  [ 'NusDef files', '.def' ],
  [ 'All files', '*' ]
]

def open_file
  fname = Tk.getOpenFile('filetypes' => $nusdef_ftypes)
  return if fname == ''
  load_file(fname)
end

def save_file
  return if Tk.messageBox('icon'=>'question', 
			  'type' => 'okcancel',
			  'title' => 'Really OK?', 
			  'message' => "Really overwrite  \"" +
			  $current + "\" ? ") != "ok"
  open($current, 'w') {|f| f.write($nusdef.to_s) }
end

def save_as
  fname = Tk.getSaveFile('initialfile'=>$current, 'filetypes'=>$nusdef_ftypes)
  return if fname == ''
  open(fname, 'w') {|f| f.write($nusdef.to_s) }
  $root.title($current = fname)
end

def show_original
  title = "<original>"
  title = $current + title unless $current.nil?
  TxtDialog::new(title, ($original.nil?) ? "# No original file.\n" : $original)

end

def show_encode
  title = "<encoded>"
  title = $current + title unless $current.nil?
  TxtDialog::new(title, $nusdef.to_s)
end

def show_version
  str = <<-END
  NusDef Editor Ver. 0.1e
  2002 (c) NPD/JMA
  END
  Tk.messageBox('type' => 'ok', 'parent' => $root, 'message' => str)
end

def show_usage
  str = <<-END

This is a GUI editor for a NuSDaS definition file.

[File Menu]

The "File" menu has items to open a definition file, to save a current status
into the definition file, and to quit the program. In most cases, your first
action to the editor will be "Open-File". Even though the editor kindly helps
you to make a definition file, it is still tough to do from scratch. So the
authors strongly recommend to open an existing definition file and to modify
it.

[View Menu]

The "View" menu has two items: to view the original file you have opened, and
to preview output to be created from the current status. An estimated data
volume is always indicated at the bottom of the main window.

[How to edit a definition file]

The main window consists of 28 statement boxes, each of which corresponds to a
statement in the definition file. The name of the statement (hereafter LABEL)
is indicated with a rectangular frame at the left in a box. You can edit the
statement by selecting one of items on a menu button and/or putting an
appropriate word in a text field. Some of the statements are set automatically
by referring another statements. When you click a help button at the right in
the box, you will see the description of the statement.

You can scroll the window with a scrollbar at the right of the window or a
wheel on a mouse. You can also pick and drag the view with the mouse. The view
will scroll horizontally with the wheel, when you push a shift key.

- Status of the statement

The color of the LABEL indicates the status of the statement as follows:

  Color  input      consistency check   output
  ------------------------------------------------------------
  Blue   mandatory  passed              yes
  Red    mandatory  error detected      not available
  Black  optional   no check            yes
  Gray   optional   no check            no

If there are any statements showing a red LABEL, you can not save the
definition file without changing them to appropriate values. When you correct
them, the LABELs change to blue and then you can save the file. The statement
with a black or gray LABEL is not checked, thus you can change it at your
responsibility. The statement with a gray LABEL is omitted in the definition
file.

[Element map editor]

When you edit the element map, use the element map editor, that appears when
you push the 'Select ELEMENT to EDIT..' button in the 'element' box and select
an element in the pop-up menu. The color of the element in the pop-up menu
shows its edit status as follows:

  Color    status
  ------------------------------------------------------------
  Black:   Not edited yet / NO further edit expected
  Red:     Not edited yet / further edit required
  Yellow:  Some of members are not edited yet
  Blue:    All members have been edited

If you add a new member, valid time or plane to the definition file, a
tentative element map is created automatically by copying the adjacent row or
column to the new portion. In such a case, all of the elements in the pop-up
menu turn to red and become subject to check. If there are any red elements in
the pop-up menu, the LABEL of the 'element' statement is also red.

- How to edit element map

The window for the element map editor comprises three components - Header,
Body and Footer.

In the Header part, the name of the element is indicated at the left. If you
want to change the name, put one you like in the text field following 'Rename
to:'. The 'DELETE this element' button expunges this element from the
definition file. A button after ' Read initial data from Member:' indicates
the member name you are editing now. You can change the member by pushing the
button and selecting a member you like from the pop-up menu. The color of the
member in the pop-up menu shows its registration status as follows;

  Blue: registered
  Red: not registered

In the Body part, the status of the element is displayed as a cell of a two
dimensional matrix in such a way that the abscissa represents valid time and
that the ordinate corresponds to vertical level (plane). A light green cell
indicates the existence of the element, while dark green means inexistent. You
can change the status by clicking the cell individually. Use the ON/OFF
buttons at the right and/or the bottom, when you like to change the status in
the series at once. You can change entire cells in the matrix simultaneously
with the ON/OFF button at the bottom-right.

In the Footer part, you can register the current status of the element map
with the 'Register' button in the status box. If there are two members or
more, you have to select what member you like to register with a pop-up menu.
The color of the member in the pop-up menu indicates the registration status
as follows;

  Blue: registered
  Black: not registered
  Gray: member currently displayed

The edit status is indicated after 'status:'. If the status is 'editing', you
can not switch the member displayed in the Body part to an another. The
'Cancel' button in the status box recovers the matrix to the state at the last
registration.

The element map editor has its own buffer area for registration and the
modified status is not reflected to the main window until you push the 'Apply'
button, which updates the buffer area for the main window and closes the
element map editor. The 'cancel' button at the right disregards any changes
performed on the element map editor and closes it.

[The other buttons in the 'element' statement box]

- The "Accept ALL current settings" button accepts all current settings of
entire elements and the 'red' LABEL turn to 'blue'. This is useful when you
are confident no further editing for the element map is needed.

- The "Add NEW element.." button adds a new element. The element map editor is
invoked. It is necessary to define the element name at least.

- The "Change Order of elements.." button changes the order of the elements.
If there are any 'red' elements, the button is not available.

  END
  str = <<-END if ENV["LANG"] =~ /^ja/
  NuSDaS `t@CɕKvȍڂǂݍ݁^ҏW^o͂܂B


  [t@C̓́Eo]

  EBhEĩj[o[ɂuFilevj[ɂA
  NuSDaS `t@C̓ǂݍ݁^ҏW̃t@Co ł܂B
   ȂׂÃt@C𐗌^ƂčƂ邱Ƃ߂܂B
  vȌIÃj[ōsȂ܂B


  [ҏW󋵂̊mF]

  j[o[́uViewvj[ɂA(̒`t@Cǂݍ
  ꍇ)ҏWO̓e̊mF^ҏW̓eł̏o͂̊mF ł܂B
  ܂AEBhEɁAҏW̓e`t@CƂ NuSDaS f[^
  Zbgۂɍ쐬ꍇɕKvƂȂfBXNeʂ̖ڈ\\܂B


  [ҏW]

  e`ږɘg\\Aꂼ̘gɂ
  [`ڂ̖́A`ړ̊evf̓͗A`ڂ̐(HELP){^]
  \\܂B

  - `ڂ̏
  ҏW̏ԂɂāAe`ڂ̖̂̕Ɏ̕\\ɎF
  ܂B

    F         K{/ȗ\\            `FbN    o
    -------------------------------------------------------------
               ȗ\\                sȂȂ     
    DF         ȗ\\                sȂȂ     Ȃ
                K  {                 Ă     
                K  {                  s      s\\

  ȂƂ܂͐ԂɂȂĂ`ڂ̊evfɂēK؂Ȓl
  ܂͑IAԂȂĂ鍀ڂȂĉBDF̒`ڂ
  ĂAeK؂łȂ΂ȂȂ̂ŒӂĉiςȒl
  ́AeuJvɂĂ܂ǂm܂jB

  `ڂԂȂĂi͓e󂯓Ȃjꍇɂ́A͂
  `Ɍ肪ꍇ̂قA͓eق̒`ڂƖ𐶂Ă
  ꍇ܂B̏ꍇA֌Wق̒`ڂ̕ɁiƂꂪ
  Ԃłjuv邱Ƃl܂̂ŁAӂKvł
  iǂ炪uv̂́Al悤ƂĂ邠Ȃf
  ł܂B̃GfB^iɌ炸AȂȊỎjɂ́A͂ꂽ
  ɖo邱Ƃł܂̂ŁAƂ낵jB

  - evf̓́^I
  ʏ́Aڂł镔̓L[{[h́AoςĂ镔̓j[
  I܂̗͂vfp̕ҏWʂ̋NAȕ͍͑̓ڂ̓e
  IɃZbǵAƂȂĂ܂Biڂł闓ɁjL[{[h
  ͂ꂽĺA^uL[}EXł̗L[̓tH[JXړ
  邩A^[L[ƂɗLɂȂ܂i܂ӎȂĂA
  ̑鎞͕ʂɓĂ͂܂̂ŁAIɔf悤
  ͂łjB

  - XN[
  `eʂɎ܂Ȃ悤ȏʂł́Ac܂͉ɃXN[
  ł悤ɂȂĂ܂BXN[́A}EX{^Łu͂Łv
  ړقAzC[ŏ㉺AVtgL[ȂzC[
  ōE̕ɃXN[邱Ƃł܂B


  [Elementmap ̕ҏW]

  uvfvƂ̍\\(oE\\񎞊ԁEʂƂ̗L)ɂẮAƗ
  ҏWʂő삵܂B

  - vf̑Ij[
  "element"  elementmap Ij[ "Select ELEMENT to EDIT.." 
  ҏWvfI܂B̃j[ɂĂAҏW̏Ԃ
  Ċevf̂悤ɐFĂ܂B

    F         
    -------------------------------------------------------------
             ҏWAҏW͕K{ł͂Ȃ
             ҏWAҏW삪K{
    F       ꕔoɂĖҏWAҏW삪K{
             SĕҏWς

  owE\\񎞊ԂǉꍇAVɑɂ(Ȃ
  ƂmF̂߂)ҏW삪KvɂȂ܂(*)B̂߁ASĂ̗vf
  uԁvƂȂ܂Buԁv̗vfłcĂA"element" 
  ԂԂ̂܂܂ƂȂ܂B
  (*) ǉéAuOv̕т̍\\̃Rs[łB

  - evf̕ҏW
  L̃j[ҏWvfIƁAVEBhE
  (uElementMap Editorv)J܂B̃EBhEł́Aw肳ꂽ
  vfɂĂ elementmap ̕ҏWsȂ܂BҏWƂ̂߂ɁA
  elementmap WJrbg}bvC[ẄꎞIȃRs[pӂA
  ҏW͂̃Rs[ɑ΂čsȂ܂BҏWƂI ElementMap 
  Editor ۂɂ́AҏWsȂC[W(Rs[){̕
  ߂Aj邩̎wsȂ܂B
  ElementMap Editor ł́AL̑삪ł܂B

    (i)
    "Rename to: "                      vf̕ύX
    "DELETE this element"              vf폜Aʂ
    "Read initial data from Member: "  ҏW̊ƂȂ\\ǂݍ
                                       o̕\\^ύXčēǂݍ(*1)
    (i)
    (:\\񎞊 / c: ̃{^) \\̕ύXiL^ ̑j
    (e̊O on / off {^)     Y̎Ŝ on / off
    (E on / off {^)           ʑŜ on / off

    (i)
    "Register" (o̎) ܂
    "Register to member.." (o)
        \\ʂƗpC[WɔfB
        ȍꍇAf̃owł(*2)
    "Cancel"
        i\\ȏꍇj\\ʏ̑̎
    "Apply & Close"
        ƗpC[W荞ŁAʂ
    "Cancel & Close"
        ƗpC[WjāAʂ

  ɃoꍇAi " Register to member.. " p
  SẴoɂēK؂ȍ\\ݒ肷Kv܂B
  (*1) Ij[ł́Aݒς݃o^ݒ胁o ƂȂ܂
  (*2) Ij[ɂ "The OTHER member.." ł́Aݒς݃o
       ^̑̑I\\o[^݂̃o[DF ƂȂ܂


  [vfɊւȂ̕ҏW]

  - "Accept ALL current settings"
  SvfɂāA݂̍\\ŕҏWÎƂ܂B"View"
  j[ȂǂɂāAȏ̕ҏW삪svł邱ƂmFł
  ꍇɕ֗łB

  - "Add NEW element.."
  vfǉ܂Bvf̍\\ҏŴƓl̉ʂJ܂B
  vft^鑀삪K{ƂȂ܂B

  - "Change Order of elements.."
  vf̕яҏW܂Buԁv̗vfƁA삪f܂B

  END
  TxtDialog::new("Usage", str)
end

class TxtDialog

  def initialize title, str
    t = TkToplevel.new
    height = str.split("\n").size + 2
    height = 30 if height > 30
    TkButton.new(t, 'text' => 'OK', 'command' => proc{t.destroy}
		 ).pack('side'=>'bottom')
    TkFrame.new(t) {|f|
      TkGrid.rowconfigure(f, 0, 'weight' => 1, 'minsize' => 0)
      TkGrid.columnconfigure(f, 0, 'weight' => 1, 'minsize' => 0)
      txt = TkText.new(f, 'font' => 'fixed', 'wrap' => 'none',
		       'height' => height)
      xscr = TkScrollbar.new(f)
      yscr = TkScrollbar.new(f)
      Tk.grid(txt, yscr, 'sticky' => 'ewsn')
      Tk.grid(xscr, 'sticky' =>  'ewsn')
      txt.xscrollbar(xscr)
      txt.yscrollbar(yscr)
      txt.insert('end', "\n" + str)
      txt.configure('state', 'disabled')
    }.pack('expand' => 'yes', 'fill' => 'both')
    t.title title
  end

end


# generate Menu

TkFrame.new{|f|
  pack('fill' => 'x', 'side' => 'top')
  TkMenubutton.new(f, 'text' => 'File', 'underline' => 0) {|btn|
    pack('side' => 'left')
    btn.menu(menu = TkMenu.new(btn, 'tearoff' => false))
    menu.add('command', 'label' => 'New File', 'command' => proc{new_file})
    menu.add('command', 'label' => 'Open File', 'command' => proc{open_file})
    menu.add('separator')
    menu.add('command', 'label' => 'Save File', 'command' => proc{save_file})
    menu.add('command', 'label' => 'Save As..', 'command' => proc{save_as})
    menu.add('separator')
    menu.add('command', 'label' => 'Quit', 'command' => proc{
	       if Tk.messageBox('icon'=>'question', 'type' => 'yesno',
		    'title' => 'Really QUIT?', 
		    'message' => "Really QUIT ? ") == "yes"
		 TkRoot.new.destroy
		 exit
	       end
	     })
  }
  TkMenubutton.new(f, 'text' => 'View', 'underline' => 0) {|btn|
    pack('side' => 'left')
    btn.menu(menu = TkMenu.new(btn, 'tearoff' => false))
    menu.add('command', 'label' => 'Original',
	     'command' => proc{show_original})
    menu.add('command', 'label' => 'Current Encoded',
	     'command' => proc{show_encode})
  }
  TkMenubutton.new(f, 'text' => 'Help', 'underline' => 0) {|btn|
    pack('side' => 'right')
    btn.menu(menu = TkMenu.new(btn, 'tearoff' => false))
    menu.add('command', 'label' => 'Version', 'command' => proc{show_version})
    menu.add('command', 'label' => 'Usage', 'command' => proc{show_usage})
  }
}


class OrderEditor
  
  def initialize nusdef, cell, apply_proc, names = nil
    order_editor = self
    t = TkToplevel.new
    t.title "Change Order"
    names = cell.value.flatten if names.nil?
    @names = cell_names = names

    # ListBox with scrollbar
    f = TkFrame.new(t).pack('fill' => 'both', 'expand' => true)
    s = TkScrollbar.new(f)
    @lb = lb = TkListbox.new(f) {
      yscrollcommand proc{|first, last| s.set first, last}
    }
    s.command(proc{|*args| lb.yview *args})
    @lb.pack('side'=>'left', 'expand'=>true, 'fill'=>'both')
    s.pack('side'=>'left', 'fill'=>'y')
    @lb.insert(0, *cell_names)

    # UP and DOWN buttons
    updwn = TkFrame.new(f).pack('side' => 'left')
    TkButton.new(updwn) {
      image TkBitmapImage.new('file' => '/usr/include/X11/bitmaps/Up')
      command proc{
	sel = lb.curselection[0]
	order_editor.move(sel, -1) if sel && sel > 0
      }
    }.pack('pady' => '10')
    TkButton.new(updwn) {
      image TkBitmapImage.new('file' => '/usr/include/X11/bitmaps/Down')
      command proc{
	sel = lb.curselection[0]
	order_editor.move(sel, +1) if sel && sel < cell_names.size - 1
      }
    }.pack('pady' => '10')

    # Commands
    f = TkFrame.new(t).pack('fill' => 'both')
    TkButton.new(f, 'text' => 'Apply') {
      command proc{
	if Tk.messageBox('icon'=>'question', 'type' => 'okcancel',
			 'title' => 'Really OK?', 'parent' => t,
			 'message' => "Really change the order ?") == "ok"
	  apply_proc.call(cell, names)
	  $edwin.refresh
	  t.destroy
	end
      }
    }.pack('side' => 'left', 'padx' => '10')
    TkButton.new(f, 'text' => 'Cancel', 'command' => proc{ t.destroy }
		 ).pack('side' => 'right', 'padx' => '10')
  end

  def move select, step
    selname = @names[select]
    replace = @names[select + step]
    @names[select]        = replace
    @names[select + step] = selname
    @lb.delete select
    @lb.insert(select + step, selname)
    @lb.selection_set(select + step)
    @lb.see(select + step)
  end
end


class EntryDialog

  def initialize msg = nil, title = nil, defstr = nil
    @top = t = TkToplevel.new
    @msg = msg
    t.grab
    t.title title unless title.nil?
    f = TkFrame.new(t).pack
    TkLabel.new(f, 'text'=> msg).pack('side' => 'left') unless msg.nil?
    @ans = ans = TkVariable.new
    ans.value = defstr
    TkEntry.new(f, 'textvariable'=>ans).pack('side' => 'left')
    btns = TkFrame.new(t).pack('side' => 'bottom')
    TkButton.new(btns, 'text'=>'OK', 'command'=>proc{
		   ans.value = ans.value.strip
		   t.destroy if ans.value.size > 0
		 }).pack('side' => 'left')
    TkButton.new(btns, 'text'=>'Cancel', 'command'=>proc{
		   ans.value = nil
		   t.destroy
		 }).pack('side' => 'left')
  end

  def wait_ret
    @top.wait_destroy
    if @ans.value.size > 0 && 
	Tk.messageBox('icon'=>'question', 'type'=>'okcancel', 
		      'title'=>'Really OK? ', 'message'=>"Really " + 
		      @msg + @ans.value + " ? ") == "ok"
      return @ans.value
    else
      return ""
    end
  end

end

class CellItemMenu

  def initialize parent, nusdef, cell, pair_name
    @nusdef = nusdef
    @cell = cell
    @pair_name = pair_name
    ci = self
    ao = TkFrame.new(parent).pack('side'=>'right', 'padx'=>3)
    @addbutton = TkButton.new(ao, 'text' => 'Add..') {
      command proc{
	ret = EntryDialog::new(" Add: ", "Add").wait_ret
	ci.add_item(ret) if ret.size > 0
      }
    }.pack('fill' => 'x', 'side'=>'top', 'ipady'=>0)
    @orderbutton = TkButton.new(ao, 'text' => 'Order..') {
      command proc{ 
	OrderEditor::new(nusdef, cell, proc{|cell, names|
			   neworder = names.map{|n| cell.value.index(n) }
			   elmap = nusdef.cell("element", "elementmap").value
			   case pair_name
			   when "memberlist/member"
			     axis = "member"
			   when "validtime1/vt_list"
			     axis = "valid"
			   when "plane1/name"
			     axis = "plane"
			   else
			     axis = nil
			   end
			   unless axis.nil?
			     elmap.each {|elem| elem.neworder(axis, neworder) }
			   end
			   cell.value = names.flatten
			 }) }
    }.pack('fill' => 'x', 'side'=>'top', 'ipady'=>0)
    @itemf = TkFrame.new(parent).pack('side'=>'left', 
				      'fill'=>'x', 'expand'=>true)
    @wf = TkFrame.new(@itemf)
    @wf.pack('fill'=>'x', 'expand'=>true)
  end

  def add_item(item)
    v = @cell.value
    if v.include?(item)
      Tk.messageBox('icon'=>'warning', 'type' => 'ok',
		    'title' => 'Alredy exists', 'parent' => @wf, 
		    'message' => "\"" + item + "\" alredy exists.")
      return
    end
    case @pair_name
    when "memberlist/member", "validtime1/vt_list", "plane1/name"
      v.push item
      elmap = @nusdef.cell("element", "elementmap").value
      elmap.each {|elem|
	case @pair_name
	when "memberlist/member"
	  nm = elem.n_member
	  elem.push_member(elem.slice_member_at(nm - 1))
	  elem.modified[nm] = nil
	when "validtime1/vt_list"
	  nv = elem.n_vt
	  elem.push_valid(elem.slice_valid_at(nv - 1))
	  elem.modified.each_index{|i| elem.modified[i] = nil}
	when "plane1/name"
	  np = elem.n_pl
	  elem.push_plane(elem.slice_plane_at(np - 1))
	  elem.modified.each_index{|i| elem.modified[i] = nil}
	end
      }
    when "subcntl/group-size"
      v.push NusDef::SubCntl::new(item.strip.split)
    end
    @cell.value = v
    $edwin.refresh
  end

  def ren_item(i, item)
    v = @cell.value.flatten
    if v[i] != item && v.include?(item)
      Tk.messageBox('icon'=>'warning', 'type' => 'ok',
		    'title' => 'Alredy exists', 'parent' => @wf, 
		    'message' => "\"" + item + "\" alredy exists.")
      return
    end
    case @pair_name
    when "memberlist/member", "validtime1/vt_list", "plane1/name"
      v[i] = item
    else
      v[i].value = item
    end
    @cell.value = v
    $edwin.refresh
  end

  def del_item(i)
    v = @cell.value.flatten
    v.delete_at(i)
    case @pair_name
    when "memberlist/member", "validtime1/vt_list", "plane1/name"
      elmap = @nusdef.cell("element", "elementmap").value
      elmap.each {|elem|
	case @pair_name
	when "memberlist/member"
	  elem.delete_member_at(i)
	when "validtime1/vt_list"
	  elem.delete_valid_at(i)
	when "plane1/name"
	  elem.delete_plane_at(i)
	end
      }
    end
    @cell.value = v
    $edwin.refresh
  end
  
  def refresh
    state = @cell.effective? ? 'normal' : 'disabled'
    @addbutton.configure('state'=>state)
    @orderbutton.configure('state'=>state)
    @wf.destroy
    @wf = TkFrame.new(@itemf, 'borderwidth'=>2, 'relief'=>'sunken')
    @wf.pack('fill'=>'x', 'expand'=>true)
    ca = TkCanvas.new(@wf, 'height'=>'1c').pack('fill'=>'x', 'expand'=>true)
    return if (items = @cell.value.size) < 1
    x = 0
    ci = self
    (0..(items - 1)).each {|i|
      w = TkMenubutton.new(ca, 'relief'=>'raised', 'text'=>@cell.value[i])
      TkcWindow.new(ca, "#{x}c", '0.5c', 'tags'=>'window', 'window'=>w)
      x += (@cell.value[i].to_s.size + 2) * 0.25
      options = TkMenu.new(w, 'tearoff'=>0)
      options.add('command', 'label'=>'Rename..', 'command' => proc{
		    ret = EntryDialog::new("\"" + @cell.value[i].to_s + 
					   "\" rename to: ", "Rename",
					   @cell.value[i].to_s).wait_ret
		    ci.ren_item(i, ret) if ret.size > 0
		  })
      options.add('command', 'label'=>'Delete', 'command' => proc{
		    ci.del_item(i) if 
		      Tk.messageBox('icon'=>'question', 
				    'type' => 'okcancel','parent' => @wf,
				    'title' => 'Really OK?', 
				    'message' => "Really DELETE  \"" +
				    @cell.value[i].to_s + "\" ? ") == "ok"
		  })
      w.menu(options)
      w.configure('state'=> @cell.effective? ? 'normal' : 'disabled')
    }
    xscr = TkScrollbar.new(@wf, 'orient'=>'horizontal').pack('fill'=>'x')
    ca.xscrollbar(xscr)
    ca.configure('scrollregion'=>['-1c','0c', "#{x}c", '0c'])
    canvas_xscroll_bind([ca])
  end

end

class EntryWindow

  def initialize parent, edwin, nusdef, linename
    @nusdef = nusdef
    @height = 1.7
    @refresh_tgts = Hash::new
    @title = TkLabel.new(parent, 'text'=>linename,
			 'borderwidth'=>'2', 'relief'=>'groove')
    @title.pack('side'=>'left', 'anchor'=>'n', 'padx'=>'10', 'pady'=>'2',
	       'ipadx' => '5', 'ipady' => '5')
    @line = nusdef.line(linename)
    @line.indices.each { |cn|
      cf = TkFrame.new(parent)
      cf.pack('side' => 'left', 'anchor' => 'n', 'fill'=>'x')
      label = TkLabel.new(cf, 'text'=>cn).pack('anchor'=>'w')
      pair_name = linename + "/" + cn
      case (vf = (cell = nusdef.cell(linename, cn)).validform).class.name
      when "Regexp"
	case pair_name
	when "memberlist/member", "validtime1/vt_list", "plane1/name"
	  cim = CellItemMenu::new(cf, nusdef, cell, pair_name)
	  @refresh_tgts[cn] = [label, cim, "Class" ]
	  @height = 2.8
	else
	  e = TkEntry.new(cf).pack
	  @refresh_tgts[cn] = [label, e, "Entry" ]
	  e.textvariable(tv = TkVariable.new)
	  input_end = TkVirtualEvent.new('Return', 'FocusOut')
	  e.bind(input_end, proc{
		   cell.value = if cell.value.is_a? Array
				  v = tv.value.strip.split
				  (v.is_a? Array) ? v : Array::new(1, v)
				else
				  tv.value
				end
		   if linename == "memberlist" && cn == "member" && 
		       cell.value == []
		     cell.value = [""]
		   end
		   edwin.refresh
		 })
	  e.width cell.width.nil? ? cn.size : cell.width
	end
      when "Array"
	mb = TkMenubutton.new(cf, 'relief' => 'raised')
	@refresh_tgts[cn] = [label, mb, "Menu" ]
	options = TkMenu.new(mb) {
	  tearoff 0
	  vf.each { |item|
	    add 'command', 'label'=> item, 'command' => proc{
	      cell.value = item
	      edwin.refresh
	    }
	  }
	}
	mb.menu(options)
	mb.pack('side' => 'left', 'anchor' => 'n')
      when "Class"
	case vf.name
	when "ElementMap"
	  ue = TkFrame.new(cf).pack('fill'=>'x', 'expand'=>true)
	  esm = ElementSelectMenu::new(ue, nusdef)
	  @refresh_tgts[cn] = [label, esm, "Class" ]
	  TkButton.new(ue, 'text'=>'Accept ALL current settings'){
	    command proc{
	      if Tk.messageBox('icon'=>'question', 'type' => 'okcancel',
			       'title' => 'Really OK?', 
			       'message' => "Really touch all " +
			       "members/elements ? ") == "ok"
		cell.value.each{|el|
		  el.elmap.each_index { |im| el.modified[im] = true }
		}
		edwin.refresh
	      end
	    }
	  }.pack('side'=>'right')
	  TkButton.new(cf, 'text' => 'Add NEW element..') {|bw|
	    command proc{ ElementMapEditor::new(bw, nusdef) }
	  }.pack('fill' => 'x', 'side'=>'left')
	  TkButton.new(cf, 'text' => 'Change Order of elements..') {
	    command proc{ 
	      OrderEditor::new(nusdef, cell, 
			       proc{|cell, names|
				 neworder = names.map{|name|
				   cell.value.reject{|elem|
				     elem.elname != name
				   }
				 }
				 cell.value = neworder.flatten
			       },
			       cell.value.map{|item| item.elname})}
	  }.pack('fill' => 'x', 'side' => 'left')
	  @height = 2.8
	when "NusDef::InOut_Unit"
	  ioumb = InOutUnitButtons::new(cf, nusdef, cell)
	  @refresh_tgts[cn] = [label, ioumb, "Class"]
	when "NusDef::SubCntl"
	  cim = CellItemMenu::new(cf, nusdef, cell, pair_name)
	  @refresh_tgts[cn] = [label, cim, "Class" ]
	  @height = 2.8
	end
      end
    }
  end
  attr_reader(:height)
  
  def refresh
    if @line.needed?
      @title.configure('foreground' => @line.valid? ? 'blue' : 'red')
    elsif @line.omit? || ! @line.valid?
      @title.configure('foreground' => 'gray60')
    else
      @title.configure('foreground' => 'black')
    end
    cns = @line.indices
    cns.each_index {|i|
      cell = @line.cell[cns[i]]
      ref = @refresh_tgts[cns[i]]
      next if ref.nil?
      ref[0].configure('foreground' => (cell.effective? ? 'black' : 'gray'))
      case ref[2]
      when "Entry"
	ref[1].configure('state' => 'normal')
	ref[1].configure('borderwidth' => 2, 'relief' => 'sunken')
	ref[1].delete(0, 'end')
	ref[1].insert('end', cell.to_s)
      when "Menu"
	ref[1].configure('text' => cell.value)
      when "Class"
	ref[1].refresh
      end
      case ref[2]
      when "Entry", "Menu"
	if cell.effective?
	  ref[1].configure('foreground' => 'black')
	  ref[1].configure('state' => 'disabled', 'borderwidth' => 1,
			   'relief' => 'solid') if cell.auto?
	else
	  ref[1].configure('foreground' => 'gray', 'state' => 'disabled')
	end
      end
    }
  end

end

class InOutUnitButtons

  def initialize parent, nusdef, cell
    @cell = cell
    @iomb = TkMenubutton.new(parent, 'relief' => 'raised')
    io_options = TkMenu.new(@iomb) {
      tearoff 0
      NusDef::InOut_Unit::InOutValid.each { |item|
	add 'command', 'label'=> item, 'command' => proc{
	  cell.value.inout = item
	  $edwin.refresh
	}
      }
    }
    @iomb.menu(io_options)
    @iomb.pack('side' => 'left', 'anchor' => 'n', 'padx'=>3)
    @unit = TkMenubutton.new(parent, 'relief' => 'raised')
    uni_options = TkMenu.new(@unit) {
      tearoff 0
      NusDef::InOut_Unit::UnitValid.each { |item|
	add 'command', 'label'=> item, 'command' => proc{
	  cell.value.unit = item
	  $edwin.refresh
	}
      }
    }
    @unit.menu(uni_options)
    @unit.pack('side' => 'left', 'anchor' => 'n', 'padx'=>3)
  end

  def refresh
    @iomb.configure('text'=>@cell.value.inout)
    @unit.configure('text'=>@cell.value.unit)
    if @cell.effective?
      @iomb.configure('foreground' => 'black')
      @unit.configure('foreground' => 'black')
      @iomb.configure('state' => 'disabled', 'borderwidth' => 1,
		      'relief' => 'solid') if @cell.auto?
      @unit.configure('state' => 'disabled', 'borderwidth' => 1,
		      'relief' => 'solid') if @cell.auto?
    else
      @iomb.configure('foreground' => 'gray', 'state' => 'disabled')
      @unit.configure('foreground' => 'gray', 'state' => 'disabled')
    end
  end

end

class EditWindow

  def initialize nusdef
    @currentfilesize = TkVariable.new
    @nusdef = nusdef
    @frame = TkFrame.new(nil).pack('expand'=>'yes', 'fill'=>'y', 'padx' => '5')
    f0 = TkFrame.new(@frame).pack
    f = TkFrame.new(f0, 'borderwidth' => '2', 'relief'=> 'sunken')
    f.pack('expand'=>'yes', 'fill'=>'both', 'padx'=>'2', 'side'=>'left')
    @canvas = canvas = TkCanvas.new(f, 'width'=>'22c', 'height'=>'17c')
    canvas.pack('side'=>'left', 'expand' => true, 'fill' => 'both')
    yscr = TkScrollbar.new(f0).pack('fill'=>'y', 'side'=>'right')
    canvas.yscrollbar(yscr)
    edwin = self
    ypos = 0.5
    @entrywins = Array::new
    nusdef.entries.each { |en|
      f1 = TkFrame.new(canvas, 'borderwidth' => '2', 'relief'=> 'sunken')
      TkButton.new(f1, 'text' => 'HELP',
		   'command' => (ENV["LANG"] =~ /^ja/) ?
		   proc{TxtDialog::new(en, nusdef.line(en).comment)} :
		   proc{TxtDialog::new(en, nusdef.line(en).comment_en)}
		   ).pack('side'=>'right', 'padx' => '10', 'fill'=>'x')
      TkcWindow.new(canvas, '1c', "#{ypos}c", 'tags'=>'window', 
		    'window' => f1, 'width' => '20c', 'anchor' => 'nw')
      ew = EntryWindow::new(f1, edwin, nusdef, en)
      @entrywins.push ew
      ypos += ew.height
    }
    canvas.configure('scrollregion' => ['0c', '0c', '0c', "#{ypos}c"])
    canvas_yscroll_bind([canvas])

    tf = TkFrame.new(@frame).pack('side'=>'bottom','fill'=>'x','expand'=>true)
    TkLabel.new(tf,'text' => 'NusDef Editor Ver.0.1e').pack('side'=>'right')
    TkLabel.new(tf,'text'=>" Current required disk space: "
		).pack('side'=>'left')
    @cursize = TkLabel.new(tf)
    @cursize.pack('side'=>'left')
    refresh
  end

  def refresh
    @entrywins.each {|ew| ew.refresh }
    s = 0
    @nusdef.cell("element", "elementmap").value.each {|elem|
      s += elem.allowed_size
    }
    xy = @nusdef.cell("size", "nx-ny").value
    scale = @nusdef.pack_size
    if scale.nil?
      s_str = "unknown" + sprintf(" (elements number: %d)",
			      s * xy[0].to_i * xy[1].to_i)
    else
      s *= xy[0].to_i * xy[1].to_i * scale
      s_str = sprintf("%.1f Mbytes", sz = s / 1024.0 / 1024.0)
      s_str = sprintf("%.1f Kbytes", sz = s / 1024.0) if sz < 1.0
      s_str = sprintf("%d bytes", s) if sz < 1.0
    end
    @cursize.configure('text'=>s_str) unless @cursize.nil?
  end

  def destroy
    @frame.destroy
  end

end


class ElementSelectMenu

  def initialize parent, nusdef
    @nusdef = nusdef
    @cell = nusdef.cell("element", "elementmap")
    @widget = TkMenubutton.new(parent,'relief' => 'raised')
    @widget.pack('side'=>'left', 'fill' => 'x', 'expand'=>'true')
  end

  def refresh
    elems = @cell.value
    @widget.configure('text' => "Select ELEMENT to EDIT..")
    TkWinfo.children(@widget).each {|c| c.destroy }
    options = TkMenu.new(@widget, "tearoff" => 0)
    nitem = 0
    elems.each {|element|
      options.add('command', 'label' => element.elname, 'command' => 
		  proc{ ElementMapEditor::new(options, @nusdef, element) } )
      color = (@cell.valid? ? 'black' : 'red')
      if ! element.modified.include?(nil) && element.modified.include?(true)
	color = (element.modified.include?(false) ? 'orange3' : 'blue')
      end
      options.entryconfigure(elems.index(element), 'foreground' => color)
      if nitem > 19
	options.entryconfigure(elems.index(element), 'columnbreak' => true)
	nitem = 0
      end
      nitem += 1
    }
    @widget.menu(options)
  end

end

load_file ARGV[0]

Tk.mainloop
