#include "nusgrib.h"
#include "nusdas.h"
#define DMAX 256
#define GTFILEK "./grib_elemtab.txt"
#define GTFILEH "./grib_elemtab.txt"

struct grib_table {
  int num;
  char name[7];
} gt[DMAX];

int  vm, num_gt, t_num, num_l, num_e;
char type1[9]="", type2[5]="", type3[5]="";
char wtype1[9], wtype2[5], wtype3[5];
int  vt[DMAX], lv[DMAX], el[DMAX], tunit, xsize, ysize;

/* -------------------------------------------------------------------------- */
void read_grib_table()
/* -------------------------------------------------------------------------- */
{
FILE *fp;
char sl[240];

  if ((fp=fopen(GTFILEK, "r"))==NULL) {
    if ((fp=fopen(GTFILEH, "r"))==NULL) {
      fprintf(stderr, "Can not read grib element table file <%s>\n", GTFILEK);
      exit(1);
    }
  }
  num_gt = 0;
  while(fgets(sl, 240, fp)!=NULL) {
    if (sscanf(sl, "%d %s", &(gt[num_gt].num), &(gt[num_gt].name))!=2)  break;
    num_gt++;
  }
  fclose(fp);
}

/* -------------------------------------------------------------------------- */
void read_def_file(FILE *fpd)
/* -------------------------------------------------------------------------- */
{
int i, j;
char ws[80];

  fscanf(fpd, "type1 %s %s %s\n", type1, &type1[4], &type1[6]);
  fscanf(fpd, "type2 %s %s\n", type2, &type2[2]);
  fscanf(fpd, "type3 %s\n", type3);
  fscanf(fpd, "validtime %d", &t_num);
  fscanf(fpd, " %s out\n", ws);
  if (strcmp(ws, "MIN")==0)       tunit = 0;
  else if (strcmp(ws, "HOUR")==0) tunit = 1;
  else if (strcmp(ws, "DAY")==0)  tunit = 2;
  else if (strcmp(ws, "MON")==0)  tunit = 3;
  fscanf(fpd, "validtime1 ALL_LIST");
  for(i=0; i<t_num; i++)  fscanf(fpd, " %d", &vt[i]);
  fscanf(fpd, "\nplane %d", &num_l);
  fscanf(fpd, "\nplane1");
  for(i=0; i<num_l; i++) {
    fscanf(fpd, " %s", ws);
    lv[i] = (strcmp(ws, "SURF")==0) ? 0 : atoi(ws);
  }
  fscanf(fpd, "\nelement %d\n", &num_e);
  for(i=0; i<num_e; i++) {
    fscanf(fpd, "\nelementmap %s 0", ws);
    for(j=0; j<num_gt; j++)  if (strcmp(ws,gt[j].name)==0)  break;
    el[i] = (j==num_gt) ? atoi(ws) : gt[j].num;
  }
  fscanf(fpd, "\nsize %d %d", &xsize, &ysize);
}

/* -------------------------------------------------------------------------- */
void write_def_file(FILE *fpd)
/* -------------------------------------------------------------------------- */
{
int i, j;

  fprintf(fpd, "validtime %d", t_num);
  switch(tunit) {
    case 0: fprintf(fpd, " MIN out\n"); break;
    case 1: fprintf(fpd, " HOUR out\n"); break;
    case 2: fprintf(fpd, " DAY out\n"); break;
    case 3: fprintf(fpd, " MON out\n"); break;
  }
  fprintf(fpd, "validtime1 ALL_LIST");
  for(i=0; i<t_num; i++)  fprintf(fpd, " %d", vt[i]);
  fprintf(fpd, "\nplane %d", num_l);
  fprintf(fpd, "\nplane1");
  for(i=0; i<num_l; i++)
    if (lv[i] <= 0)  fprintf(fpd, " SURF");
    else             fprintf(fpd, " %d", lv[i]);
  fprintf(fpd, "\nelement %d\n", num_e);
  for(i=0; i<num_e; i++) {
    for(j=0; j<num_gt; j++)  if (el[i]==gt[j].num)  break;
    if (j==num_gt)  fprintf(fpd, "elementmap %d 0\n", el[i]);
    else            fprintf(fpd, "elementmap %s 0\n", gt[j].name);
  }
  fprintf(fpd, "size %d %d\n", xsize, ysize);
  fprintf(fpd, "packing 2PAC\n");
}

/* -------------------------------------------------------------------------- */
int main(int argc, char *argv[])
/* -------------------------------------------------------------------------- */
{
FILE *fp, *fpd=NULL, *fpg;
DIR  *dp;
char    fname[80], dname[160], clev[7], celem[7], cmd[80], hc[3], vc[3];
unsigned char wk[8], *ud;
float	*d;
int	rt, tlen, i, j, k, p12, ief;
int     yy, mm, dd, hh, mn, btt, vtt, dnum;
long	offset;
struct sect1 s1;
struct sect2 s2;

for(i=1, vm=0; i<argc; i++)  if (strcmp(argv[i], "-v")==0) vm = 1;
strcpy(dname, "NUSDAS01");
for(i=1; i<argc; i++)
  if (strncmp(argv[i], "-NUSDAS", 7)==0)  strcpy(dname, &argv[i][1]);
strcat(dname, "/nusdas_def");
if ((dp=opendir(dname))==NULL) {
/*
  fprintf(stdout, "No NuSDaS root directory <%s>!!\n", dname); 
  fprintf(stdout, "Do you create the directory <yes or no>??\n");
  scanf("%s", cmd);
  if (strcmp(cmd, "yes")!=0)  exit(1);
*/
  sprintf(cmd, "mkdir -p %s", dname);
  system(cmd);
} else 
  closedir(dp);

read_grib_table();

/* *********************************** */
/* k = 0 : Make NuSDaS define file     */
/* k = 1 : Convert GRIB file to NuSDaS */
/* *********************************** */

for(k=0; k<2; k++) {

for(j=1; j<argc; j++) {

  if (argv[j][0]=='-')  continue;
  if ((fp=fopen(argv[j], "r"))==NULL)  {
    fprintf(stderr, "Can not open GRIB file <%s>!!\n", argv[j]);
    continue;
  }

  ief = 0;
  while(1) {

    offset = 0;
    while(1) {
      fseek(fp, offset, SEEK_CUR);
      if (fread_native(wk, 1, 8, fp)!=1)  { ief=1; break; }
      tlen = grib_dec_sect0(wk);
      if (tlen!=-1) break;
      offset = -7;
    }
    if (ief==1)  break;

    SOUT("GRIB%d", tlen);
    tlen -= 8;
    ud = (unsigned char *)malloc(sizeof(unsigned char)*tlen);
    fread_native(ud, 1, tlen, fp);

    if (k==0) { /* Make NuSDaS define file */

      rt = grib_dec_sect1_5(ud, &s1, &s2, NULL);
      switch((int)s2.proj) {
        case 0:  strcpy(hc, "LL");  break;
        case 1:  strcpy(hc, "MR");  break;
        case 3:  strcpy(hc, "LM");  break;
        case 5:  strcpy(hc, "PS");  break;
        default: strcpy(hc, "XX");  break;
      }
      switch((int)s1.lindx) {
        case   1:
        case 102:
                  strcpy(vc, "LY");  break;
        case 100: strcpy(vc, "PP");  break;
        case 103: strcpy(vc, "ZZ");  break;
        case 107: strcpy(vc, "SG");  break;
        case 109: strcpy(vc, "HB");  break;
        case 119: strcpy(vc, "ET");  break;
        default:  strcpy(vc, "XX");  break;
      }
      sprintf(wtype1, "M%3.3d%2s%2s", s1.proc, hc, vc);
      switch((int)s1.pindx2) {
        case  0: if (s1.p1==0)  strcpy(wtype2, "EASV");
                 else           strcpy(wtype2, "FCSV"); break;
        case  1: strcpy(wtype2, "FCSV");                break;
        case  3: strcpy(wtype2, "XXMV");                break;
        case  4: strcpy(wtype2, "XXAV");                break;
        case 10: strcpy(wtype2, "FCSV");                break;
        default: strcpy(wtype2, "XXXX");                break;
      }
      sprintf(wtype3, "C%3.3d", s1.center);
      if (strcmp(type1,wtype1)!=0 || strcmp(type2,wtype2)!=0
                                  || strcmp(type3,wtype3)!=0) {
        if (fpd!=NULL) {
          write_def_file(fpd);
          fclose(fpd);  fpd = NULL;
        }
        sprintf(fname, "%s/%s-%s-%s.def", dname, wtype1, wtype2, wtype3);
        if ((fpd=fopen(fname, "r"))==NULL) {
          if ((fpd=fopen(fname, "w"))==NULL) {
            fprintf(stderr, "Can not create new define file <%s>\n", fname);
            exit(1);
          }
          fprintf(fpd,"type1 %4.4s %2.2s %2.2s\n",wtype1,&wtype1[4],&wtype1[6]);
          fprintf(fpd,"type2 %2.2s %2.2s\n", wtype2,&wtype2[2]);
          fprintf(fpd,"type3 %4.4s\n", wtype3);
          t_num = 0; for(i=0; i<DMAX; i++)  vt[i] = -9;
          num_l = 0; for(i=0; i<DMAX; i++)  lv[i] = -9;
          num_e = 0; for(i=0; i<DMAX; i++)  el[i] = -9;
          tunit = s1.pindx1; xsize = s2.xs; ysize = s2.ys;
          strcpy(type1, wtype1); strcpy(type2, wtype2); strcpy(type3, wtype3);
        } else {
          fclose(fpd);
          fpd = fopen(fname, "r+");
          read_def_file(fpd);
          rewind(fpd);
          fprintf(fpd, "type1 %4.4s %2.2s %2.2s\n", type1,&type1[4],&type1[6]);
          fprintf(fpd, "type2 %2.2s %2.2s\n", type2,&type2[2]);
          fprintf(fpd, "type3 %4.4s\n", type3);
        }
      }
      switch(s1.pindx2) {
        case  4:
        case  5: p12 = s1.p2;  break;
        case 10: p12 = s1.p1*256+s1.p2;  break;
        default: p12 = s1.p1;  break;
      }
      for(i=0; i<t_num; i++)  if (p12==vt[i])  break;
      if (i==t_num) { vt[i] = p12; t_num++; }
      for(i=0; i<num_l; i++)  if (s1.levl==lv[i])  break;
      if (i==num_l) { lv[i] = s1.levl; num_l++; }
      for(i=0; i<num_e; i++)  if (s1.parm==el[i])  break;
      if (i==num_e) { el[i] = s1.parm; num_e++; }
      if (tunit!=s1.pindx1)
        fprintf(stderr, "This file includes different time unit!!\n");
      if (xsize!=s2.xs || ysize!=s2.ys)
        fprintf(stderr, "This file includes different grid size!!\n");
      free(ud);

    } else if (k==1) {  /* Convert GRIB file to NuSDaS */

      rt = grib_dec_sect1_5(ud, &s1, &s2, &d);
      switch((int)s2.proj) {
        case 0:  strcpy(hc, "LL");  break;
        case 1:  strcpy(hc, "MR");  break;
        case 3:  strcpy(hc, "LM");  break;
        case 5:  strcpy(hc, "PS");  break;
        default: strcpy(hc, "XX");  break;
      }
      switch((int)s1.lindx) {
        case   1:
        case 102:
                  strcpy(vc, "LY");  break;
        case 100: strcpy(vc, "PP");  break;
        case 103: strcpy(vc, "ZZ");  break;
        case 107: strcpy(vc, "SG");  break;
        case 109: strcpy(vc, "HB");  break;
        case 119: strcpy(vc, "ET");  break;
        default:  strcpy(vc, "XX");  break;
      }
      sprintf(wtype1, "M%3.3d%2s%2s", s1.proc, hc, vc);
      switch((int)s1.pindx2) {
        case  0: if (s1.p1==0)  strcpy(wtype2, "EASV");
                 else           strcpy(wtype2, "FCSV"); break;
        case  1: strcpy(wtype2, "FCSV");                break;
        case  3: strcpy(wtype2, "XXMV");                break;
        case  4: strcpy(wtype2, "XXAV");                break;
        case 10: strcpy(wtype2, "FCSV");                break;
        default: strcpy(wtype2, "XXXX");                break;
      }
      sprintf(wtype3, "C%3.3d", s1.center);
      yy = (s1.century-1)*100+s1.year;
      mm = s1.month; dd = s1.date; hh = s1.hour; mn = s1.min;
      btt = nwp_ymdhm2seq(yy, mm, dd, hh, mn);
      switch(s1.pindx2) {
        case  4:
        case  5: p12 = s1.p2;  break;
        case 10: p12 = s1.p1*256+s1.p2;  break;
        default: p12 = s1.p1;  break;
      }
      switch((int)s1.pindx1) {
        case 0:  vtt = btt+p12;       break;
        case 2:  vtt = btt+p12*1440;  break;
        case 1:
        default: vtt = btt+p12*60;    break;
      }
      
      if (s1.levl <= 0)  strcpy(clev, "SURF");
      else               sprintf(clev, "%-d", s1.levl);
      for(i=strlen(clev); i<6; i++) clev[i] = ' ';  clev[i] = '\0';
      for(i=0; i<num_gt; i++)  if (s1.parm==gt[i].num)  break;
      if (i==num_gt)  sprintf(celem, "%-d", s1.parm);
      else            sprintf(celem, "%s", gt[i].name);
      for(i=strlen(celem); i<6; i++) celem[i] = ' ';  celem[i] = '\0';
      dnum = s2.xs*s2.ys;
      rt = nusdas_write(wtype1,wtype2,wtype3,(N_SI4 *)&btt,"    ",(N_SI4 *)&vtt,
                        clev, celem, d, N_R4, (N_SI4 *)&dnum);
      if (rt!=dnum) 
        fprintf(stderr, "nusdas_write error! %d %s %s\n", p12, clev, celem);
/*
      fpg = fopen("sample.dat", "w");
      fwrite(d, sizeof(float), dnum, fpg);
      fclose(fpg);
      exit(1);
*/
      free(d);

    }

    SOUT("\n"); fflush(stdout);

  } /* end while */

  fclose(fp);

} /* end of for loop j */

if (fpd!=NULL) {
  write_def_file(fpd);
  fclose(fpd); fpd = NULL;
}

} /* end of for loop k */
return 0;
}
