#define N_DEF_READDEF_SOURCE
#include "nusdas_sys.h"

/* -------------------------------------------------------------------------- */
N_SI4 n_read_def(FILE *fp, N_ST_DEF *def)
/* -------------------------------------------------------------------------- */
/* read the target define file                                                */
/*  input:  fp  = NuSDaS define file pointer                                  */
/*          def = N_ST_DEF pointer                                            */
/*  return: >=0 = bit flag (off -> the neccesary item specified )             */
/*                         (on ->                not specified )              */
/*          <0  = definition or system ( malloc ) error                       */
/* -------------------------------------------------------------------------- */
/* bit information in a rt. if all specified, rt = 0.                         */
/*  0  = type1                                                                */
/*  1  = type2                                                                */
/*  2  = type3                                                                */
/*  3  = number of validtime                                                  */
/*  4  = list of validtime                                                    */
/*  5  = number of plane                                                      */
/*  6  = list of plane                                                        */
/*  7  = number of element                                                    */
/*  8  = list of element                                                      */
/*  9  = grid size                                                            */
/* -------------------------------------------------------------------------- */
{
char  line[N_LINE_MAX*20],pline[N_LINE_MAX*20], kw[N_LINE_MAX],s[N_LINE_MAX];
char  fmt[N_LINE_MAX], *sp, c1, c2, c3, c4, s1[80], s2[80], s3[80], s4[80];
static unsigned char  c8init[8];
long   offset=0;
N_SI4  i, is, ie, j, js, je, k, ks, ke, l, m, m2, m3, n, fl;
N_SI4  nk, pk, bk, item, pitem=0;
N_SI4  py, pm, pd, ph, pn, rt=N_F_ALL;
N_SI4  c_dc=0, c_vt=0, c_lv=0, c_el=0, f_vt2=0, f_lv2=0;
N_UI4  p, bp;
float  f1, f2, f3, f4;

for(i=0; i<6; i++)  c8init[i] = 0x20;
for(i=6; i<8; i++)  c8init[i] = 0x00;

strcpy(n_routine_name[n_routine_d++], "read_def");

  n_init_def(def);
  fseek(fp, offset, SEEK_SET);

  while((item=n_getline(fp, line))>=0) {
    p = 0;
    if (line[0]=='#')  continue;
    if (sscanf(line, "%s", kw)!=1)  continue;
    if ((sp=n_spscanf(line,&p,"%s"))==NULL)  continue;
    n_char_upper(strcpy(kw, sp));
    bk = nk = n_keyword_set(kw);
    if (nk==N_KW_CONTINUE) {
      strcat(pline," "); strcat(pline,line); strcpy(line,pline);
      strcat(line,"\n");
      nk = pk; p= bp;
      if (nk!=N_KW_ELEMENT_MAP)  item += pitem;
    } else {
      for(i=0; i<N_LINE_MAX*20; i++) pline[i]='\0';
    }
    bp = p;
/* */
switch(nk) {
/* */
case N_KW_NUSDAS:
  if ((sp=n_spscanf(line,&p,"%s"))!=NULL && (i=atol(sp))>0)
    def->version = i;
  else {
    N_ERP1(stderr, "NUSDAS version <%d> is invalid. System version is used\n", i);
  }
  break;
case N_KW_PATH:
  if (sscanf(line, "%s %s", kw, s)!=2) {
    N_ERP1(stderr, "PATH line is error and neglected.\n"); break;
  } else  n_char_upper(s);
  def->path = (strcmp(s, "NWP_PATH_S")==0)    ? N_NWP_PATH_S :
              (strcmp(s, "NWP_PATH_M")==0)    ? N_NWP_PATH_M :
              (strcmp(s, "NWP_PATH_BS")==0)   ? N_NWP_PATH_BS :
              (strcmp(s, "NWP_PATH_VM")==0)   ? N_NWP_PATH_VM :
              (strcmp(s, "RELATIVE_PATH")==0) ? N_RELATIVE_PATH :
              (strcmp(s, "NWP_ESF")==0)       ? N_NWP_ESF : N_DEFAULT;
  if (def->path==N_RELATIVE_PATH) {
    if (sscanf(line, "%s %s %s", kw, s, s)!=3) {
      N_ERP1(stderr, "Invalid relative path name <%s>\n", s);
      fprintf(stderr, "Default rule is used\n");
      def->path = N_DEFAULT;
    } else  strcpy(def->pathname, s);
  }
  break;
case N_KW_FILENAME:
  if (sscanf(line, "%s %s", kw, s)!=2) {
    N_ERP1(stderr, "FILENAME line is error and neglected.\n");
  } else  strcpy(def->filename, s);
  break;
case N_KW_CREATOR:
  if (sscanf(line,"%s %s", kw, s)!=2) {
    N_ERP1(stderr, "CREATOR line is error and neglected.\n");
  } else  strcpy(def->creator, s);
  break;
case N_KW_TYPE1:
  strcpy(def->type1, line);
  rt &= (N_F_ALL^N_F_TYPE1);
  break;
case N_KW_TYPE2:
  strcpy(def->type2, line);
  rt &= (N_F_ALL^N_F_TYPE2);
  break;
case N_KW_TYPE3:
  strcpy(def->type3, line);
  rt &= (N_F_ALL^N_F_TYPE3);
  break;
case N_KW_BASETIME:
  if ((sp=n_spscanf(line, &p, "%d"))==NULL ||
      sscanf(sp,"%4d%2d%2d%2d%2d", &py,&pm,&pd,&ph,&pn)!=5) {
    N_ERP1(stderr, "BASETIME <%s> is invalid and neglected.\n", sp);
  } else  def->basetime_t = nwp_ymdhm2seq(py, pm, pd, ph, pn);
  break;
case N_KW_MEMBER:
  if ((i=atol(n_spscanf(line, &p, "%d")))<=0 || i>N_DC_MAX) {
    N_ERP2(stderr, "number of MEMBER <%d> is out of limit.\n", i);
    N_ERTM(-1);
  }
  def->n_dc = i;
  if (def->dc!=NULL) free(def->dc);
  if ((def->dc=(N_UI4 *)malloc(sizeof(N_UI4)*def->n_dc))==NULL) {
    N_ERP2(stderr, "malloc failure <member:%d>\n", i);
    N_ERTM(-2);
  }
  if ((sp=n_spscanf(line, &p, "%s"))!=NULL) {
    n_char_upper(sp);
    if (strcmp(sp, "OUT")==0)  def->infile_dc = N_OUT_FILE;
  }
  break;
case N_KW_MEMBERLIST:
  c_dc = (def->n_dc>item-1) ? item-1 : def->n_dc;
  for(i=0; i<c_dc; i++) {
    sp = n_spscanf(line, &p, "%s");
    if ((k=strlen(sp))>4)  k = 4;
    *(def->dc+i) = 0x20202020;
    memcpy(def->dc+i, sp, k);
  }
  break;
case N_KW_VALID_NUM:
  if ((i=atol(n_spscanf(line,&p,"%d")))<=0 || i>N_VT_MAX) {
    N_ERP2(stderr, "number of VALIDTIME <%d> is out of limit.\n",i);
    N_ERTM(-1);
  }
  def->n_vt = i;
  if (def->vt1!=NULL) {
    N_ERP1(stderr, "VALIDTIME1 is perhaps re-defined.\n");
    free(def->vt1);
  }
  if ((def->vt1=(N_SI4 *)malloc(sizeof(N_SI4)*def->n_vt))==NULL) {
    N_ERP2(stderr,"malloc failure <validtime:%d>\n", def->n_vt);
    N_ERTM(-2);
  } else  for(i=0; i<def->n_vt; i++) *(def->vt1+i)=0;
  if (def->vt2 != NULL) {
    N_ERP1(stderr, "VALIDTIME2 is perhaps re-defined.\n");
    free(def->vt2);
  }
  if ((def->vt2=(N_SI4 *)malloc(sizeof(N_SI4)*def->n_vt))==NULL) {
    N_ERP2(stderr, "malloc failure <validtime:%d>\n", def->n_vt);
    N_ERTM(-2);
  } else  for(i=0; i<def->n_vt; i++) *(def->vt2+i)=1;
  for(i=0; i<2; i++) {
    if ((sp=n_spscanf(line, &p, "%s"))==NULL)  break;
    n_char_upper(sp);
    if (strcmp(sp,"OUT")==0) { def->infile_vt=N_OUT_FILE; continue; }
    if (strcmp(sp,"IN")==0)  { def->infile_vt=N_IN_FILE; continue; }
    if ((k=strlen(sp))>4) k=4;
    def->time_unit = 0x20202020;
    memcpy(&(def->time_unit), sp, k);
    for(j=0; j<n_tunit_num; j++)
      if (memcmp(&(def->time_unit), n_tunit_string[j],4)==0)  break;
    if (j==n_tunit_num) {
      N_ERP1(stderr, "VALIDTIME time unit <%s> is not supported in NuSDaS!\n", sp);
    }
  }
  rt &= (N_F_ALL^N_F_NVALID);
  break;
case N_KW_VALIDTIME1:
  if (def->vt1==NULL) {
    N_ERP2(stderr,"VALIDTIME must be defined before VALIDTIME1.\n");
    N_ERTM(-3);
  }
  j = (def->n_vt>item-2) ? item-2 : def->n_vt;
  if ((sp=n_spscanf(line, &p, "%s"))==NULL)  break;
  n_char_upper(sp);
  if (strcmp(sp, "ARITHMETIC")==0) { 
    if ((sp=n_spscanf(line, &p, "%d"))==NULL)  break;
    *(def->vt1+0) = (N_SI4)atol(sp);
    if ((sp=n_spscanf(line, &p, "%d"))==NULL)  break;
    k = (N_SI4)atol(sp);
    for(i=1, j=1; i<def->n_vt; i++, j++) *(def->vt1+i) = *(def->vt1+0)+k*i;
  } else if (strcmp(sp,"ALL_LIST")==0) { 
    for(i=0; i<j; i++) {
      if ((sp=n_spscanf(line, &p, "%d"))==NULL)  break;
      *(def->vt1+i) = (N_SI4)atol(sp);
    }
  } else {
    N_ERP2(stderr,"VALIDTIME1 list type is not defined.\n");
    N_ERTM(-3);
  }
  c_vt = j;
  break;
case N_KW_VALIDTIME2:
  if (def->vt2==NULL) {
    N_ERP2(stderr,"VALIDTIME must be defined before VALIDTIME2.\n");
    N_ERTM(-3);
  }
  j = (def->n_vt>item-1) ? item-1 : def->n_vt;
  if ((sp=n_spscanf(line, &p, "%d"))==NULL)  break;
  if ((*(def->vt2+0)=atol(sp))<0) {
    *(def->vt2+0) *= -1;
    for(i=1; i<j; i++)  *(def->vt2+i)=*(def->vt2+0);
  } else {
    for(i=1; i<j; i++) {
      if ((sp=n_spscanf(line, &p, "%d"))==NULL)  break;
      *(def->vt2+i) = atol(sp);
    }
  }
  f_vt2++;
  break;
case N_KW_PLANE_NUM:
  if ((i=atol(n_spscanf(line,&p,"%d")))<=0 || i>N_LV_MAX) {
    N_ERP2(stderr, "number of PLANE <%d> is out of range.\n", i);
    N_ERTM(-1);
  }
  def->n_lv = i;
  if (def->lv1 != NULL) {
    N_ERP1(stderr, "PLANE1 is perhaps re-defined.\n");
    free(def->lv1);
  }
  if ((def->lv1=(N_UI8 *)malloc(sizeof(N_UI8)*def->n_lv))==NULL) {
    N_ERP2(stderr,"malloc failure <plane:%d>\n", def->n_lv);
    N_ERTM(-2);
  } else  for(i=0; i<def->n_lv; i++) memcpy((N_UI8 *)(def->lv1+i), c8init, 8);
  if (def->lv2 != NULL) {
    N_ERP1(stderr, "PLANE2 is perhaps re-defined.\n");
    free(def->lv2);
  }
  if ((def->lv2=(N_UI8 *)malloc(sizeof(N_UI8)*def->n_lv))==NULL) {
    N_ERP2(stderr,"malloc failure <plane:%d>\n", def->n_lv);
    N_ERTM(-2);
  } else  for(i=0; i<def->n_lv; i++) memcpy((N_UI8 *)(def->lv2+i), c8init, 8);
  rt &= (N_F_ALL^N_F_NPLANE);
  break;
case N_KW_PLANE1:
  if (def->lv1==NULL) {
    N_ERP2(stderr,"PLANE must be defined before PLANE1.\n");
    N_ERTM(-3);
  }
  j = (def->n_lv>item-1) ? item-1 : def->n_lv;
  for(i=0; i<j; i++) {
    if ((sp=n_spscanf(line, &p, "%s"))==NULL) break;
    if ((k=strlen(sp))>6)  k = 6;
    memcpy((N_UI8 *)(def->lv1+i), sp, k);
  }
  c_lv = j;
  break;
case N_KW_PLANE2:
  if (def->lv2==NULL) {
    N_ERP2(stderr,"PLANE must be defined before PLANE2.\n");
    N_ERTM(-3);
  }
  j = (def->n_lv>item-1) ? item-1 : def->n_lv;
  for(i=0; i<j; i++) {
    if ((sp=n_spscanf(line, &p, "%s"))==NULL) break;
    if ((k=strlen(sp))>6)  k = 6;
    memcpy((N_UI8 *)(def->lv2+i), sp, k);
  }
  f_lv2++;
  break;
case N_KW_ELEMENT_NUM:
  if ((i=atol(n_spscanf(line,&p,"%d")))<=0 || i>N_EL_MAX) {
    N_ERP2(stderr, "number of element [%d] is out of range.\n", i);
    N_ERTM(-1);
  }
  def->n_el = i;
  if (def->el != NULL) {
    N_ERP1(stderr, "ELEMENT is perhaps re-defined.\n");
    free(def->el);
  }
  if ((def->el=(N_UI8 *)malloc(sizeof(N_UI8)*def->n_el))==NULL) {
    N_ERP2(stderr,"malloc failure <element:%d>\n", def->n_el);
    N_ERTM(-2);
  } else  for(i=0; i<def->n_el; i++) memcpy((N_UI8 *)(def->el+i), c8init, 8);
  rt &= (N_F_ALL^N_F_NELEM);
  break;
case N_KW_ELEMENT_MAP:
  if (bk==N_KW_CONTINUE) c_el--;
  if (def->el==NULL || def->vt1==NULL || def->lv1==NULL) {
    N_ERP2(stderr, "VALIDTIME, PLANE and ELEMENT must be defined before ELEMENTMAP\n");
    N_ERTM(-3);
  }
  if (bk==N_KW_CONTINUE && item<2)  break;
  if (bk!=N_KW_CONTINUE && item<3)  break;
  if ((sp=n_spscanf(line,&p,"%s"))==NULL)  break;
  if (def->elmap==NULL) {
    l = def->n_el*def->n_vt*def->n_lv*def->n_dc;
    if((def->elmap=(N_UI1 *)malloc(sizeof(N_UI1)*l))==NULL){
      N_ERP2(stderr,"malloc failure <elementmap:%d>\n", l);
      N_ERTM(-2);
    } else  for(i=0; i<l; i++) *(def->elmap+i)=0;
  }
  if ((k=strlen(sp))>6) k = 6;
  memcpy((N_UI8 *)(def->el+c_el), sp, k);
  for(i=0; i<c_el-1; i++)
    if (memcmp((N_UI8 *)(def->el+i),(N_UI8 *)(def->el+c_el), 6)==0) {
      N_ERP1(stderr,"Same element is re-defined. Perhaps first definition is only used!.\n");
    }
  m  = def->n_el;
  m2 = def->n_el*def->n_lv;
  m3 = def->n_el*def->n_lv*def->n_vt;
  if ((sp=n_spscanf(line,&p,"%d"))==NULL)  break;
  if ((l=atol(sp))==0) {
    for(k=0; k<def->n_lv; k++)
    for(j=0; j<def->n_vt; j++)
    for(i=0; i<def->n_dc; i++)  *(def->elmap+c_el+k*m+j*m2+i*m3) = 1;
  } else if (l==1) {
    ke = (def->n_lv>item-3) ? item-3 : def->n_lv;
    for(k=0; k<ke; k++) {
      if ((sp=(n_spscanf(line,&p,"%d")))==NULL || ((fl=atol(sp))!=1 && fl!=0)) {
        N_ERP2(stderr,"Invalid elementmap flag for %6.6s.\n",
                      (char *)(def->el+c_el));
        N_ERTM(-4);
      }
      for(j=0; j<def->n_vt; j++)
      for(i=0; i<def->n_dc; i++)  *(def->elmap+c_el+k*m+j*m2+i*m3) = fl;
    }
  } else if (l==2) {
    js = 0;
    while(1) {
      if ((sp=n_spscanf(line,&p,"%d"))==NULL)  break;
      n = atol(sp);
      je = js+n;
      if (je>def->n_vt) {
        N_ERP2(stderr,"Number of ELEMENTMAP flag for %6.6s is too large.\n", (char *)(def->el+c_el)); N_ERTM(-1);
      }
      for(k=0; k<def->n_lv; k++) {
        if ((sp=(n_spscanf(line,&p,"%d")))==NULL || ((fl=atol(sp))!=1&&fl!=0)) {
          N_ERP2(stderr,"Invalid elementmap flag for %6.6s.\n",
                         (char *)(def->el+c_el));
          N_ERTM(-4);
        }
        for(j=js; j<je; j++)
        for(i=0; i<def->n_dc; i++)  *(def->elmap+c_el+k*m+j*m2+i*m3) = fl;
      }
      js += n;
    }
  } else if (l==3) {
    is = 0;
    while(1) {
      if ((sp=n_spscanf(line,&p,"%d"))==NULL) break;
      ie = is+atol(sp);
      if (ie>def->n_dc) {
        N_ERP2(stderr,"Number of ELEMENTMAP flag for %6.6s is too large.\n", (char *)(def->el+c_el)); N_ERTM(-1);
      }
      if ((sp=(n_spscanf(line,&p,"%d")))==NULL || ((fl=atol(sp))!=1 && fl!=2)) {
        N_ERP2(stderr,"Invalid elementmap flag for %6.6s.\n",
                       (char *)(def->el+c_el));
        N_ERTM(-4);
      } else if (fl==1) {
        for(k=0; k<def->n_lv; k++) {
          if ((sp=(n_spscanf(line,&p,"%d")))==NULL||((fl=atol(sp))!=1&&fl!=0)) {
            N_ERP2(stderr,"Invalid elementmap flag for %6.6s.\n",
                           (char *)(def->el+c_el));
            N_ERTM(-4);
          }
          for(i=is; i<ie; i++) 
          for(j=0; j<def->n_vt; j++)  *(def->elmap+c_el+k*m+j*m2+i*m3) = fl;
        }
      } else if (fl==2) {
        js = 0;
        while(1) {
          if ((sp=n_spscanf(line,&p,"%d"))==NULL)  break;
          je = js+atol(sp);
          if (je>def->n_vt) {
            N_ERP2(stderr,"Number of ELEMENTMAP flag for %6.6s is too large.\n", (char *)(def->el+c_el)); N_ERTM(-1);
          }
          for(k=0; k<def->n_lv; k++) {
            if ((sp=(n_spscanf(line,&p,"%d")))==NULL ||
					((fl=atol(sp))!=1 && fl!=0)) {
              N_ERP2(stderr,"Invalid elementmap flag for %6.6s.\n",
                             (char *)(def->el+c_el));
              N_ERTM(-4);
            }
            for(j=js; j<je; j++)
            for(i=is; i<ie; i++)  *(def->elmap+c_el+k*m+j*m2+i*m3) = fl;
          }
          js = je;
          if (js>=def->n_vt)  break;
        }
      }
      is = ie;
      if (is>=def->n_dc)  break;
    }
  }
  c_el++;
  break;
case N_KW_SIZE:
  if (sscanf(line,"%s %d %d", kw, &(def->xs), &(def->ys))!=3) {
    N_ERP1(stderr,"SIZE line error!\n");
  } else 
    rt &= (N_F_ALL^N_F_GRIDS);
  break;
case N_KW_BASEPOINT:
  if ((sscanf(line,"%s %f %f %s %s", kw, &(def->x0), &(def->y0), s1, s2)!=5) ||
              n_set_latlon(s1, s2, &(def->lat_0),&(def->lon_0))<0) {
    N_ERP1(stderr,"BASEPOINT line error!\n");
  }
  break;
case N_KW_DISTANCE:
  if (sscanf(line,"%s %f %f", kw, &(def->xd), &(def->yd))!=3) {
    N_ERP1(stderr,"DISTANCE line error!\n");
  }
  break;
case N_KW_STANDARD:
  if ((i = sscanf(line,"%s %s %s %s %s", kw, s1, s2, s3, s4))<3 ||
           n_set_latlon(s1, s2 ,&(def->lat_s1),&(def->lon_s1))<0){
    N_ERP1(stderr,"STANDARD line error (1st standard latlon)!\n");
  } else if (i==5 && n_set_latlon(s3, s4, &(def->lat_s2), &(def->lon_s2))<0){
    N_ERP1(stderr,"STANDARD line error (2nd standard latlon)!\n");
  } else if (def->lat_s1<0) {
  }
  break;
case N_KW_OTHERS:
  if ((i = sscanf(line,"%s %s %s %s %s", kw, s1, s2, s3, s4))<3 ||
           n_set_latlon(s1, s2, &(def->lat_1), &(def->lon_1))<0) {
    N_ERP1(stderr,"OTHERS line error (1st latlon)!\n");
  } else if (i==5 && n_set_latlon(s3, s4, &(def->lat_2), &(def->lon_2))<0) {
    N_ERP1(stderr,"OTHERS line error (2nd latlon)!\n");
  }
  break;
case N_KW_VALUE:
  if ((sp=n_spscanf(line, &p, "%s"))==NULL) {
    N_ERP1(stderr, "VALUE line error!\n"); break;
  }
  n_char_upper(sp);
  if ((k=strlen(sp))>4)  k = 4;
  def->gpv_mean = 0x20202020;
  memcpy(&(def->gpv_mean), sp, k);
  for(i=0; i<n_mean_num; i++)
    if (memcmp(&(def->gpv_mean), n_mean_string[i], 4)==0)  break;
  if (i==n_mean_num) {
    N_ERP1(stderr, "VALUE <%s> is not supported in NuSDaS!\n", sp);
  }
  break;
case N_KW_PACKING:
  if ((sp=n_spscanf(line, &p, "%s"))==NULL) {
    N_ERP1(stderr, "VALUE line error!\n"); break;
  }
  n_char_upper(sp);
  if ((k=strlen(sp))>4)  k = 4;
  def->pack = 0x20202020;
  memcpy(&(def->pack), sp, k);
  for(i=0; i<n_pack_num; i++)
    if (memcmp(&(def->pack), n_pack_string[i], 4)==0)  break;
  if (i==n_pack_num) {
    N_ERP1(stderr, "PACK <%s> is not supported in NuSDaS!\n", sp);
  }
  break;
case N_KW_MISSING:
  if ((sp=n_spscanf(line, &p, "%s"))==NULL) {
    N_ERP1(stderr, "MISSING line error!\n"); break;
  }
  n_char_upper(sp);
  if ((k=strlen(sp))>4)  k = 4;
  def->miss = 0x20202020;
  memcpy(&(def->miss), sp, k);
  for(i=0; i<n_miss_num; i++)
    if (memcmp(&(def->miss), n_miss_string[i], 4)==0)  break;
  if (i==n_miss_num) {
    N_ERP1(stderr,"MISSING <%s> is not supported in NuSDaS!\n", sp);
  }
  break;
case N_KW_INFORMATION:
  if ((sp=n_spscanf(line, &p, "%s"))==NULL) {
    N_ERP1(stderr, "INFORMATION line error!\n");  break;
  }
  n_char_upper(sp);
  if ((k=strlen(sp))>4)  k = 4;
  def->info[def->n_infoc] = 0x20202020;
  memcpy(&(def->info[def->n_infoc]), sp, k);
  if (sscanf(&line[p], "%s", s)!=1) {
    N_ERP1(stderr, "INFORMATION line error!\n");  break;
  }
  strcpy(def->info_file[def->n_infoc++], s);
  break;
case N_KW_SUBCNTL:
  if ((i=atol(n_spscanf(line,&p,"%d")))<0 || i>N_SUBC_MAX) {
    N_ERP2(stderr, "number of subcntl [%d] is out of limit.\n", i);
    N_ERTM(-1);
  }
  def->n_subc = i;
  for(i=0; i<def->n_subc; i++) {
    if ((sp=n_spscanf(line, &p, "%s"))==NULL) {
      N_ERP1(stderr, "SUBCNTL line error!\n");  break;
    }
    n_char_upper(sp);
    if ((k=strlen(sp))>4)  k = 4;
    def->subc[i] = 0x20202020;
    memcpy(&(def->subc[i]), sp, k);
    for(j=0; j<n_subc_num; j++)
      if (memcmp(&(def->subc[i]),n_subc_string[j],4)==0)  break;
    if (j==n_subc_num) {
      N_ERP1(stderr, "SUBCNTL <%s> is not supported in NuSDaS!\n", sp);
    }
    if ((j=atol(n_spscanf(line,&p,"%d")))<0) {
      N_ERP1(stderr, "SUBCNTL line error!\n");  break;
    }
    def->subc_size[i] = j;
  }
  if (i!=def->n_subc)  def->n_subc = i;
  break;
case N_KW_FORCED_RLEN:
  if ((i=atol(n_spscanf(line,&p,"%d")))<0 || i>N_FRLEN_MAX || i<N_FRLEN_MIN) {
    N_ERP1(stderr, "Record length [%d] is not allowed.\n", i);
    fprintf(stderr, "Variable record length is used.\n");
  } else  def->forced_rlen = i;
  break;
case N_KW_GRIBINFO:
  if (sscanf(line,"%s %s", kw, s)!=2) {
    N_ERP1(stderr, "GRIBINFO line error!\n");
  } else  strcpy(def->grib_info_file, s);
  break;
/* switch end */
}
/* switch end */
    strcpy(pline, line);
    pk = nk;
    pitem = item;
/* while end */
  }
/* while end */
  if (c_vt==def->n_vt)  rt &= (N_F_ALL^N_F_VALIDL);
  if (c_lv==def->n_lv)  rt &= (N_F_ALL^N_F_PLANEL);
  if (c_el==def->n_el)  rt &= (N_F_ALL^N_F_ELEML);
  if (rt!=0) {
    N_ERP2(stderr, "No necessary define item.\n");
    fprintf(stderr, "Lacking or insufficient item list is as follows;\n");
    for(i=0; i<N_F_NUM; i++)
      if ((rt&n_ipow(2,i))!=0)  fprintf(stderr, "\t%s\n", n_f_string[i]);
    N_ERTM(rt);
  }
/* PROJECTION */
  sscanf(def->type1, "%s %s %s", kw, s1, s1);
  if (strcmp(s1,"LM")==0 || strcmp(s1,"PS")==0) {
      if (def->lat_s1>=0)  strcat(s1, "N");
      else                 strcat(s1, "S");
  }
  if ((k=strlen(s1))>4) k=4;
  def->projection = 0x20202020;
  memcpy(&(def->projection), s1, k);
  for(i=0; i<n_proj_num; i++)
    if (memcmp(&(def->projection), n_proj_string[i], 2)==0)  break;
  if (i==n_proj_num) {
    N_ERP1(stderr, "PROJECTION <%4.4s> is not supported in NuSDaS!\n", s1);
  }
/* LV2 */
  if (f_lv2==0)
    for(i=0; i<def->n_lv; i++)
      memcpy((N_UI8 *)(def->lv2+i), (N_UI8 *)(def->lv1+i), 8);

n_routine_d--;
return(rt);
}
