/* dirsub.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <nwpl_timecnv.h>
#include <nwpl_string.h>
#include <nusdas.h>
#include "nusdir.h"
#include "textout.h"

/*
 * === NuSDaS Version 2 pV[` ===
 */

#define MAXENTRYSTR 2048

typedef struct entry {
	char str[MAXENTRYSTR];
	struct entry *left;
	struct entry *right;
} nusdim_entry;

nusdim_entry *entry_top = NULL;

	static nusdim_entry *
add_entry(nusdim_entry *p, char *str) {
	int cond;

	if (p == NULL) {
		p = (nusdim_entry *)malloc(sizeof(nusdim_entry));
		if (p == NULL)
			return NULL;
		strcpy(p->str, str);
		p->left = p->right = NULL;
	} else if ((cond = strcmp(str, p->str)) < 0)
		p->left  = add_entry(p->left,  str);
	else   if (cond > 0)
		p->right = add_entry(p->right, str);
	return p;
}
	void
flush_entries(nusdim_entry *p) {
	if (p != NULL) {
		flush_entries(p->left);
		tprintf(NULL, "%s\n", p->str);
		flush_entries(p->right);
	}
}

	static int
iprint(void *vop, N_SI4 i)
{
	char work[MAXENTRYSTR];
	strcpy(work, minute_to_str(i, NULL, 0));
	entry_top = add_entry(entry_top, work);
}

	static int
cprint(void *vop, char *c, size_t len)
{
	char work[MAXENTRYSTR];

	if (len == 4 && memcmp(c, "\x20\x20\x20\x20", 4) == 0) {
		c = "none";
	}
	if (len >= MAXENTRYSTR)
		len = MAXENTRYSTR - 1;
	strncpy(work, c, len);
	work[len] = '\0';
	entry_top = add_entry(entry_top, work);
}

	int
nusdir_generic(struct nus_dims_t *dp, struct nusdir_opts *op)
{
	int	r;
	r = nusdas_list_dims(dp, (void *)op, iprint, cprint, op->debug);
	flush_entries(entry_top);
	return r;
}

/*
 * === NuSDaS Version 1 p鋌[` ===
 */

#define MAX_BASETIMES 1000

	int
nusdir_base(struct nus_dims_t *dp, struct nusdir_opts *op)
{
	struct nus_type_t	*sp;
	int	i, r;
	long	siz, n, gridsize[2];
	long	*longbuf;

	sp = &dp->type;

	longbuf = malloc(MAX_BASETIMES * sizeof(long));
	if (longbuf == NULL)
		return 1;
	siz = MAX_BASETIMES;
	r = nusdas_inq_nrdbtime(sp->type1, sp->type2, sp->type3, longbuf,
		&siz, (op->longe ? N_ON : N_OFF));
	if (r < 0) {
		eprintf(HTTP_ERR, "nusdas_inq_nrdbtime %d\n", r);
		return r;
	}

	for (i = 0; i < r; i++) {
		const char *p = minute_to_str(longbuf[i], NULL, 0);
		if (p == NULL)
			continue;
		if (strncmp(p, "min", 3) == 0)
			errprintf("basetime %s\n", p);
		else if (longbuf[i] < 31536000L)
			errprintf("basetime %ld\n", longbuf[i]);
		else
			tprintf(0, "%s\n", p);
	}
	free(longbuf);
	
	return 0;
}

	int
nusdir_member(struct nus_dims_t *dp, struct nusdir_opts *op)
{
	struct nus_type_t	*sp;
	int	i, r;
	long	siz, n, gridsize[2];
	char	*charbuf;

	sp = &dp->type;

	siz = 1;
	r = nusdas_inq_def(sp->type1, sp->type2, sp->type3, N_MEMBER_NUM,
		&n, &siz);
	if (r < 0) return r;
	charbuf = malloc(n * 4);
	r = nusdas_inq_def(sp->type1, sp->type2, sp->type3, N_MEMBER_LIST,
		charbuf, &n);
	if (r < 0) return r;

	for (i = 0; i < n; i++) {
		char *p;
		p = charbuf + (4 * i);
		if (strncmp(p, "    ", 4) == 0) p = "none";
		tprintf(0, "%.4s\n", p);
	}
	free(charbuf);

	return 0;
}

	int
nusdir_valid(struct nus_dims_t *dp, struct nusdir_opts *op)
{
	struct nus_type_t	*sp;
	int	i, r;
	long	siz, n, gridsize[2];
	char	*charbuf;
	long	*longbuf;

	sp = &dp->type;
	longbuf = malloc(MAX_BASETIMES * sizeof(long));
	if (longbuf == NULL)
		return 1;
	siz = MAX_BASETIMES;
	r = nusdas_inq_nrdvtime(sp->type1, sp->type2, sp->type3, longbuf,
		&siz, &dp->base, (op->debug ? N_ON : N_OFF));
	if (r < 0) {
		eprintf(HTTP_ERR, "nusdas_inq_nrdvtime %d\n", r);
		return r;
	}

	for (i = 0; i < r; i++) {
		const char *p = minute_to_str(longbuf[i], NULL, 0);
		if (p == NULL)
			continue;
		if (strncmp(p, "min", 3) == 0)
			errprintf("validtime %s\n", p);
		else if (longbuf[i] < 31536000L)
			errprintf("validtime %ld\n", longbuf[i]);
		else
			tprintf(0, "%s\n", p);
	}
	free(longbuf);

	return 0;
}

	int
nusdir_valid2(struct nus_dims_t *dp, struct nusdir_opts *op)
{
	tprintf(0, "min1\n");
}

	int
nusdir_plane(struct nus_dims_t *dp, struct nusdir_opts *op)
{
	struct nus_type_t	*s;
	int	i, r;
	long	siz, n, gridsize[2];
	char	*charbuf;

	s = &dp->type;
	siz = 1;
	r = nusdas_inq_cntl(s->type1, s->type2, s->type3, &(dp->base),
		dp->member, &(dp->valid), N_PLANE_NUM, &n, &siz);
	if (r < 0)
		return r;
	charbuf = malloc(n * 6);
	r = nusdas_inq_cntl(s->type1, s->type2, s->type3, &(dp->base),
		dp->member, &(dp->valid), N_PLANE_LIST, charbuf, &n);
	if (r < 0)
		return r;

	for (i = 0; i < n; i++) {
		char *p;
		p = charbuf + (6 * i);
		if (strncmp(p, "      ", 6) == 0) p = "none";
		tprintf(0, "%.6s\n", p);
	}
	free(charbuf);
	return 0;
}

	int
nusdir_plane2(struct nus_dims_t *dp, struct nusdir_opts *op)
{
	struct nus_type_t	*s;
	int	i, r;
	long	siz, n, gridsize[2];
	char	*charbuf;

	if (memcmp(dp->plane, "      ", 6) == 0) {
		s = &dp->type;
		siz = 1;
		r = nusdas_inq_cntl2(s->type1, s->type2, s->type3, &(dp->base),
			dp->member, &(dp->valid), &(dp->valid2), N_PLANE_NUM, 
			&n, &siz);
		if (r < 0)
			return r;
		charbuf = malloc(n * 6);
		r = nusdas_inq_cntl2(s->type1, s->type2, s->type3, &(dp->base),
			dp->member, &(dp->valid), &(dp->valid2), N_PLANE2_LIST,
			 charbuf, &n);
		if (r < 0)
			return r;

		for (i = 0; i < n; i++) {
			char *p;
			p = charbuf + (6 * i);
			if (strncmp(p, "      ", 6) == 0) p = "none";
			tprintf(0, "%.6s\n", p);
		}
		free(charbuf);
	} else {
		tprintf(0, "%.6s\n", dp->plane);
	}
	return 0;
}

	int
nusdir_element(struct nus_dims_t *dp, struct nusdir_opts *op)
{
	struct nus_type_t	*s;
	int		i, r;
	long		siz, nelem, nmember, nvalid, nplane;
	long		imember, ivalid, iplane;
	char		*elembuf, *mapbuf, *charbuf;
	long		*longbuf;

	s = &(dp->type);

	siz = 1;
	r = nusdas_inq_def(s->type1, s->type2, s->type3,
		N_ELEMENT_NUM, &nelem, &siz);
	if (r < 1) return r;

	r = nusdas_inq_def(s->type1, s->type2, s->type3, N_PLANE_NUM,
		&nplane, &siz);
	if (r < 1) return r;
	if ((charbuf = malloc(nplane * 6)) == NULL)
		return -2;
	r = nusdas_inq_def(s->type1, s->type2, s->type3, N_PLANE_LIST,
		charbuf, &nplane);
	iplane = -1;
	for (i = 0; i < nplane; i++) {
	    if (memcmp(charbuf + 6 * i, dp->plane, 6) == 0) {
		iplane = i;
		break;
	    }
	}
	free(charbuf);
	if (iplane == -1) {
	    eprintf(HTTP_ERR, "plane %.6s not found\n", dp->plane);
	    return -3;
	}

	r = nusdas_inq_def(s->type1, s->type2, s->type3, N_MEMBER_NUM,
		&nmember, &siz);
	if (r < 1) return r;
	if ((charbuf = malloc(nmember * 4)) == NULL)
		return -2;
	r = nusdas_inq_def(s->type1, s->type2, s->type3, N_MEMBER_LIST,
		charbuf, &nmember);
	imember = -1;
	for (i = 0; i < nmember; i++) {
	    if (memcmp(charbuf + 4 * i, dp->member, 4) == 0) {
		imember = i;
		break;
	    }
	}
	free(charbuf);
	if (imember == -1) {
	    eprintf(HTTP_ERR, "member %.4s not found\n", dp->member);
	    return -3;
	}

	r = nusdas_inq_def(s->type1, s->type2, s->type3,
	    N_VALIDTIME_NUM, &nvalid, &siz);
	if (r < 1) return r;
	/* dirty sigsegv guard + 256 */
	if ((longbuf = malloc(nvalid * 4 + 256)) == NULL)
		return -2;
	r = nusdas_inq_nrdvtime(s->type1, s->type2, s->type3,
	    longbuf, &nvalid, &(dp->base), N_OFF);
	if (r < 1) return r;
	ivalid = -1;
	for (i = 0; i < nvalid; i++) {
	    if (longbuf[i] == dp->valid) {
		ivalid = i;
		break;
	    }
	}
	free(longbuf);
	if (ivalid == -1) {
	    eprintf(HTTP_ERR, "validtime %d not found\n", dp->valid);
	}

	siz = nelem * nplane * nmember * nvalid;
	mapbuf = malloc(siz);
	if (mapbuf == NULL) return 1;
	r = nusdas_inq_def(s->type1, s->type2, s->type3, N_ELEMENT_MAP,
		mapbuf, &siz);
	if (r < 0) return r;

	elembuf = malloc(nelem * 6 + 1);
	if (elembuf == NULL) return 1;
	r = nusdas_inq_cntl(s->type1, s->type2, s->type3,
		&(dp->base), dp->member, &(dp->valid),
		N_ELEMENT_LIST, elembuf, &nelem);
	if (r < 0) return r;
	elembuf[nelem * 6] = '\0';

	for (i = 0; i < nelem; i++) {
		char *p, *q;
		q = mapbuf + i + nelem * (iplane + nplane *
			(ivalid + nvalid * imember));
		if (!*q) continue;
		p = elembuf + (6 * i);
		if (strncmp(p, "      ", 6) == 0) p = "none";
		tprintf(0, "%.6s\n", p);
	}
	free(elembuf);
	free(mapbuf);
	return 0;
}
