/* nusdump.c
 * 2001-08-31 TOYODA Eizi
 */

#include <stdio.h>
#include <stdlib.h> /* for malloc */
#include <string.h>
#include <nusdas.h>
#include "nusdim.h"
#include "textout.h"

static char rcsid[] = "$Id: nusdump.c,v 1.9 2003/03/07 02:30:37 tabito Exp $";

struct nusdump_opts {
	int (*handler)(struct grids_t *gp, struct nusdump_opts *op);
        int (*grid_getter)(struct nus_dims_t *dp, struct grids_t *gp);
	int	pnm;
};

	int
grids_dump_native(struct grids_t *gp, struct nusdump_opts *op)
{
	unsigned long	siz;
	size_t		elemsiz;
	void		*buffer;
	if (gp->ibuf != NULL) {
		tset_content_type("application/x-int32-stream");
		elemsiz = sizeof(N_SI4);
		buffer = gp->ibuf;
	} else if (gp->fbuf != NULL) {
		tset_content_type("application/x-float32-stream");
		elemsiz = sizeof(float);
		buffer = gp->fbuf;
	} else if (gp->dbuf != NULL) {
		tset_content_type("application/x-float64-stream");
		elemsiz = sizeof(double);
		buffer = gp->dbuf;
	} else {
		eprintf(HTTP_ERR, "error: null pointer buffer");
		return -1;
	}
	siz = (unsigned long)gp->xysize[0] * (unsigned long)gp->xysize[1];
	twrite(buffer, elemsiz, siz);
	return 0;
}

	int
grids_convert_float32(struct grids_t *gp)
{
	unsigned long siz, ofs;

	if (gp->fbuf != NULL)
		return 0;
	siz = (unsigned long)gp->xysize[0] * (unsigned long)gp->xysize[1];
	gp->fbuf = malloc(siz * sizeof(float));
	if (gp->ibuf != NULL) {
		tprintf("notice", "converted from int32 to float32");
		for (ofs = 0; ofs < siz; ofs++) {
			gp->fbuf[ofs] = gp->ibuf[ofs];
		}
	} else if (gp->dbuf != NULL) {
		tprintf("notice", "converted from float64 to float32");
		for (ofs = 0; ofs < siz; ofs++) {
			gp->fbuf[ofs] = gp->dbuf[ofs];
		}
	} else {
		return -1;
	}
	return 0;
}

	int
grids_dump_float32(struct grids_t *gp, struct nusdump_opts *op)
{
	int	r;
	unsigned long siz;
	r = grids_convert_float32(gp);
	if (r)
		return r;
	siz = (unsigned long)gp->xysize[0] * (unsigned long)gp->xysize[1];
	tset_content_type("application/x-float32-stream");
	twrite(gp->fbuf, sizeof(float), siz);
	return 0;
}

int
grids_dump_uint8(struct grids_t *gp, struct nusdump_opts *op)
{
    unsigned long siz;

    if(gp->ucbuf == NULL){
        eprintf(HTTP_ERR, "error: null pointer buffer <ucbuf>");
        return -1;
    }
    siz = (unsigned long)gp->xysize[0] * (unsigned long)gp->xysize[1];
    tset_content_type("application/x-uint8-stream");
    twrite(gp->ucbuf, sizeof(char), siz);
    return 0;
}

int
grids_dump_rlen8(struct grids_t *gp, struct nusdump_opts *op)
{
    unsigned long siz;
    int cmp_size;
    int maxv;
    unsigned char *tmp_alloc;
    int nbit = 8;

    struct sized_uchar{
        unsigned char *data;
        int alloc_size;
    } sd;

    if(gp->ucbuf == NULL){
        eprintf(HTTP_ERR, "error: null pointer buffer <ucbuf>");
        return -1;
    }
    siz = (unsigned long)gp->xysize[0] * (unsigned long)gp->xysize[1];
    tset_content_type("application/x-rlen8-stream");
    
    sd.alloc_size = 300000;
    sd.data = (unsigned char*)malloc(sd.alloc_size * sizeof(sd.data));
    if(sd.data == NULL){
        return -32;
    }

    maxv = -1;
    while(1){
        cmp_size = n_encode_rlen_8bit_I1(gp->ucbuf, sd.data, siz, 
                                      sd.alloc_size,&maxv);
        if(cmp_size == -1){
            sd.alloc_size += 100000;
            tmp_alloc = (unsigned char*)malloc(sd.alloc_size*sizeof(sd.data));
            if(tmp_alloc == NULL){
                free(sd.data);
                return -32;
            }
            else{
                sd.data = tmp_alloc;
            }
        }
        else if(cmp_size == -2){
            eprintf(HTTP_ERR, "error: some data is out of range ");
            free(sd.data);
            return -2;
        }
        else{
            break;
        }
    }
    twrite(&nbit, sizeof(int), 1);
    twrite(&maxv, sizeof(int), 1);
    twrite(&cmp_size, sizeof(int), 1);
    twrite(sd.data, sizeof(char), cmp_size);
    free(sd.data);
    return 0;
}

int
grids_dump_int32(struct grids_t *gp, struct nusdump_opts *op)
{
    unsigned long siz;

    if(gp->ibuf == NULL){
        eprintf(HTTP_ERR, "error: null pointer buffer <ibuf>");
        return -1;
    }
    siz = (unsigned long)gp->xysize[0] * (unsigned long)gp->xysize[1];
    tset_content_type("application/x-int32-stream");
    twrite(gp->ibuf, sizeof(int), siz);
    return 0;
}

int
grids_dump_double64(struct grids_t *gp, struct nusdump_opts *op)
{
    unsigned long siz;

    if(gp->dbuf == NULL){
        eprintf(HTTP_ERR, "error: null pointer buffer <ibuf>");
        return -1;
    }
    siz = (unsigned long)gp->xysize[0] * (unsigned long)gp->xysize[1];
    tset_content_type("application/x-double64-stream");
    twrite(gp->dbuf, sizeof(double), siz);
    return 0;
}



	int
grids_dump_pnm(struct grids_t *gp, struct nusdump_opts *op)
{
	int	r;
	unsigned long i, n;
	float	min, max, scale;
	char *buffer;

	r = grids_convert_float32(gp);
	if (r)
		return r;
	n = (unsigned long)gp->xysize[0] * (unsigned long)gp->xysize[1];

	max = -1.0e10f;
	min = 1.0e10f;
        for (i = 0; i < n; i++) {
		if (gp->fbuf[i] < min) min = gp->fbuf[i];
		if (gp->fbuf[i] > max) max = gp->fbuf[i];
	}
	scale = max - min;
	if (scale == 0.0f)
		scale = 1.0f;
	else
		scale = scale / 255.0f;

	tprintf("value-max", "%g", max);
	tprintf("value-min", "%g", min);
	tprintf("gradation-step", "%g", scale);

	if (op->pnm == 6) {
		tset_content_type("image/x-portable-pixmap; opt=binary");

		buffer = malloc(n * 3);
		tprintf(0, "P6\n%d %d\n255 255 255\n", gp->xysize[0], gp->xysize[1]);
		for (i = 0; i < n; i++) {
			buffer[i * 3 + 0] = (unsigned char)(256 - (gp->fbuf[i] - min) / scale);
			buffer[i * 3 + 1] = (unsigned char)((gp->fbuf[i] - min) / scale);
			buffer[i * 3 + 2] = (unsigned char)((gp->fbuf[i] - min) / scale);
		}
		twritemem(buffer, 1, n * 3);
	} else if (op->pnm == 5) {
		tset_content_type("image/x-portable-graymap");
		buffer = malloc(n);
		tprintf(0, "P5\n%d %d\n255\n", gp->xysize[0], gp->xysize[1]);
		for (i = 0; i < n; i++) {
			buffer[i] = (unsigned char)((gp->fbuf[i] - min) / scale);
		}
		twritemem(buffer, 1, n);
	} else {
		tset_content_type("image/x-portable-graymap; opt=text");

		tprintf(0, "P2\n%d %d\n255\n", gp->xysize[0], gp->xysize[1]);
		for (i = 0; i < n; i++) {
			tprintf(0, "%d\n", (unsigned char)((gp->fbuf[i] - min) / scale));
		}
	}
	return 0;
}

	int
grids_dump_printf(struct grids_t *gp, struct nusdump_opts *op)
{
	int	i, j, ofs;

	tset_content_type("text/plain");

	for (j = 0; j < gp->xysize[1]; j++) {
		for (i = 0; i < gp->xysize[0]; i++) {
			ofs = i + gp->xysize[0] * j;
			if (gp->ibuf) {
				tprintf(0, " %d ", gp->ibuf[ofs]);
			} else if (gp->fbuf) {
				tprintf(0, " %.7g", gp->fbuf[ofs]);
			} else if (gp->dbuf) {
				tprintf(0, " %.16g", gp->dbuf[ofs]);
			} else {
				return -1;
			}
		}
		tprintf(0, "\n");
	}
	return 0;
}

	void
opt_clear(struct nusdump_opts *op)
{
	op->handler = grids_dump_native;
        op->grid_getter = nusdims_get_grids;
	op->pnm = 0;
}

	void
option(struct nusdump_opts *op, const char *str)
{
	if (str == NULL)
		return;
	if (*str == '-')
		str++;
	switch (*str) {
	case 'r':
		if (str[1]) {
			nusdas_addroot(str+1);
		} else {
			set_dbdir();
		}
		break;
	case 't':
		if (str[1] == 'f') {
			op->handler = grids_dump_float32;
		} else if (str[1] == 'p') {
			op->handler = grids_dump_printf;
		} else if (str[1] == 'u') { /* unsigned char */
                    op->handler = grids_dump_uint8;
                    op->grid_getter = nusdims_get_grids_uint8;
                } else if (str[1] == 'i') { /* int */
                    op->handler = grids_dump_int32;
                    op->grid_getter = nusdims_get_grids_int32;
                } else if (str[1] == 'r') { /* rlen 8bit */
                    op->handler = grids_dump_rlen8;
                    op->grid_getter = nusdims_get_grids_uint8;
                } else if (str[1] == 'd'){ /* double */
                    op->handler = grids_dump_double64;
                    op->grid_getter = nusdims_get_grids_double64;
                }
		break;
	case 'p':
	case 'P':
		if (str[1] == 'G') {
			op->handler = grids_dump_pnm;
			op->pnm = 2;
		} else if (str[1] == 'g') {
			op->handler = grids_dump_pnm;
			op->pnm = 5;
		} else if (str[1] == 'p') {
			op->handler = grids_dump_pnm;
			op->pnm = 6;
		}
		break;
	default:
		break;
	}
}

/* dp Ŏw肳 NuSDaS ʂ_v */

	int
nusdims_dump(struct nus_dims_t *dp, struct nusdump_opts *op)
{
	struct grids_t g;
	int	r;
	r = op->grid_getter(dp, &g);
	if (r <= 0)
		eprintf(HTTP_ERR, "nusdims_get_grids %d\n", r);
	if (r == 0)
		r = -1;
	if (r < 0)
		return r;

	r = op->handler(&g, op);
	if (r < 0)
		eprintf(HTTP_ERR, "nusdims_dump %d\n", r);
	return r;
}

    int
main(int argc, char **argv)
{
	struct nus_dims_t dims;
	struct nusdump_opts opt;
	int	argind;
	int	r;

	text_init("text/plain");
	if (argv[1] == NULL) {
		eprintf(HTTP_ERR, "usage: %s <path>\n\n", argv[0]);
		text_end();
		return 1;
	}
	opt_clear(&opt);
	for (argind = 1; argv[argind] != NULL; argind++) {
		if (argv[argind][0] == '-') {
			option(&opt, argv[argind]);
			continue;
		}
		r = path_to_nusdims(argv[argind], &dims);
		if (!(r & NUSDIM_ELEMENT)) {
			eprintf(HTTP_ERR, "element not specified\n");
			tprintf("resolved-path", "0x%x %s\n", r, nusdims_to_path(&dims));
			r = 2;
			break;
		}
		r = nusdims_dump(&dims, &opt);
		if (r != 0) {
			r = 3;
			break;
		}
		r = 0;
	}
	text_end();
	return r;
}
