/* nusmeta.c */

#include <stdio.h>
#include <stdlib.h> /* for malloc */
#include <string.h>
#include <nwpl_string.h>
#include <nusdas.h>
#include "textout.h"
#include "nusdim.h"

static char rcsid[] = "$Id: nusmeta.c,v 1.3 2002/03/27 10:13:39 suuchi43 Exp $";

	int
nusdims_meta(struct nus_dims_t *dp)
{
	int	r;
	char	proj[4], pvalp[4];
	N_SI4	gridsize[2];
	float	grid[7][2];
	
	r = nusdas_grid2(dp->type.type1, dp->type.type2, dp->type.type3,
		&dp->base, dp->member, &dp->valid, &dp->valid2,
		proj, gridsize, (float *)grid, pvalp, N_IO_GET);
	if (r < 0) {
		eprintf(HTTP_ERR, "nusdas error %d\n", r);
		return r;
	}
	tagopen("html", NULL);
	tagopen("head", NULL);
	tagopen("title", NULL);
	tprintf(NULL, "meta for %s", nusdims_to_path(dp));
	tagclose("title");	
	tagclose("head");
	tputs("\r\n", NULL);

	tagopen("body", NULL);
	tagopen("table", "border", NULL);

#define TABLE_ROW(cname, desc, format, value) \
	tagopen("tr", NULL); \
	tagopen("tr", NULL); \
	tagopen("td", "name", cname, NULL); \
	tputs(desc, NULL); \
	tagclose("td"); \
	tagopen("td", NULL); \
	tprintf(NULL, format, value); \
	tagclose("td"); \
	tagclose("tr"); tputs("\r\n", NULL);

	TABLE_ROW("proj_type", "projection type\t", "%4.4s", proj);
	TABLE_ROW("xs", "number of x grids\t", "%d", gridsize[0]);
	TABLE_ROW("ys", "number of y grids\t", "%d", gridsize[1]);
	TABLE_ROW("x0", "base point x\t", "%g", grid[0][0]);
	TABLE_ROW("y0", "base point y\t", "%g", grid[0][1]);
	TABLE_ROW("lat_0", "base point lat\t", "%g", grid[1][0]);
	TABLE_ROW("lon_0", "base point lon\t", "%g", grid[1][1]);
	TABLE_ROW("xd", "grid interval x\t", "%g", grid[2][0]);
	TABLE_ROW("yd", "grid interval y\t", "%g", grid[2][1]);
	TABLE_ROW("lat_b1", "standard lat 1\t", "%g", grid[3][0]);
	TABLE_ROW("lon_b1", "standard lon 1\t", "%g", grid[3][1]);
	TABLE_ROW("lat_b2", "standard lat 2\t", "%g", grid[4][0]);
	TABLE_ROW("lon_b2", "standard lon 2\t", "%g", grid[4][1]);
	TABLE_ROW("lat_1", "latitude 1\t", "%g", grid[5][0]);
	TABLE_ROW("lon_1", "longitude 1\t", "%g", grid[5][1]);
	TABLE_ROW("lat_2", "latitude 2\t", "%g", grid[6][0]);
	TABLE_ROW("lon_2", "longitude 2\t", "%g", grid[6][1]);
	TABLE_ROW("pval", "time operation\t", "%4.4s", pvalp);

	tagclose("table");
	tagclose("body");
	tagclose("html");
	tputs("\r\n", NULL);
	return 0;
}

    int
main(int argc, char **argv)
{
	struct nus_dims_t dims;
	int	argind;
	int	r;
	if (argv[1] == NULL) {
		eprintf(HTTP_ERR, "usage: %s <path>\n", argv[0]);
		text_end();
		return 1;
	}
	text_init("text/html");
	for (argind = 1; argv[argind] != NULL; argind++) {
		if (streq(argv[argind], "-r")) {
			set_dbdir();
			continue;
		} else if (streq(argv[argind], "-t")) {
			tset_content_type("text/plain");
			continue;
		} else if (argv[argind][0] == '-') {
			continue;
		}
		r = path_to_nusdims(argv[argind], &dims);
		if (!(r & NUSDIM_VALID)) {
			eprintf(HTTP_ERR, "validtime not specified\n");
			tprintf("resolved-path", "0x%x %s\n", r, nusdims_to_path(&dims));
			r = 2;
			break;
		}
		r = nusdims_meta(&dims);
		if (r != 0) {
			r = 3;
			break;
		}
		r = 0;
	}
	text_end();
	return r;
}
