/* nusmeta.c */

#include <stdio.h>
#include <stdlib.h> /* for malloc */
#include <string.h>
#include <nwpl_string.h>
#include <nusdas.h>
#include "textout.h"
#include "nusdim.h"

static char rcsid[] = "$Id: nusschema.c,v 1.5 2002/04/05 11:48:32 suuchi43 Exp $";

	int
nusdims_schema(struct nus_dims_t *dp)
{
	int	r;
	N_SI4	n;
	char	cbuf[5];
	char	*namebuf;
	struct nus_dims_t	qdims;
	struct nus_dims_t	*dimslist;
	int	i;
	int	v2used;
	int	tunit;
	unsigned char	*elemlist;

	tputs("# element data type: application/x-float32-stream\n", NULL);
	tputs("2\n", NULL);

	r = nusdas_inq_bydims(dp, N_VALIDTIME_UNIT, cbuf, 1);
	if (r != 1)
		goto Error;
	cbuf[4] = '\0';
	tprintf(0, "# units of fcst/valid time: %.4s\n", cbuf);
	if (memcmp(cbuf, "MIN ",  4) == 0) {
		tputs("0\n", NULL);
		tunit = 1;
	} else if (memcmp(cbuf, "HOUR",  4) == 0) {
		tputs("1\n", NULL);
		tunit = 60;
	} else if (memcmp(cbuf, "DAY ",  4) == 0) {
		tputs("2\n", NULL);
		tunit = 60 * 24;
	} else {
		eprintf(0, "%.4s cannot be encoded\n", cbuf);
		goto Error;
	}

	qdims = *dp;
	qdims.stat = NUSDIM_TYPE;
	r = nusdas_getlist_bydims(&qdims, NUSDIM_BASE,
		&dimslist, N_ON);
	if (r < 0)
		goto Error;
	if (dp->stat & NUSDIM_BASE) {
		qdims.base = dp->base;
		tprintf(0, "# specified base = %s\n",
			minute_to_str(qdims.base, NULL, 0));
	} else {
		if (r == 1) {
			qdims.base = dimslist[0].base;
			tprintf(0, "# last base = %s\n",
				minute_to_str(qdims.base, NULL, 0));
		} else {
			qdims.base = dimslist[r - 2].base;
			tprintf(0, "# 2nd last base = %s\n",
				minute_to_str(qdims.base, NULL, 0));
		}
	}
	qdims.stat |= NUSDIM_BASE;
	free(dimslist);

	if (!(dp->stat & NUSDIM_MEMBER)) {
		r = nusdas_getlist_bydims(&qdims, NUSDIM_MEMBER,
			&dimslist, N_ON);
		if (r < 1)
			goto Error;
		if (r > 1) {
			r = NUSERR_NOTUNIQUE;
			free(dimslist);
			goto Error;
		}
		tprintf(NULL, "# member set to [%.4s]\n", dimslist[0].member);
		memcpy(qdims.member, dimslist[0].member, 4);
		qdims.stat |= NUSDIM_MEMBER;
		free(dimslist);
	}

	r = nusdas_getlist_bydims(&qdims, (NUSDIM_VALID | NUSDIM_VALID2),
		&dimslist, N_ON);
	if (r < 1)
		goto Error;
	v2used = 0;
	for (i = 0; i < r; i++) {
		if (dimslist[i].valid2 != 1)
			v2used = 1;
	}
	if (v2used) {
		tputs("# valid2 is used\n", NULL);
		tputs("2\n", NULL);
	} else {
		tputs("# valid2 is NOT used\n", NULL);
		tputs("1\n", NULL);
	}

	tprintf(NULL, "# number of fcsttime\n%d\n", r);

	tputs("# list of fcsttime-1\n", NULL);
	for (i = 0; i < r; i++) {
		int	ft;
		ft = (dimslist[i].valid - qdims.base) / tunit;
		tprintf(NULL, "%4d ", ft);
	}
	tputs("\n", NULL);

	tputs("# list of fcsttime-2\n", NULL);
	for (i = 0; i < r; i++) {
		int	ft;
		ft = (dimslist[i].valid2 - qdims.base) / tunit;
		tprintf(NULL, "%4d ", (v2used ? ft : 999));
	}
	tputs("\n", NULL);

	free(dimslist);

	r = nusdas_elemlist_ladess(&qdims, &elemlist, N_ON);
	if (r < 0)
		goto Error;
	tputs("# number of elements\n", NULL);
	tprintf(NULL, "%d\n", r);

	for (i = 0; i < r; i++) {
		unsigned char	*p;
		p = elemlist + 4 * i;
		tprintf(NULL, "%5d %5d %5d", p[1], p[0],
			((int)p[2] * 256 + p[3]));
	}
	tputs("\n", NULL);

	tputs("# data type in detail: 4 bytes real(2)\n", NULL);
	tputs("4\n2\n", NULL);

	return 0;

Error:
	eprintf(HTTP_ERR, "%d, %s\n", r, nusdas_strerror(r));
	return r;
}

    int
main(int argc, char **argv)
{
	struct nus_dims_t dims;
	int	argind;
	int	r;
	if (argv[1] == NULL) {
		eprintf(HTTP_ERR, "usage: %s <path>\n", argv[0]);
		text_end();
		return 1;
	}
	text_init("text/plain");
	for (argind = 1; argv[argind] != NULL; argind++) {
		if (streq(argv[argind], "-r")) {
			set_dbdir();
			continue;
		} else if (streq(argv[argind], "-d")) {
			tset_content_type(NULL);
			continue;
		} else if (argv[argind][0] == '-') {
			continue;
		}
		path_to_nusdims(argv[argind], &dims);
		r = nusdims_schema(&dims);
		if (r != 0) {
			r = 3;
			break;
		}
		r = 0;
	}
	text_end();
	return r;
}
