/*
 * binarytree.c: 񕪖؃\[g[`
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "nusdas.h"
#include "binarytree.h"

int bt_cmp_dims = ~0;

	static int
bt_cmpdims(const struct nus_dims_t *lhs, const struct nus_dims_t *rhs,
	const int flags)
{
	int	r;
	if (flags & NUSDIM_TYPE) {
		r = memcmp(&(lhs->type), &(rhs->type),
			sizeof(struct nus_type_t));
		if (r)
			return r;
	}
	if (flags & NUSDIM_BASE) {
		if ((r = lhs->base - rhs->base) != 0)
			return r;
	}
	if (flags & NUSDIM_MEMBER) {
		if ((r = memcmp(lhs->member, rhs->member, 4)) != 0)
			return r;
	}
	if (flags & NUSDIM_VALID) {
		if ((r = lhs->valid - rhs->valid) != 0)
			return r;
	}
	if (flags & NUSDIM_VALID2) {
		if ((r = lhs->valid2 - rhs->valid2) != 0)
			return r;
	}
	if (flags & NUSDIM_PLANE) {
		if ((r = memcmp(lhs->plane, rhs->plane, 4)) != 0)
			return r;
	}
	if (flags & NUSDIM_PLANE2) {
		if ((r = memcmp(lhs->plane2, rhs->plane2, 4)) != 0)
			return r;
	}
	if (flags & NUSDIM_ELEMENT) {
		if ((r = memcmp(lhs->element, rhs->element, 4)) != 0)
			return r;
	}
	return 0;
}

	static int
bt_compare(btree_sort sort, const void *lhs, const void *rhs)
{
	switch (sort) {
		case BS_CHAR4:
		case BS_CHAR6:
		case BS_CHAR16:
			return memcmp(lhs, rhs, (int)sort);
			break;
		case BS_NWPTIME:
			if (*(N_SI4 *)lhs > *(N_SI4 *)rhs)
				return 1;
			else if (*(N_SI4 *)lhs < *(N_SI4 *)rhs)
				return -1;
			else
				return 0;
			break;
		case BS_DIMS:
			return bt_cmpdims(lhs, rhs, bt_cmp_dims);
			break;
		default:
			return NUSERR_INTERNAL;
	}
}

	static size_t
bt_copy(btree_sort sort, void *dest, const void *src)
{
	switch (sort) {
		case BS_CHAR4:
		case BS_CHAR6:
		case BS_CHAR16:
			memcpy(dest, src, (int)sort);
			return (int)sort;
			break;
		case BS_NWPTIME:
			*(N_SI4 *)dest = *(N_SI4 *)src;
			return sizeof(N_SI4);
			break;
		case BS_DIMS:
			*(struct nus_dims_t *)dest = *(struct nus_dims_t *)src;
			return sizeof(struct nus_dims_t);
			break;
		default:
			/* 肦Ȃ */
			return 0;
	}
}

	binary_tree_t *
btree_add(binary_tree_t *p, btree_sort sort, const void *data)
{
	int cond;

	if (p == NULL) {
		p = (binary_tree_t *)malloc(sizeof(binary_tree_t));
		if (p == NULL)
			return NULL;
		p->left = p->right = NULL;
		bt_copy(sort, p->buf, data);
		return p;
	}

	cond = bt_compare(sort, data, p->buf);
	if (cond < 0) {
		p->left  = btree_add(p->left, sort, data);
	} else if (cond > 0) {
		p->right = btree_add(p->right, sort, data);
	}
	return p;
}

	size_t
btree_count(binary_tree_t *p)
{
	size_t	n;
	if (p == NULL)
		return 0;
	n = 1;
	n += btree_count(p->left);
	n += btree_count(p->right);
	return n;
}

	int
btree_dump(binary_tree_t *p, btree_sort sort, char *buf)
{
	int	r;
	size_t	n;

	if (p == NULL)
		return 0;

	r = btree_dump(p->left, sort, buf);
	n = bt_copy(sort, buf, p->buf);
	buf += n;
	if (n == 0)
		return NUSERR_INTERNAL;
	r |= btree_dump(p->right, sort, buf);
	free(p);

	return r;
}

	void
btree_dispose(binary_tree_t *p)
{
	int	r;
	if (p == NULL)
		return;
	btree_dispose(p->left);
	btree_dispose(p->right);
	free(p);
}
