#include <stdio.h>
#include <stdlib.h>
#include "nusdas.h"
#include "nwpl_string.h"
#include "binarytree.h"

/* this has definitions of elemtab_t and elemtab */
#include "dgrb_elem.h"

	int
nusdas_dims2laelem(const struct nus_dims_t *dimsp,
	unsigned char *cbuf)
{
	int	elemfound;
	struct elemtab_t *e;
	char	plane[7], plane2[7];

	for (elemfound = 0, e = elemtab; e->nuselem[0]; e++) {
		if (memeqi(e->nuselem, dimsp->element, 6)) {
			elemfound = 1;
			break;
		}
	}
	if (!elemfound)
		return NUSERR_UNKNOWNELEM;

	fstring_to_string(plane, sizeof plane, dimsp->plane, 6);
	fstring_to_string(plane2, sizeof plane, dimsp->plane2, 6);

	if (memcmp(dimsp->type.type1 + 6, "PP", 2) == 0) {
		if (streqi(plane, plane2)) {
			cbuf[0] = 100;
			cbuf[1] = e->elemcode;
			cbuf[2] = atoi(plane) / 256;
			cbuf[3] = atoi(plane) % 256;
		} else {
			cbuf[0] = 101;
			cbuf[1] = e->elemcode;
			cbuf[2] = atoi(plane) / 10;
			cbuf[3] = atoi(plane2) / 10;
		}
	} else if (memcmp(dimsp->type.type1 + 6, "ET", 2) == 0) {
		cbuf[0] = 105;
		cbuf[1] = e->elemcode;
		cbuf[2] = atoi(plane); /* fake */
		cbuf[3] = 0;
	} else if (memcmp(dimsp->type.type1 + 4, "YPL0", 4) == 0) {
		return NUSERR_UNKNOWNELEM;
	} else if (memcmp(dimsp->type.type1 + 4, "STSF", 4) == 0) {
		return NUSERR_UNKNOWNELEM;
	} else {
		int	p;
		if (p = atoi(plane)) {
			cbuf[0] = 105;
			cbuf[1] = e->elemcode;
			cbuf[2] = p / 256;
			cbuf[3] = p % 256;
		} else if (memcmp(dimsp->element, "PSEA", 4) == 0) {
			cbuf[0] = 102;
			cbuf[1] = e->elemcode;
			cbuf[2] = 0; /* fake */
			cbuf[3] = 0;
		} else {
			cbuf[0] = 1;
			cbuf[1] = e->elemcode;
			cbuf[2] = 0; /* fake */
			cbuf[3] = 0;
		}
	}

	return 0;
}

	int
nusdas_elemlist_ladess(const struct nus_dims_t *dp,
	unsigned char	**elemlist,
	int	debug)
{
	int	r, n, i;
	struct nus_dims_t	*dimslist;
	struct binary_tree	*btree;

	n = r = nusdas_getlist_bydims(dp,
		(NUSDIM_PLANE | NUSDIM_PLANE2 | NUSDIM_ELEMENT),
		&dimslist, debug);
	if (r < 1)
		return r;

	btree = NULL;
	for (i = 0; i < n; i++) {
		unsigned char	cbuf[4];
		if (nusdas_dims2laelem(dimslist + i, cbuf) != 0)
			continue;
		btree = btree_add(btree, BS_CHAR4, cbuf);
	}

	free(dimslist);

	n = btree_count(btree);
	*elemlist = (char *)malloc(n * 4);
	if (*elemlist == NULL)
		return NUSERR_NOMEM;

	r = btree_dump(btree, BS_CHAR4, *elemlist);
	if (r)
		return r;

	return n;
}
