#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <ctype.h>
#include "nwpl_string.h"
#include "nusdas.h"

static char rcsid[] = "$Id: listing.c,v 1.6 2002/04/05 13:08:38 suuchi43 Exp $";

struct RootList {
        struct RootList *next;
        char            *value;
        DIR             *dirp;
};

int     nus_error = 0;

        struct RootList *
nus_roots() {
        struct RootList *roots, *cursor;
        int             root_no;

        cursor = roots = malloc(sizeof(struct RootList));
        cursor->value = strdup(".");
        cursor->next = NULL;
        cursor->dirp = NULL;
        for (root_no = 1; root_no <= 99; root_no++) {
                char    nusdasnn[9];
                char    *dirname, *env;
                dirname = NULL;
                sprintf(nusdasnn, "NUSDAS%2.2d", root_no);
                if ((env = getenv(nusdasnn)) != NULL) {
                        if (test_dir(env) > 0)
                                dirname = env;
                        else
                                tprintf("warning",
                                "${%s} = %s is not a directory\n",
                                nusdasnn, env);
                } else {
                        if (test_dir(nusdasnn) > 0)
                                dirname = nusdasnn;
                }
                if (!dirname)
                        continue;
                cursor->next = malloc(sizeof(struct RootList));
                cursor = cursor->next;
                cursor->value = strdup(dirname);
                cursor->next = NULL;
                cursor->dirp = NULL;
        }
        return roots;
}

/* `t@C type 𓾂B܂ 0 Ԃ */
	int
nus_gettype(const char *deffile, char *type1, char *type2, char *type3) {
	FILE	*input;
	char	line[256], head[12], w1[5], w2[3], w3[3];
	int	nwords;
	input = fopen(deffile, "rt");
	if (input == NULL) return EOF;
	strcpy(type1, "_XXXXXXX");
	strcpy(type2, "XXXX");
	strcpy(type3, "XXXX");
	while (fgets(line, sizeof line, input)) {
		nwords = sscanf(line, "%11s%4s%2s%2s", head, w1, w2, w3);
		if (streq(head, "type1") && (nwords >= 4)) {
			replace_space(w1, 4);
			replace_space(w2, 2);
			replace_space(w3, 2);
			sprintf(type1, "%4s%2s%2s", w1, w2, w3);
		}
		if (streq(head, "type2") && (nwords >= 3)) {
			replace_space(w1, 2);
			replace_space(w2, 2);
			sprintf(type2, "%2s%2s", w1, w2);
		}
		if (streq(head, "type3") && (nwords >= 2)) {
			replace_space(w1, 4);
			sprintf(type3, "%4s", w1);
		}
	}
	fclose(input);
	return 0;
}

	int
nus_list_inroot(struct RootList *nroot,
	char *type1, char *type2, char *type3, char **defnamep) {
	int		r;
	struct dirent	*entry;

	if (defnamep)
		*defnamep = NULL;
	if (nroot == NULL)
		return -11;
	if (nroot->dirp == NULL) {
		char	*defdir;
		defdir = strdup3(nroot->value, "/", "nusdas_def");
		if ((nroot->dirp = opendir(defdir)) == NULL) {
			return -12;
		}
		free(defdir);
	}

	for (;;) {
		char 	*deffile;
		entry = readdir(nroot->dirp);
		if (entry == NULL)
			return -13;
		if (entry->d_name[0] == '.')
			continue;
		deffile = strdup3(nroot->value, "/nusdas_def/", entry->d_name);
		r = nus_gettype(deffile, type1, type2, type3);
		if (r == 0) {
			if (defnamep) {
				*defnamep = deffile;
			} else {
				free(deffile);
			}
			return 0;
		}
		free(deffile);
	}
}

	static struct RootList	*roots = NULL;

/* type1 == NULL ̂Ƃ͒TZbg */
/* 0 ԂƂ type1, type2, type3 󎚉\ƕۏ؂ */
	int
nusdas_list_type(char *type1, char *type2, char *type3, char **defnamep) {
	static struct RootList	*cursor = NULL;

	if (defnamep)
		*defnamep = NULL;
	if (type1 == NULL || roots == NULL) {
		/* roots  NuSDaS [gXg\z */
		cursor = roots = nus_roots();
		return EOF;
	}

	if (cursor == NULL) return EOF;
	while (nus_list_inroot(cursor, type1, type2, type3, defnamep) != 0) {
		if (cursor->dirp) {
			closedir(cursor->dirp);
			cursor->dirp = NULL;
		}
		if (cursor->next == NULL) {
			return EOF;
		}
		cursor = cursor->next;
	}
	return 0;
}

/* 0 ԂƂ type1, type2, type3 󎚉\ƕۏ؂ */
	int
nusdas_find_root(const struct nus_type_t *tp, char **rootname) {
	struct RootList	*cursor;
	char xp[3][9] = {"fakellpp", "fake", "fake"};

	if (rootname)
		*rootname = NULL;
	/* roots  NuSDaS [gXg\z */
	if (roots == NULL) {
		roots = nus_roots();
		if (roots == NULL) 
			return -2;
	}

	for (cursor = roots; cursor; cursor = cursor->next) {
		int	r;
		while (1) {
			r = nus_list_inroot(cursor, xp[0], xp[1], xp[2], 0);
			if (r != 0)
				break;
			if (strncmp(tp->type1, xp[0], 8) == 0 &&
				strncmp(tp->type2, xp[1], 4) == 0 &&
				strncmp(tp->type3, xp[2], 4) == 0) {
				if (rootname) {
					*rootname =
					strdup3(cursor->value, 0, 0);
				}
				break;
			}
		}
		if (cursor->dirp) {
			closedir(cursor->dirp);
			cursor->dirp = NULL;
		}
		if (r == 0)
			return 0;
	}
	return -4;
}

/* stolen from nusdas_com.h */

#define N_DEFAULT 999

#define N_NWP_PATH_S    0
#define N_NWP_PATH_M    1
#define N_RELATIVE_PATH 2
#define N_NWP_ESF       3
#define N_NWP_PATH_BS   4
#define N_NWP_PATH_VM   5

#define N_INFOC_MAX     10
#define N_SUBC_MAX      10
#define N_LINE_MAX	256

#define PATH_LENGTH 1024

typedef struct {
        N_SI4   version;
        N_SI4   path;
        char    pathname[256], filename[256];
        char    type1[256], type2[256], type3[256];
        char    creator[256];
        N_SI4   basetime_t;
        N_SI4   infile_dc;
        N_SI4   infile_vt;
        N_UI4   time_unit;
        N_SI4   n_dc;           /* descriptor */
        N_SI4   n_vt;           /* valid time */
        N_SI4   n_lv;           /* level */
        N_SI4   n_el;           /* element */
        N_UI4   projection;     /* projection */
        N_SI4   xs, ys;         /* grid size */
        float   x0, y0;         /* base point coordinate */
        float   lat_0, lon_0;   /* base point latlon */
        float   xd, yd;         /* grid distance */
        float   lat_s1, lon_s1; /* first standard latlon */
        float   lat_s2, lon_s2; /* second standard latlon */
        float   lat_1, lon_1;   /* spare latlon1 */
        float   lat_2, lon_2;   /* spare latlon2 */
        N_UI4   gpv_mean;
        N_UI4   pack;
        N_UI4   miss;
        N_UI4    * dc;
        N_SI4    * vt1, *vt2;
        N_UI8    * lv1, *lv2;   /* use only 6 bytes */
        N_UI8    * el;          /* use only 6 bytes */
        N_UI1    * elmap;               /* elmap[n_dc][n_vt][n_lv][n_el] */
        N_SI4   n_infoc;
        N_UI4   info[N_INFOC_MAX];
        char    info_file[N_INFOC_MAX][256];
        char    grib_info_file[256];
        N_SI4   n_subc;
        N_UI4   subc[N_SUBC_MAX];
        N_SI4   subc_size[N_SUBC_MAX];
        N_SI4   forced_rlen;
} N_ST_DEF;

extern N_ST_DEF n_def[];
extern char n_nrd[][N_LINE_MAX];

N_SI4 n_set_defn(char *, char *, char *);
void n_copy_type(const char *, const char *, const char *, const char *,
                   char *, char *, char *, char *);
void n_make_subpath(char *buffer, size_t size, const char *dir,
                    const char *file);
void
replace_path(char *realized, size_t size, const char *template,
             const char *type1, const char *type2, const char *type3,
             const char *member, N_SI4 * basetime, int *unresolved_depth);

	char *
replace_path_bydims(const struct nus_dims_t *dp, char *rootname)
{
	char	ut1[9], ut2[5], ut3[5], umem[5];
	char	template[PATH_LENGTH];
	static char	realized[PATH_LENGTH];
	int	defn;
	int	unresolved_depth;

	n_copy_type(dp->type.type1, dp->type.type2, dp->type.type3,
		dp->member, ut1, ut2, ut3, umem);
	defn = n_set_defn(ut1, ut2, ut3);
	if (defn < 0) {
		return NULL;
	}

	/* t@CT template ̃ZbgAbv */
	switch (n_def[defn].path) {
	case N_NWP_PATH_S:
	case N_NWP_PATH_VM:
		n_make_subpath(template, PATH_LENGTH, n_nrd[defn],
		       "./_3D_name/_validtime");
	break;
	case N_NWP_PATH_M:
		n_make_subpath(template, PATH_LENGTH, n_nrd[defn],
		       "./_3D_name/_member/_validtime");
	break;
	case N_NWP_PATH_BS:
		n_make_subpath(template, PATH_LENGTH, n_nrd[defn],
		       "./_3D_name/_basetime");
	break;
	case N_RELATIVE_PATH:
		n_make_subpath(template, PATH_LENGTH, n_nrd[defn],
		       n_def[defn].pathname);
		n_make_subpath(template, PATH_LENGTH, NULL, n_def[defn].filename);
	break;
	case N_DEFAULT:
	default:
		n_make_subpath(template, PATH_LENGTH, n_nrd[defn],
		       "./_model/_attribute/_space/_time/_name/_basetime");
	break;
	}

        replace_path(realized, sizeof realized, template,
                     ut1, ut2, ut3, umem, (N_SI4 *)&(dp->base),
                     &unresolved_depth);
	errprintf("#replace [%s] [%s]\n", realized, template);
	return realized;
}
