#include <stdio.h>
#include <stdlib.h>
#include "nwpl_string.h"
#include "nusdas.h"
#include "recursive.h"

static char rcsid[] = "$Id: listing2.c,v 1.4 2002/04/03 11:16:35 suuchi43 Exp $";

	/* T̎w */
struct search {
	const struct nus_dims_t	*dimsp;
	int		target;
	void		*hint;
	int		 (*inthandler)(void *hint, N_SI4 i );
	int 		(*charhandler)(void *hint, char *c, size_t len);
	int		member_mismatch;
	int		valid_mismatch;
	int		debug;
};

	static int
check_cntl(const char *fnam, struct search *sp, const unsigned char *rec)
{
	N_SI4	base;

	if (sp->debug)
		errprintf("debug: search file [%s]\n", fnam);

	/* type uɗꍇ */
	if (sp->target == NUSDIM_TYPE) {
		(sp->charhandler)(sp->hint, (char *)rec + 12, 12);
		return RET_SKIPFILE;
	}

	/* basetime uɗꍇ */
	if (memcmp(rec + 12, (char *)&(sp->dimsp->type), 16) != 0) {
		return RET_BADDIR;
	}

	base = btol(rec + 40);
	if (sp->target == NUSDIM_BASE) {
		(sp->inthandler)(sp->hint, base);
		return RET_SKIPFILE;
	}

	/* member ȉ̂̂uɗꍇ */
	if (base != sp->dimsp->base) {
		return RET_SKIPFILE;
	}

	if (btol(rec + 48) == 1)
		sp->member_mismatch = RET_SKIPFILE;
	if (btol(rec + 52) == 1)
		sp->valid_mismatch = RET_SKIPFILE;

	return 0;
}

	static int
check_data(const struct search *sp, const unsigned char *rec)
{
	N_SI4 valid;
	N_SI4 valid2;

	if (sp->target == NUSDIM_MEMBER) {
		(sp->charhandler)(sp->hint, (char *)rec + 12, 4);
		return 0;
	}

	if (memcmp(rec + 12, sp->dimsp->member, 4) != 0)
		return sp->member_mismatch;

	valid = btol(rec + 16);
	if (sp->target == NUSDIM_VALID) {
		(sp->inthandler)(sp->hint, valid);		
		return 0;
	}

	if (sp->dimsp->valid != valid)
		return sp->valid_mismatch;

	valid2 = btol(rec + 20);
	if (sp->target == NUSDIM_VALID2) {
		(sp->inthandler)(sp->hint, valid2);
		return 0;
	}

	if (sp->dimsp->valid2 != valid2)
		return 0;

	if (sp->target == NUSDIM_PLANE) {
		(sp->charhandler)(sp->hint, (char *)rec + 24, 6);
		return 0;
	}

	if (memcmp(rec + 24, sp->dimsp->plane, 6) != 0)
		return 0;

	if (sp->target == NUSDIM_PLANE2) {
		(sp->charhandler)(sp->hint, (char *)rec + 30, 6);		
		return 0;
	}

	if (memcmp(rec + 30, sp->dimsp->plane2, 6) != 0)
		return 0;

	if (sp->target == NUSDIM_ELEMENT) {
		(sp->charhandler)(sp->hint, (char *)rec + 36, 6);
		return 0;		
	}

	return EOF;
}

	static int
record_handler(const char *fnam, const unsigned char *rec, void *sp)
{
	if (memcmp(rec, "CNTL", 4) == 0) {
		return check_cntl(fnam, (struct search *)sp, rec);
	} else if (memcmp(rec, "DATA", 4) == 0) {
		return check_data((struct search *)sp, rec);
	}
	/* mȂR[h̓G[ł͂ȂAƂɂ */
	return 0;
}

/*
 * dp->stat ̂ 0 ɂȂĂŉʂ̃rbgoB
 */
	static int
first_undefined_dim(const struct nus_dims_t *dp)
{
	int	r;
	for (r = 1; r; r <<= 1) {
		if (r & ~(dp->stat))
			return r;
	}
	return 0;
}

/*
 * ^ꂽ (RpC) NuSDaS pX dp  NuSDaS [goA
 * ̉̃t@CׂĒT邱Ƃɂ
 * dp ɂĖmɂȂĂŏ̎ɂăXgƂB
 * ̎^ł inthandler ĂяoA
 * łȂ charhandler ĂяoB
 */
	int
nusdas_list_dims(const struct nus_dims_t *dp,
	void *hint,
	int (*inthandler)(void *hint, N_SI4 i),
	int (*charhandler)(void *hint, char *c, size_t len),
	int debug)
{
	struct search	sp;
	char		*rootname;
	int		r;

	r = nusdas_find_root(&(dp->type), &rootname);
	if (r != 0)
		return r;

	sp.dimsp = dp;
	sp.target = first_undefined_dim(dp);
	sp.hint = hint;
	sp.inthandler = inthandler;
	sp.charhandler = charhandler;
	sp.valid_mismatch = sp.member_mismatch = 0;
	sp.debug = debug;

	r = recursive_search(rootname, &sp, record_handler);
	free(rootname);

	return r;
}
