#define N_DEF_DEC2PAC_SOURCE
#include "nusdas_sys.h"

/* -------------------------------------------------------------------------- */
void
n_decode_2pac_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		 N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_f, base, amp;
    N_SI4 i, j, je, k, l;
    N_SI2 *si2;
    N_UI1 uc8;

    d_f = (float *) data;
    memcpy(&base, (float *) d, sizeof(float));
    memcpy(&amp, (float *) (d + sizeof(float)), sizeof(float));
    si2 = (N_SI2 *) (d + sizeof(float) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_f[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r4 : amp * si2[k++] + base;
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_f[i] = (si2[i] == *((N_SI2 *) p_miss)) ? n_miss_r4
		: amp * si2[i] + base;
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_f[i] = amp * si2[i] + base;
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_2pac_r8(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		 N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    double *d_d;
    float base, amp;
    N_SI4 i, j, je, k, l;
    N_SI2 *si2;
    N_UI1 uc8;

    d_d = (double *) data;
    memcpy(&base, (float *) d, sizeof(float));
    memcpy(&amp, (float *) (d + sizeof(float)), sizeof(float));
    si2 = (N_SI2 *) (d + sizeof(float) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_d[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r8 : (double) (amp * si2[k++] + base);
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_d[i] = (si2[i] == *((N_SI2 *) p_miss)) ? n_miss_r8
		: (double) (amp * si2[i] + base);
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_d[i] = (double) (amp * si2[i] + base);
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_2pac_i4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		 N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 *d_i;
    float base, amp, fl4;
    N_SI4 i, j, je, k, l;
    N_SI2 *si2;
    N_UI1 uc8;

    d_i = (N_SI4 *) data;
    memcpy(&base, (float *) d, sizeof(float));
    memcpy(&amp, (float *) (d + sizeof(float)), sizeof(float));
    si2 = (N_SI2 *) (d + sizeof(float) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_i[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_si4 : n_round_i4_r4(amp * si2[k++] + base);
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_i[i] = (si2[i] == *((N_SI2 *) p_miss)) ? n_miss_si4
		: n_round_i4_r4(amp * si2[i] + base);
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_i[i] = n_round_i4_r4(amp * si2[i] + base);
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_2pac_nc(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		 N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI2 *d_s;
    N_SI4 i, j, je, k, l;
    N_SI2 *si2;
    N_UI1 uc8;

    memcpy((N_UI1 *) data, (N_UI1 *) d, sizeof(float));
    memcpy((N_UI1 *) (data + sizeof(float)),
	   (N_UI1 *) (d + sizeof(float)), sizeof(float));
    si2 = (N_SI2 *) (d + sizeof(float) * 2);
    d_s = (N_SI2 *) (data + sizeof(float) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_s[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_si2 : si2[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_s[i] = (si2[i] == *((N_SI2 *) p_miss)) ? n_miss_si2 : si2[i];
    } else {
	memcpy(d_s, si2, sizeof(N_SI2) * ng);
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_4pac_r8(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		 N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    double *d_d, base, amp;
    N_SI4 i, j, je, k, l;
    N_SI4 *si4;
    N_UI1 uc8;

    d_d = (double *) data;
    memcpy(&base, (double *) d, sizeof(double));
    memcpy(&amp, (double *) (d + sizeof(double)), sizeof(double));
    si4 = (N_SI4 *) (d + sizeof(double) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_d[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r8 : (double) (amp * si4[k++] + base);
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_d[i] = (si4[i] == *((N_SI4 *) p_miss)) ? n_miss_r8
		: (double) (amp * si4[i] + base);
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_d[i] = (double) (amp * si4[i] + base);
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_4pac_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		 N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_f;
    double base, amp;
    N_SI4 i, j, je, k, l;
    N_SI4 *si4;
    N_UI1 uc8;

    d_f = (float *) data;
    memcpy(&base, (double *) d, sizeof(double));
    memcpy(&amp, (double *) (d + sizeof(double)), sizeof(double));
    si4 = (N_SI4 *) (d + sizeof(double) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_f[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r4 : (float) (amp * si4[k++] + base);
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_f[i] = (si4[i] == *((N_SI4 *) p_miss)) ? n_miss_r4
		: (float) (amp * si4[i] + base);
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_f[i] = (float) (amp * si4[i] + base);
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_1pac_i4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		 N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 *d_i;
    float base, amp;
    N_SI4 i, j, je, k, l;
    N_UI1 uc8, *uc1;

    d_i = (N_SI4 *) data;
    memcpy(&base, (float *) d, sizeof(float));
    memcpy(&amp, (float *) (d + sizeof(float)), sizeof(float));
    uc1 = (N_UI1 *) (d + sizeof(float) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_i[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_si4 : n_round_i4_r4(amp * uc1[k++] + base);
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_i[i] = (uc1[i] == *((N_UI1 *) p_miss)) ? n_miss_si4
		: n_round_i4_r4(amp * uc1[i] + base);
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_i[i] = n_round_i4_r4(amp * uc1[i] + base);
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_1pac_i2(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		 N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI2 *d_s;
    float base, amp;
    N_SI4 i, j, je, k, l;
    N_UI1 uc8, *uc1;

    d_s = (N_SI2 *) data;
    memcpy(&base, (float *) d, sizeof(float));
    memcpy(&amp, (float *) (d + sizeof(float)), sizeof(float));
    uc1 = (N_UI1 *) (d + sizeof(float) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_s[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_si2 : n_round_i2_r4(amp * uc1[k++] + base);
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_s[i] = (uc1[i] == *((N_UI1 *) p_miss)) ? n_miss_si2
		: n_round_i2_r4(amp * uc1[i] + base);
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_s[i] = n_round_i2_r4(amp * uc1[i] + base);
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_1pac_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		 N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float base, amp, *d_f;
    N_SI4 i, j, je, k, l;
    N_UI1 uc8, *uc1;

    d_f = (float *) data;
    memcpy(&base, (float *) d, sizeof(float));
    memcpy(&amp, (float *) (d + sizeof(float)), sizeof(float));
    uc1 = (N_UI1 *) (d + sizeof(float) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_f[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r4 : (float) (amp * uc1[k++] + base);
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_f[i] = (uc1[i] == *((N_UI1 *) p_miss)) ? n_miss_r4
		: (float) (amp * uc1[i] + base);
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_f[i] = (float) (amp * uc1[i] + base);
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_2upac_r8(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		  N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    double *d_d;
    float base, amp;
    N_SI4 i, j, je, k, l;
    N_UI2 *ui2;
    N_UI1 uc8;

    d_d = (double *) data;
    memcpy(&base, (float *) d, sizeof(float));
    memcpy(&amp, (float *) (d + sizeof(float)), sizeof(float));
    ui2 = (N_UI2 *) (d + sizeof(float) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_d[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r8 : (double) (amp * ui2[k++] + base);
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_d[i] = (ui2[i] == *((N_UI2 *) p_miss)) ? n_miss_r8
		: (double) (amp * ui2[i] + base);
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_d[i] = (double) (amp * ui2[i] + base);
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_2upac_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
		  N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_f, base, amp;
    N_SI4 i, j, je, k, l;
    N_UI2 *ui2;
    N_UI1 uc8;

    d_f = (float *) data;
    memcpy(&base, (float *) d, sizeof(float));
    memcpy(&amp, (float *) (d + sizeof(float)), sizeof(float));
    ui2 = (N_UI2 *) (d + sizeof(float) * 2);
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_f[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r4 : amp * ui2[k++] + base;
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_f[i] = (ui2[i] == *((N_UI2 *) p_miss)) ? n_miss_r4 :
		amp * ui2[i] + base;
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_f[i] = amp * ui2[i] + base;
    }
}
