#define N_DEF_DECNCMP_SOURCE
#include "nusdas_sys.h"

/* -------------------------------------------------------------------------- */
/* subroutine archive: N_SI4 n_decode_??_**()                                 */
/* decode work data from DATA (data type ??) to the user pointer (type **)    */
/*   output:  *data   = user data pointer (must be allocated)                 */
/*   input:   *d      = encoded data pointer                                  */
/*            ng      = item number of user data                              */
/*            *p_miss = missing value in a DATA record                        */
/*            *p_mask = the pointer to mask bit                               */
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
void
n_decode_i4_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_fl4;
    N_SI4 i, j, je, k, l, tbyte = 0, *si4;
    N_UI1 uc8;

    d_fl4 = (float *) data;
    si4 = (N_SI4 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_fl4[l] = (((uc8 >> (7 - j)) & 0x01) ^
			    0x01) ? n_miss_r4 : (float) si4[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_fl4[i] =
		(si4[i] == (N_SI4) (*p_miss)) ? n_miss_r4 : (float) si4[i];
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_fl4[i] = (float) si4[i];
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_i4_i4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0, *d_si4, *si4;
    N_UI1 uc8;

    d_si4 = (N_SI4 *) data;
    si4 = (N_SI4 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_si4[l] = (((uc8 >> (7 - j)) & 0x01) ^
			    0x01) ? n_miss_si4 : si4[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_si4[i] = (si4[i] == (N_SI4) (*p_miss)) ? n_miss_si4 : si4[i];
    } else {
	memcpy(d_si4, si4, ng * sizeof(N_SI4));
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_i2_i4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0, *d_si4;
    N_SI2 *si2;
    N_UI1 uc8;

    d_si4 = (N_SI4 *) data;
    si2 = (N_SI2 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_si4[l] = (((uc8 >> (7 - j)) & 0x01) ^
			    0x01) ? n_miss_si4 : (N_SI4) si2[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_si4[i] = (si2[i] == (N_SI2) (*p_miss)) ? n_miss_si4
		: (N_SI4) si2[i];
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_si4[i] = (N_SI4) si2[i];
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_i2_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng,
	       N_UI1 * p_miss, N_UI1 * p_mask, N_SI4 xn)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0;
    float *d_fl4;
    N_SI2 *si2;
    N_UI1 uc8;

    d_fl4 = (float *) data;
    si2 = (N_SI2 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_fl4[l] = (((uc8 >> (7 - j)) & 0x01) ^
			    0x01) ? n_miss_r4 : (float) si2[k++] / xn;
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_fl4[i] = (si2[i] == (N_SI2) (*p_miss)) ? n_miss_r4
		: (float) si2[i] / xn;
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_fl4[i] = (float) si2[i] / xn;
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_i2_i2(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_SI2 *d_si2, *si2;
    N_UI1 uc8;

    d_si2 = (N_SI2 *) data;
    si2 = (N_SI2 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_si2[l] = (((uc8 >> (7 - j)) & 0x01) ^
			    0x01) ? n_miss_si2 : si2[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_si2[i] = (si2[i] == (N_SI2) (*p_miss)) ? n_miss_si2 : si2[i];
    } else {
	memcpy(d_si2, si2, ng * sizeof(N_SI2));
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_i1_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_fl4;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8, *uc1;

    d_fl4 = (float *) data;
    uc1 = (N_UI1 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_fl4[l] = (((uc8 >> (7 - j)) & 0x01) ^
			    0x01) ? n_miss_r4 : (float) uc1[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_fl4[i] =
		(uc1[i] == (N_UI1) (*p_miss)) ? n_miss_r4 : (float) uc1[i];
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_fl4[i] = (float) uc1[i];
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_i1_i4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0, *d_si4;
    N_UI1 uc8, *uc1;

    d_si4 = (N_SI4 *) data;
    uc1 = (N_UI1 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_si4[l] = (((uc8 >> (7 - j)) & 0x01) ^
			    0x01) ? n_miss_si4 : (N_SI4) uc1[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_si4[i] = (uc1[i] == (N_UI1) (*p_miss)) ? n_miss_si4 :
		(N_SI4) uc1[i];
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_si4[i] = (N_SI4) uc1[i];
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_i1_i2(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_SI2 *d_si2;
    N_UI1 uc8, *uc1;

    d_si2 = (N_SI2 *) data;
    uc1 = (N_UI1 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_si2[l] = (((uc8 >> (7 - j)) & 0x01) ^
			    0x01) ? n_miss_si2 : (N_SI2) uc1[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_si2[i] = (uc1[i] == (N_UI1) (*p_miss)) ? n_miss_si2 :
		(N_SI2) uc1[i];
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_si2[i] = (N_SI2) uc1[i];
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_i1_i1(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8, *d_uc1, *uc1;

    d_uc1 = (N_UI1 *) data;
    uc1 = (N_UI1 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_uc1[l] = (((uc8 >> (7 - j)) & 0x01) ^
			    0x01) ? n_miss_uc1 : uc1[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_uc1[i] = (uc1[i] == (N_UI1) (*p_miss)) ? n_miss_uc1 : uc1[i];
    } else {
	memcpy(d_uc1, uc1, ng * sizeof(N_UI1));
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_r4_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_f, *fl4;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8;

    d_f = (float *) data;
    fl4 = (float *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_f[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r4 : fl4[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_f[i] = (fl4[i] == (float) (*p_miss)) ? n_miss_r4 : fl4[i];
    } else {
	memcpy(d_f, fl4, ng * sizeof(float));
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_r4_r8(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    double *d_d;
    float *fl4;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8;

    d_d = (double *) data;
    fl4 = (float *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_d[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r8 : (double) fl4[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_d[i] =
		(fl4[i] ==
		 (float) (*p_miss)) ? n_miss_r8 : (double) fl4[i];
    } else {
	for (i = 0; i < ng; i++)
	    d_d[i] = (double) fl4[i];
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_r8_r8(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    double *d_d, *db8;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8;

    d_d = (double *) data;
    db8 = (double *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_d[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r8 : db8[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_d[i] = (db8[i] == (double) (*p_miss)) ? n_miss_r8 : db8[i];
    } else {
	memcpy(d_d, db8, ng * sizeof(double));
    }
}


/* -------------------------------------------------------------------------- */
void
n_decode_r8_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_f;
    double *db8;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8;

    d_f = (float *) data;
    db8 = (double *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, k = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++, l++)
		d_f[l] = (((uc8 >> (7 - j)) & 0x01) ^ 0x01) ?
		    n_miss_r4 : (float) db8[k++];
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++)
	    d_f[i] =
		(db8[i] ==
		 (double) (*p_miss)) ? n_miss_r4 : (float) db8[i];
    } else {
	for (i = 0; i < ng; i++)
	    d_f[i] = (float) db8[i];
    }
}
