#define N_DEF_ENC2PAC_SOURCE
#include "nusdas_sys.h"
#include <float.h>
#include <limits.h>

/* -------------------------------------------------------------------------- */
void
n_maxmin_i2(N_SI2 * d_s, N_SI4 ng, N_UI1 * p_miss, N_UI1 * p_mask,
	    N_SI2 * max_s, N_SI2 * min_s)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l;
    N_SI4 max, min;
    N_UI1 uc8;

    min = SHRT_MAX - 1;
    max = SHRT_MIN + 1;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		k = i + j;
		if (d_s[k] > max)
		    max = d_s[k];
		if (d_s[k] < min)
		    min = d_s[k];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++) {
	    if (d_s[i] == n_miss_si2)
		continue;
	    if (d_s[i] > max)
		max = d_s[i];
	    if (d_s[i] < min)
		min = d_s[i];
	}
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++) {
	    if (d_s[i] > max)
		max = d_s[i];
	    if (d_s[i] < min)
		min = d_s[i];
	}
    }
    *max_s = max;
    *min_s = min;
}


/* -------------------------------------------------------------------------- */
void
n_maxmin_i4(N_SI4 * d_i, N_SI4 ng, N_UI1 * p_miss, N_UI1 * p_mask,
	    N_SI4 * max_i, N_SI4 * min_i)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l;
    N_SI4 max, min;
    N_UI1 uc8;

    min = LONG_MAX - 1;
    max = LONG_MIN + 1;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		k = i + j;
		if (d_i[k] > max)
		    max = d_i[k];
		if (d_i[k] < min)
		    min = d_i[k];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++) {
	    if (d_i[i] == n_miss_si4)
		continue;
	    if (d_i[i] > max)
		max = d_i[i];
	    if (d_i[i] < min)
		min = d_i[i];
	}
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++) {
	    if (d_i[i] > max)
		max = d_i[i];
	    if (d_i[i] < min)
		min = d_i[i];
	}
    }
    *max_i = max;
    *min_i = min;
}


/* -------------------------------------------------------------------------- */
void
n_maxmin_r8(double *d_d, N_SI4 ng, N_UI1 * p_miss, N_UI1 * p_mask,
	    double *max_d, double *min_d)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l;
    double max, min;
    N_UI1 uc8;

    min = DBL_MAX - 1;
    max = (-1) * DBL_MAX + 1;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		k = i + j;
		if (d_d[k] > max)
		    max = d_d[k];
		if (d_d[k] < min)
		    min = d_d[k];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++) {
	    if (d_d[i] == n_miss_r8)
		continue;
	    if (d_d[i] > max)
		max = d_d[i];
	    if (d_d[i] < min)
		min = d_d[i];
	}
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++) {
	    if (d_d[i] > max)
		max = d_d[i];
	    if (d_d[i] < min)
		min = d_d[i];
	}
    }
    *max_d = max;
    *min_d = min;
}


/* -------------------------------------------------------------------------- */
void
n_maxmin_r4(float *d_f, N_SI4 ng, N_UI1 * p_miss, N_UI1 * p_mask,
	    float *max_f, float *min_f)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l;
    float max, min;
    N_UI1 uc8;

    min = FLT_MAX - 1;
    max = (-1) * FLT_MAX + 1;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		k = i + j;
		if (d_f[k] > max)
		    max = d_f[k];
		if (d_f[k] < min)
		    min = d_f[k];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (i = 0; i < ng; i++) {
	    if (d_f[i] == n_miss_r4)
		continue;
	    if (d_f[i] > max)
		max = d_f[i];
	    if (d_f[i] < min)
		min = d_f[i];
	}
    } else {
	/*poption parallel */
	for (i = 0; i < ng; i++) {
	    if (d_f[i] > max)
		max = d_f[i];
	    if (d_f[i] < min)
		min = d_f[i];
	}
    }
    *max_f = max;
    *min_f = min;
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_2pac_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng,
		 N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_f, max_f, min_f, base, amp;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_SI2 *si2;
    N_UI1 uc8;

    d_f = (float *) data;
    n_maxmin_r4(d_f, ng, p_miss, p_mask, &max_f, &min_f);
    base = (max_f + min_f) / 2;
    amp = (max_f == min_f) ? 1.0 : (max_f - min_f) / (2 * (SHRT_MAX - 1));
    memcpy(d, &base, 4);
    memcpy((N_UI1 *) (d + 4), &amp, 4);
    si2 = (N_SI2 *) (d + 8);
    if (p_mask != NULL) {
	/*poption noparallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		k = i + j;
		si2[l++] = (d_f[k] >= base) ? (N_SI2) ((d_f[k]
							- base) / amp +
						       0.5)
		    : (N_SI2) ((d_f[k] - base) / amp - 0.5);
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    if (d_f[l] == n_miss_r4)
		si2[l] = *((N_SI2 *) p_miss);
	    else {
		si2[l] = (d_f[l] >= base) ? (N_SI2) ((d_f[l]
						      - base) / amp + 0.5)
		    : (N_SI2) ((d_f[l] - base) / amp - 0.5);
	    }
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    si2[l] = (d_f[l] >= base) ? (N_SI2) ((d_f[l] - base)
						 / amp + 0.5)
		: (N_SI2) ((d_f[l] - base) / amp - 0.5);
    }
    tbyte = sizeof(float) * 2 + sizeof(N_SI2) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_2pac_r8(N_UI1 * data, N_UI1 * d, N_SI4 ng,
		 N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    double *d_d, max_d, min_d;
    float base, amp;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_SI2 *si2;
    N_UI1 uc8;

    d_d = (double *) data;
    n_maxmin_r8(d_d, ng, p_miss, p_mask, &max_d, &min_d);
    base = (float) ((max_d + min_d) / 2);
    amp = (float)((max_d - min_d) / (2 * (SHRT_MAX - 1)));
    if (amp == 0.0)
	amp = 1.0;
    memcpy(d, &base, 4);
    memcpy((N_UI1 *) (d + 4), &amp, 4);
    si2 = (N_SI2 *) (d + 8);
    if (p_mask != NULL) {
	/*poption noparallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		k = i + j;
		if (d_d[k] >= base) {
		    si2[l++] = (N_SI2) ((d_d[k] - base) / amp + 0.5);
		} else {
		    si2[l++] = (N_SI2) ((d_d[k] - base) / amp - 0.5);
		}
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++) {
	    if (d_d[l] == n_miss_r8) {
		si2[l] = *((N_SI2 *) p_miss);
	    } else if (d_d[l] >= base) {
		si2[l] = (N_SI2) ((d_d[l] - base) / amp + 0.5);
	    } else {
		si2[l] = (N_SI2) ((d_d[l] - base) / amp - 0.5);
            }
        }
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    si2[l] = (d_d[l] >= base) ? (N_SI2) ((d_d[l] - base)
						 / amp + 0.5)
		: (N_SI2) ((d_d[l] - base) / amp - 0.5);
    }
    tbyte = sizeof(float) * 2 + sizeof(N_SI2) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_2pac_i4(N_UI1 * data, N_UI1 * d, N_SI4 ng,
		 N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float base, amp;
    N_SI4 *d_i, max_i, min_i, i, j, je, k, l, tbyte = 0;
    N_SI2 *si2;
    N_UI1 uc8;

    d_i = (N_SI4 *) data;
    n_maxmin_i4(d_i, ng, p_miss, p_mask, &max_i, &min_i);
    base = (float) ((max_i + min_i)) / 2;
    amp =
	(max_i ==
	 min_i) ? 1.0 : (float) ((max_i - min_i)) / (2 * (SHRT_MAX - 1));
    memcpy(d, &base, 4);
    memcpy((N_UI1 *) (d + 4), &amp, 4);
    si2 = (N_SI2 *) (d + 8);
    if (p_mask != NULL) {
	/*poption noparallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		k = i + j;
		si2[l++] = (d_i[k] >= base) ? (N_SI2) ((d_i[k]
							- base) / amp +
						       0.5)
		    : (N_SI2) ((d_i[k] - base) / amp - 0.5);
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    if (d_i[l] == n_miss_si4)
		si2[l] = *((N_SI2 *) p_miss);
	    else
		si2[l] = (d_i[l] >= base) ? (N_SI2) ((d_i[l]
						      - base) / amp + 0.5)
		    : (N_SI2) ((d_i[l] - base) / amp - 0.5);
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    si2[l] = (d_i[l] >= base) ? (N_SI2) ((d_i[l] - base)
						 / amp + 0.5)
		: (N_SI2) ((d_i[l] - base) / amp - 0.5);
    }
    tbyte = sizeof(float) * 2 + sizeof(N_SI2) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_2pac_nc(N_UI1 * data, N_UI1 * d, N_SI4 ng,
		 N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float base, amp;
    N_SI4 *d_i, max_i, min_i, i, j, je, k, l, tbyte = 0;
    N_SI2 *si2, *d_s;
    N_UI1 uc8;

    memcpy(&base, (N_UI1 *) data, sizeof(float));
    memcpy(&amp, (N_UI1 *) (data + sizeof(float)), sizeof(float));
    memcpy(d, &base, 4);
    memcpy((N_UI1 *) (d + 4), &amp, 4);
    d_s = (N_SI2 *) (data + sizeof(float) * 2);
    si2 = (N_SI2 *) (d + sizeof(float) * 2);
    if (p_mask != NULL) {
	/*poption noparallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		k = i + j;
		si2[l++] = d_s[k];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    if (d_s[l] == n_miss_si2)
		si2[l] = *((N_SI2 *) p_miss);
	    else
		si2[l] = d_s[l];
    } else {
	l = ng;
	memcpy(si2, d_s, sizeof(N_SI2) * ng);
    }
    tbyte = sizeof(float) * 2 + sizeof(N_SI2) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_4pac_r8(N_UI1 * data, N_UI1 * d, N_SI4 ng,
		 N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    double *d_d, max_d, min_d, base, amp;
    N_SI4 i, j, je, k, l, tbyte = 0, *si4;
    N_UI1 uc8;

    d_d = (double *) data;
    n_maxmin_r8(d_d, ng, p_miss, p_mask, &max_d, &min_d);
    base = (double) ((max_d + min_d) / 2);
    amp =
	(max_d ==
	 min_d) ? 1.0 : (double) ((max_d - min_d) / 2 / (LONG_MAX - 1));
    memcpy(d, &base, 8);
    memcpy((N_UI1 *) (d + 8), &amp, 8);
    si4 = (N_SI4 *) (d + 16);
    if (p_mask != NULL) {
	/*poption noparallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		k = i + j;
		si4[l++] = (d_d[k] >= base) ? (N_SI4) ((d_d[k]
							- base) / amp +
						       0.5)
		    : (N_SI4) ((d_d[k] - base) / amp - 0.5);
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    if (d_d[l] == n_miss_r8)
		si4[l] = *((N_SI4 *) p_miss);
	    else
		si4[l] = (d_d[l] >= base) ? (N_SI4) ((d_d[l]
						      - base) / amp + 0.5)
		    : (N_SI4) ((d_d[l] - base) / amp - 0.5);
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    si4[l] = (d_d[l] >= base) ? (N_SI4) ((d_d[l] - base)
						 / amp + 0.5)
		: (N_SI4) ((d_d[l] - base) / amp - 0.5);
    }
    tbyte = sizeof(double) * 2 + sizeof(N_SI4) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_1pac_i2(N_UI1 * data, N_UI1 * d, N_SI4 ng,
		 N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float base, amp;
    N_SI2 *d_s, max_s, min_s;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8, *uc1;

    d_s = (N_SI2 *) data;
    n_maxmin_i2(d_s, ng, p_miss, p_mask, &max_s, &min_s);
    base = (float) min_s;
    amp =
	(max_s ==
	 min_s) ? 1.0 : (float) ((max_s - min_s)) / (UCHAR_MAX - 1);
    memcpy(d, &base, 4);
    memcpy((N_UI1 *) (d + 4), &amp, 4);
    uc1 = (N_UI1 *) (d + 8);
    if (p_mask != NULL) {
	/*poption noparallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		uc1[l++] = (N_UI1) ((d_s[i + j] - base) / amp + 0.5);
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] = (d_s[l] == n_miss_si2) ? *((N_UI1 *) p_miss)
		: (N_UI1) ((d_s[l] - base) / amp + 0.5);
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] = (N_UI1) ((d_s[l] - base) / amp + 0.5);
    }
    tbyte = sizeof(float) * 2 + sizeof(N_UI1) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_1pac_i4(N_UI1 * data, N_UI1 * d, N_SI4 ng,
		 N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float base, amp;
    N_SI4 *d_i, max_i, min_i;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8, *uc1;

    d_i = (N_SI4 *) data;
    n_maxmin_i4(d_i, ng, p_miss, p_mask, &max_i, &min_i);
    base = (float) min_i;
    amp =
	(max_i ==
	 min_i) ? 1.0 : (float) ((max_i - min_i)) / (UCHAR_MAX - 1);
    memcpy(d, &base, 4);
    memcpy((N_UI1 *) (d + 4), &amp, 4);
    uc1 = (N_UI1 *) (d + 8);
    if (p_mask != NULL) {
	/*poption noparallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		uc1[l++] = (N_UI1) ((d_i[i + j] - base) / amp + 0.5);
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] = (d_i[l] == n_miss_si4) ? *((N_UI1 *) p_miss)
		: (N_UI1) ((d_i[l] - base) / amp + 0.5);
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] = (N_UI1) ((d_i[l] - base) / amp + 0.5);
    }
    tbyte = sizeof(float) * 2 + sizeof(N_UI1) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_1pac_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng,
		 N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_f, max_f, min_f, base, amp;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8, *uc1;

    d_f = (float *) data;
    n_maxmin_r4(d_f, ng, p_miss, p_mask, &max_f, &min_f);
    base = min_f;
    amp = (max_f == min_f) ? 1.0 : (max_f - min_f) / (UCHAR_MAX - 1);
    memcpy(d, &base, 4);
    memcpy((N_UI1 *) (d + 4), &amp, 4);
    uc1 = (N_UI1 *) (d + 8);
    if (p_mask != NULL) {
	/*poption noparallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		uc1[l++] = (N_UI1) ((d_f[i + j] - base) / amp + 0.5);
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] = (d_f[l] == n_miss_r4) ? *((N_UI1 *) p_miss)
		: (N_UI1) ((d_f[l] - base) / amp + 0.5);
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] = (N_UI1) ((d_f[l] - base) / amp + 0.5);
    }
    tbyte = sizeof(float) * 2 + sizeof(N_UI1) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_2upac_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng,
		  N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_f, max_f, min_f, base, amp;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI2 *ui2;
    N_UI1 uc8;

    d_f = (float *) data;
    n_maxmin_r4(d_f, ng, p_miss, p_mask, &max_f, &min_f);
    base = min_f;
    amp = (max_f == min_f) ? 1.0 : (max_f - min_f) / (2 * (SHRT_MAX - 1));
    memcpy(d, &base, 4);
    memcpy((N_UI1 *) (d + 4), &amp, 4);
    ui2 = (N_UI2 *) (d + 8);
    if (p_mask != NULL) {
	/*poption noparallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		ui2[l++] = (N_UI2) ((d_f[i + j] - base) / amp + 0.5);
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    ui2[l] = (d_f[l] == n_miss_r4) ? *((N_UI2 *) p_miss)
		: (N_UI2) ((d_f[l] - base) / amp + 0.5);
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    ui2[l] = (N_UI2) ((d_f[l] - base) / amp + 0.5);
    }
    tbyte = sizeof(float) * 2 + sizeof(N_UI2) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_2upac_r8(N_UI1 * data, N_UI1 * d, N_SI4 ng,
		  N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    double *d_d, max_d, min_d, *d_temp;
    float base, amp;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI2 *ui2;
    N_UI1 uc8;

    d_d = (double *) data;
    n_maxmin_r8(d_d, ng, p_miss, p_mask, &max_d, &min_d);
    base = (float) min_d;
    amp =
	(max_d ==
	 min_d) ? 1.0 : (float) ((max_d - min_d) / (2 * (SHRT_MAX - 1)));
    memcpy(d, &base, 4);
    memcpy((N_UI1 *) (d + 4), &amp, 4);
    ui2 = (N_UI2 *) (d + 8);
    if (p_mask != NULL) {
	/*poption noparallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		k = i + j;
		ui2[l++] = (N_UI2) ((d_d[k] - base) / amp + 0.5);
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    if (d_d[l] == n_miss_r8)
		ui2[l] = *((N_UI2 *) p_miss);
	    else
		ui2[l] = (N_UI2) ((d_d[l] - base) / amp + 0.5);
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    ui2[l] = (N_UI2) ((d_d[l] - base) / amp + 0.5);
    }
    tbyte = sizeof(float) * 2 + sizeof(N_SI2) * l;

    return (tbyte);
}
