#define N_DEF_ENCNCMP_SOURCE
#include "nusdas_sys.h"

/* -------------------------------------------------------------------------- */
/* subroutine archive: N_SI4 n_encode_??_**()                                 */
/* encode user data (data type **) to the work pointer for DATA (type ??)     */
/*   input:   *data   = user data pointer                                     */
/*   output:  *d      = encoded data pointer (must be allocated)              */
/*   input:   ng      = item number of user data                              */
/*            *p_miss = missing value in a DATA record                        */
/*            *p_mask = the pointer to mask bit                               */
/*   return:  >=0     = data size (byte) of set data pointer                  */
/*            -1      = user data is greater than max of data for encode      */
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
N_SI4
n_encode_i4_i4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0, *si4, *d_si4;
    N_UI1 uc8;

    d_si4 = (N_SI4 *) data;
    si4 = (N_SI4 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		si4[l++] = d_si4[i + j];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    si4[l] =
		(d_si4[l] == n_miss_si4) ? (N_SI4) (*p_miss) : d_si4[l];
    } else {
	l = ng;
	memcpy(si4, d_si4, ng * sizeof(N_SI4));
    }
    tbyte = sizeof(N_SI4) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_i2_i4(N_UI1 * data, N_UI1 * d, N_SI4 ng,
	       N_UI1 * p_miss, N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0, *d_si4;
    N_SI2 *si2;
    N_UI1 uc8;

    d_si4 = (N_SI4 *) data;
    /*poption parallel */
    for (i = 0; i < ng; i++)
	if (*(d_si4 + i) != n_miss_si4 &&
	    (*(d_si4 + i) < SHRT_MIN || *(d_si4 + i) > SHRT_MAX))
	    return (-1);
    si2 = (N_SI2 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		si2[l++] = (N_SI2) d_si4[i + j];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    si2[l] = (d_si4[l] == n_miss_si4) ? (N_SI2) (*p_miss)
		: (N_SI2) d_si4[l];
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    si2[l] = (N_SI2) d_si4[l];
    }
    tbyte = sizeof(N_SI2) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_i2_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng,
	       N_UI1 * p_miss, N_UI1 * p_mask, N_SI4 xn)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0;
    float *d_fl4;
    N_SI2 *si2;
    N_UI1 uc8;

    d_fl4 = (float *) data;
    /*poption parallel */
    for (i = 0; i < ng; i++)
	if (*(d_fl4 + i) != n_miss_r4 &&
	    (*(d_fl4 + i) * xn < SHRT_MIN || *(d_fl4 + i) * xn > SHRT_MAX))
	    return (-1);
    si2 = (N_SI2 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		si2[l++] = (N_SI2) (d_fl4[i + j] * xn);
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    si2[l] = (d_fl4[l] == n_miss_r4) ? (N_SI2) (*p_miss)
		: (N_SI2) (d_fl4[l] * xn);
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    si2[l] = (N_SI2) (d_fl4[l] * xn);
    }
    tbyte = sizeof(N_SI2) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_i2_i2(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_SI2 *si2, *d_si2;
    N_UI1 uc8;

    d_si2 = (N_SI2 *) data;
    si2 = (N_SI2 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		si2[l++] = d_si2[i + j];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    si2[l] =
		(d_si2[l] == n_miss_si2) ? (N_SI2) (*p_miss) : d_si2[l];
    } else {
	l = ng;
	memcpy(si2, d_si2, ng * sizeof(N_SI2));
    }
    tbyte = sizeof(N_SI2) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_i1_i1(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8, *uc1, *d_uc1;

    d_uc1 = (N_UI1 *) data;
    uc1 = (N_UI1 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		uc1[l++] = d_uc1[i + j];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] =
		(d_uc1[l] == n_miss_uc1) ? (N_UI1) (*p_miss) : d_uc1[l];
    } else {
	l = ng;
	memcpy(uc1, d_uc1, ng * sizeof(N_UI1));
    }
    tbyte = sizeof(N_UI1) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_i1_i2(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_SI2 *d_si2;
    N_UI1 uc8, *uc1;

    d_si2 = (N_SI2 *) data;
    /*poption parallel */
    for (i = 0; i < ng; i++)
	if (d_si2[i] != n_miss_si2
	    && (d_si2[i] < 0 || d_si2[i] > UCHAR_MAX)) return (-1);
    uc1 = (N_UI1 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		uc1[l++] = (N_UI1) d_si2[i + j];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] = (d_si2[l] == n_miss_si2) ? (N_UI1) (*p_miss)
		: (N_UI1) d_si2[l];
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] = (N_UI1) d_si2[l];
    }
    tbyte = sizeof(N_UI1) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_i1_i4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, je, k, l, tbyte = 0, *d_si4;
    N_UI1 uc8, *uc1;

    d_si4 = (N_SI4 *) data;
    /*poption parallel */
    for (i = 0; i < ng; i++)
	if (d_si4[i] != n_miss_si4
	    && (d_si4[i] < 0 || d_si4[i] > UCHAR_MAX)) return (-1);
    uc1 = (N_UI1 *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		uc1[l++] = (N_UI1) d_si4[i + j];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] = (d_si4[l] == n_miss_si4) ? (N_UI1) (*p_miss)
		: (N_UI1) d_si4[l];
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    uc1[l] = (N_UI1) d_si4[l];
    }
    tbyte = sizeof(N_UI1) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_r4_r4(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    float *d_f, *fl4;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8;

    d_f = (float *) data;
    fl4 = (float *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		fl4[l++] = d_f[i + j];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    fl4[l] = (d_f[l] == n_miss_r4) ? (float) (*p_miss) : d_f[l];
    } else {
	l = ng;
	memcpy(fl4, d_f, ng * sizeof(float));
    }
    tbyte = sizeof(float) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_r4_r8(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    double *d_d;
    float *fl4;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8;

    d_d = (double *) data;
    fl4 = (float *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		fl4[l++] = (float) d_d[i + j];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    fl4[l] =
		(d_d[l] == n_miss_r8) ? (float) (*p_miss) : (float) d_d[l];
    } else {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    fl4[l] = (float) d_d[l];
    }
    tbyte = sizeof(float) * l;

    return (tbyte);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_r8_r8(N_UI1 * data, N_UI1 * d, N_SI4 ng, N_UI1 * p_miss,
	       N_UI1 * p_mask)
/* -------------------------------------------------------------------------- */
{
    double *d_d, *db8;
    N_SI4 i, j, je, k, l, tbyte = 0;
    N_UI1 uc8;

    d_d = (double *) data;
    db8 = (double *) d;
    if (p_mask != NULL) {
	/*poption parallel */
	for (i = 0, l = 0; i < ng; i += 8) {
	    uc8 = *(p_mask + i / 8);
	    je = (i > ng - 8) ? ng - i : 8;
	    for (j = 0; j < je; j++) {
		if (((uc8 >> (7 - j)) & 0x01) ^ 0x01)
		    continue;
		db8[l++] = (double) d_d[i + j];
	    }
	}
    } else if (p_miss != NULL) {
	/*poption parallel */
	for (l = 0; l < ng; l++)
	    db8[l] =
		(d_d[l] ==
		 n_miss_r8) ? (double) (*p_miss) : (double) d_d[l];
    } else {
	l = ng;
	memcpy(db8, d_d, ng * sizeof(double));
    }
    tbyte = sizeof(double) * l;
    return (tbyte);
}
