#define N_DEF_FILESYS_SOURCE
#include "nusdas_sys.h"

N_SI4 n_mday[] = {
    31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
};


N_SI4 n_tunit_num = 7;
char n_tunit_string[][5] = {
    "HOUR", "MIN ", "DAY ", "WEEK", "PENT",
    "MONT", "JUN "
};


N_SI4 n_tunit_min[] = {
    60, 1, 60 * 24, 60 * 24 * 7, 60 * 24 * 5, 60 * 24 * 10,
    60 * 24 * 30, 1
};


N_SI4 n_pack_num = 12;
char n_pack_string[][5] = {
    "2PAC", "4PAC", "R4  ", "R8  ", "I1  ", "I2  ",
    "I4  ", "GRIB", "RLEN", "1PAC", "N1I2", "2UPC"
};


N_SI4 n_miss_num = 3;
char n_miss_string[][5] = {
    "NONE", "UDFV", "MASK"
};


N_SI4 n_mean_num = 3;
char n_mean_string[][5] = {
    "PVAL", "MEAN", "REPR"
};


N_SI4 n_subc_num = 7;
char n_subc_string[][5] = {
    "ETA ", "SIGM", "ZSTA", "TDIF", "RADR",
    "ISPC", "DPRD"
};


N_SI4 n_proj_num = 12;
char n_proj_string[][5] = {
    "XX  ", "LL  ", "LM  ", "PS  ", "MR  ", "GS  ",
    "OL  ", "XP  ", "YP  ", "FG  ", "RD  ", "ST  "
};



/* -------------------------------------------------------------------------- */
void n_clear_cntl_mem(N_ST_CNTL * n_cntl)
/* -------------------------------------------------------------------------- */
{
    if (n_cntl->dc != NULL) {
	free(n_cntl->dc);
	n_cntl->dc = NULL;
    }
    if (n_cntl->vt1 != NULL) {
	free(n_cntl->vt1);
	n_cntl->vt1 = NULL;
    }
    if (n_cntl->vt2 != NULL) {
	free(n_cntl->vt2);
	n_cntl->vt2 = NULL;
    }
    if (n_cntl->lv1 != NULL) {
	free(n_cntl->lv1);
	n_cntl->lv1 = NULL;
    }
    if (n_cntl->lv2 != NULL) {
	free(n_cntl->lv2);
	n_cntl->lv2 = NULL;
    }
    if (n_cntl->el != NULL) {
	free(n_cntl->el);
	n_cntl->el = NULL;
    }
}


/* -------------------------------------------------------------------------- */
void n_clear_def_mem(N_ST_DEF * def)
/* -------------------------------------------------------------------------- */
{
    if (def->dc != NULL) {
	free(def->dc);
	def->dc = NULL;
    }
    if (def->vt1 != NULL) {
	free(def->vt1);
	def->vt1 = NULL;
    }
    if (def->vt2 != NULL) {
	free(def->vt2);
	def->vt2 = NULL;
    }
    if (def->lv1 != NULL) {
	free(def->lv1);
	def->lv1 = NULL;
    }
    if (def->lv2 != NULL) {
	free(def->lv2);
	def->lv2 = NULL;
    }
    if (def->el != NULL) {
	free(def->el);
	def->el = NULL;
    }
    if (def->elmap != NULL) {
	free(def->elmap);
	def->elmap = NULL;
    }
}


/* -------------------------------------------------------------------------- */
N_SI4 n_ep(const N_SI4 eindex)
/* -------------------------------------------------------------------------- */
/* print a message to stderr with a subroutine name                           */
/*  input:  eindex  = header message type                                     */
/* -------------------------------------------------------------------------- */
{
    N_SI4 wout;

    wout = (n_io_cntl & N_IO_WARNING_OUT) ? N_ON : N_OFF;
    if (eindex == 2)
	errprintf( "Error nusdas(%s): ",
		n_routine_name[n_routine_d - 1]);
    else if (eindex == 1 && wout)
	errprintf( "Warning nusdas(%s): ",
		n_routine_name[n_routine_d - 1]);
    else if (eindex == 0)
	errprintf( "Notice nusdas(%s): ",
		n_routine_name[n_routine_d - 1]);
    errflush();

    return ((eindex == 1 && wout == N_OFF) ? 0 : 1);
}


/* -------------------------------------------------------------------------- */
N_SI4 n_set_latlon(char *s1, char *s2, float *lat, float *lon)
/* -------------------------------------------------------------------------- */
/* convert { s1, s2 } to { lat, lon }                                         */
/*  input:  s1    = latlon information character 1                            */
/*          s2    = latlon information character 2                            */
/*  output: *lat  = latitude                                                  */
/*          *lon  = longitude                                                 */
/* -------------------------------------------------------------------------- */
{
    char c1, c2, s3[80];
    float f1, f2;
    int i;
    i = strlen(s1);
    c1 = *(s1 + i - 1);
    strncpy(s3, s1, i - 1);
    s3[i - 1] = '\0';
    sscanf(s3, "%f", &f1);
    i = strlen(s2);
    c2 = *(s2 + i - 1);
    strncpy(s3, s2, i - 1);
    s3[i - 1] = '\0';
    sscanf(s3, "%f", &f2);

    if (((c1 == 'N' || c1 == 'n' || c1 == 'S' || c1 == 's') &&
	 (c2 == 'N' || c2 == 'n' || c2 == 'S' || c2 == 's')) ||
	((c1 == 'W' || c1 == 'w' || c1 == 'E' || c1 == 'e') &&
	 (c2 == 'W' || c2 == 'w' || c2 == 'E' || c2 == 'e')))
	return (-1);
    if (c1 == 'N' || c1 == 'n')
	*lat = f1;
    else if (c1 == 'S' || c1 == 's')
	*lat = f1 * (-1);
    else if (c1 == 'E' || c1 == 'e')
	*lon = f1;
    else if (c1 == 'W' || c1 == 'w')
	*lon = f1 * (-1);
    else
	return (-1);
    if (c2 == 'N' || c2 == 'n')
	*lat = f2;
    else if (c2 == 'S' || c2 == 's')
	*lat = f2 * (-1);
    else if (c2 == 'E' || c2 == 'e')
	*lon = f2;
    else if (c2 == 'W' || c2 == 'w')
	*lon = f2 * (-1);
    else
	return (-1);

    return (0);
}


/* -------------------------------------------------------------------------- */
char *n_spscanf(char *s, N_UI4 * p, const char *fmt)
/* -------------------------------------------------------------------------- */
/* read a word from the string                                                */
/*  input:  s       = string                                                  */
/*  both:   p       = offset in the string                                    */
/*  input:  fmt     = format                                                  */
/*  return: char *  = pointer for the static string                           */
/*          NULL    = error                                                   */
/* -------------------------------------------------------------------------- */
{
    char c;
    N_SI4 n = 0, l = 0, m;

    m = strlen(s);
    if (*p > N_LINE_MAX * 20 - 1 || m > N_LINE_MAX * 20 - 1)
	return (NULL);
    n_spscanf_string[0] = '\0';
    while (1) {
	c = s[(*p)++];
	if (c == '\0')
	    break;
	else if (c == ' ' || c == '\t' || c == '\n') {
	    if (n == 0)
		continue;
	    break;
	} else if (strcmp(fmt, "%s") == 0) {
	    if (c == '+' || c == '-' || c == '_' || c == '@'
		|| (n != 0 && c == '.') || isalnum(c))
		n_spscanf_string[n++] = c;
	} else if (strcmp(fmt, "%c") == 0) {
	    if (c == '+' || c == '-' || c == '_' || c == '@' || isalnum(c))
		n_spscanf_string[n++] = c;
	    break;
	} else if (strcmp(fmt, "%d") == 0) {
	    if ((n == 0 && (c == '+' || c == '-')) || isdigit(c))
		n_spscanf_string[n++] = c;
	    else
		break;
	} else if (strcmp(fmt, "%f") == 0) {
	    if ((n == 0 && (c == '+' || c == '-')) || isdigit(c))
		n_spscanf_string[n++] = c;
	    else if (l == 0 && c == '.') {
		l++;
		n_spscanf_string[n++] = c;
	    } else
		break;
	} else {
	    break;
	}
    }
    if (n > 0)
	n_spscanf_string[n] = '\0';
    else
	return (NULL);

    return (n_spscanf_string);
}


/**
 *  c ̏啶ɒuB
 */
void n_char_upper(char *c)
{
    while (*c) {
	*c = toupper(*c);
	c++;
    }
}

/**
 * fBNgƃt@CpX쐬B
 * (size - 1) oCgȏɂȂȂ悤Ƀ`FbNB
 */
void n_make_subpath(char *buffer, size_t size, const char *dir,
		    const char *file)
{
    char *p;
    size_t len;

    /* NUL ̂Ԃ */
    size--;

    p = buffer;
    if (dir == NULL) {
	len = strlen(buffer);
	p += len;
	size -= len;
    } else {
	len = strlen(dir);
	if (len > size)
	    len = size;
	strncpy(p, dir, len);
	p += len;
	size -= len;
	if (size < 2) {
	    *p = '\0';
	    return;
	}
    }

    if (len && p[-1] != '/') {
	*p = '/';
	p++;
	size--;
    }

    if (file[0] == '.' && file[1] == '/')
	file += 2;

    len = strlen(file);
    if (len > size)
	len = size;
    strncpy(p, file, len);
    p[len] = '\0';
}


/* -------------------------------------------------------------------------- */
void n_space_string(char *s, char *c)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i;
    for (i = 0; i < strlen(s); i++)
	if (*(s + i) == ' ')
	    *(s + i) = *c;
}


/* -------------------------------------------------------------------------- */
N_SI4 n_char_check(char c)
/* -------------------------------------------------------------------------- */
/*  input:  c  = check character                                              */
/*  return: 0  = O.K.                                                         */
/*          1  = N.G.                                                         */
/* -------------------------------------------------------------------------- */
{
    if (c == '+' || c == '-' || c == '_' || c == '@' || isalnum(c))
	return (0);
    else
	return (1);
}


/* -------------------------------------------------------------------------- */
N_SI4 n_getline(FILE * fp, char *line)
/* -------------------------------------------------------------------------- */
/* read next line from file                                                   */
/*  both:    fp    = ascii text file pointer                                  */
/*  output:  line  = line string                                              */
/*  return:  n     = item number                                              */
/*           -1    = EOF                                                      */
/*           -2    = error                                                    */
/* -------------------------------------------------------------------------- */
{
    char c, s[N_LINE_MAX];
    N_SI4 n = 0, p = 0, sf = 0, i;

    while (1) {
	c = (char) getc(fp);
	if (c == EOF || c == '\n')
	    break;
	else if (c == ' ' || c == '\t') {
	    if (sf == 1) {
		n++;
		sf = 0;
	    }
	} else {
	    if (sf == 0)
		sf = 1;
	}
	s[p] = c;
	if (p++ >= N_LINE_MAX)
	    return (-2);
    }
    if (sf == 1)
	n++;
    for (i = p; i < N_LINE_MAX; i++)
	s[i] = '\0';
    if (p == 0 && c == EOF)
	return (-1);
    memcpy(line, s, N_LINE_MAX);

    return (n);
}


/* -------------------------------------------------------------------------- */
N_SI4 n_default_type(char *type1, char *type2)
/* -------------------------------------------------------------------------- */
/* set omitted parameter in type1 and type2 strings                           */
/*  both:   type1  = type1 string (_model_2d_3d)                              */
/*          type2  = type2 string (_attribute_time)                           */
/*  return: 0      = no change                                                */
/*          bit0:1 = type1 changed                                            */
/*          bit1:1 = type2 changed                                            */
/*          -1     = _model or _attribute is not defined                      */
/*          -2     = unpermitted character used                               */
/*          -3     = nusdas undefined type                                    */
/*          -9     = nusdas system define type file open error                */
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, l1, l2, rt = 0;
    N_UI1 dum = 0;
    char cmodel[5], c2d[3], cattribute[3], ctime[3], c3d[3];
    char fmodel[5], f2d[3], fattribute[3], ftime[3], f3d[3];
    char filename[256], nus_sysdir[256], line[N_LINE_MAX], *char_set, *s;
    FILE *fp;
    for (i = 0; i < 5; i++)
	cmodel[i] = dum;
    for (i = 0; i < 3; i++)
	c2d[i] = dum;
    for (i = 0; i < 3; i++)
	cattribute[i] = dum;
    for (i = 0; i < 3; i++)
	ctime[i] = dum;
    for (i = 0; i < 3; i++)
	c3d[i] = dum;

    strcpy(n_routine_name[n_routine_d++], "default_type");

    /* specify character set */
    n_char_set = N_CHAR_SET_ALP_NUM;
    if ((char_set = getenv("NUSDAS_CHARSET")) != NULL &&
	(strcmp(char_set, "ISO9660") == 0 || strcmp(char_set, "iso9660")
	 == 0))
	n_char_set = N_CHAR_SET_ISO9660;

    /* specify nusdas system directory */
    if ((s = getenv("NUSDAS_DIR")) == NULL) {
	strcpy(nus_sysdir, N_SYSDEF_DIR);
    } else {
	sprintf(nus_sysdir, "%s/Sysdef/", s);
    }

    /* type1 check */
    if ((l1 = strlen(type1)) < 4) {
	N_ERTN(-1);
    }
    for (i = 0; i < l1; i++)
	if (n_char_check(*(type1 + i))) {
	    N_ERTN(-2);
	}
    if ((i = sscanf(type1, "%4s%2s%2s", cmodel, c2d, c3d)) <= 0) {
	N_ERTN(-1);
    }
	else
	if (!(i == 3 && strcmp(c2d, "  ") != 0 && strcmp(c3d, "  ") != 0))
    {
	strcpy(filename, nus_sysdir);
	strcat(filename, "_MODEL");
	if ((fp = fopen(filename, "r")) == NULL) {
	    N_ERP2("Can not open type defaults file <%s>\n", filename);
	    N_ERP2("You must specify type1 with no omission!\n");
	    N_ERTN(-9);
	}
	i = 0;
	while (1) {
	    if (n_getline(fp, line) < 0)
		break;
	    sscanf(line, "%4s %2s %2s", fmodel, f2d, f3d);
	    if (strcmp(cmodel, fmodel) == 0) {
		i = 1;
		break;
	    }
	}
	fclose(fp);
	if (i == 1) {
	    if (strlen(c2d) == 0 || strcmp(c2d, "  ") == 0)
		strcpy(c2d, f2d);
	    if (strlen(c3d) == 0 || strcmp(c3d, "  ") == 0)
		strcpy(c3d, f3d);
	    sprintf(type1, "%4s%2s%2s", cmodel, c2d, c3d);
	    rt += 1;
	} else {
	    N_ERP2("model<%s> has no defaults in NuSDaS\n",
		   cmodel);
	    N_ERP2("You must specify type1 with no omission!\n");
	    N_ERTN(-3);
	}
    }
    if (n_char_set)
	n_char_upper(type1);

    /* type2 check */
    if ((l2 = strlen(type2)) < 2) {
	N_ERTN(-1);
    }
    for (i = 0; i < l2; i++)
	if (n_char_check(*(type2 + i))) {
	    N_ERTN(-2);
	}
    if ((i = sscanf(type2, "%2s%2s", cattribute, ctime)) <= 0) {
	N_ERTN(-1);
    }
	else if (!(i == 2 && strcmp(cattribute, "  ") != 0 && strcmp(ctime,
								     "  ")
		   != 0)) {
	strcpy(filename, nus_sysdir);
	strcat(filename, "_ATTRIBUTE");
	if ((fp = fopen(filename, "r")) == NULL) {
	    N_ERP2("Can not open type defaults file <%s>\n",
		   filename);
	    N_ERP2("You must specify type2 with no omission!\n");
	}
	i = 0;
	while (1) {
	    if (n_getline(fp, line) < 0)
		break;
	    sscanf(line, "%s %s", fattribute, ftime);
	    if (strcmp(cattribute, fattribute) == 0) {
		i = 1;
		break;
	    }
	}
	fclose(fp);
	if (i == 1) {
	    if (strlen(ctime) == 0 || strcmp(ctime, "  ") == 0)
		strcpy(ctime, ftime);
	    sprintf(type2, "%2s%2s", cattribute, ctime);
	    rt += 2;
	} else {
	    N_ERP2("attribute<%s> has no default in NuSDaS\n",
		   cattribute);
	    N_ERP2("You must specify type2 with no omission!\n");
	    N_ERTN(-3);
	}
    }
    if (n_char_set)
	n_char_upper(type2);

    n_routine_d--;
    return (rt);
}


/* -------------------------------------------------------------------------- */
N_SI4 n_def_read_key(FILE * fp, const char *key, char *line)
/* -------------------------------------------------------------------------- */
/* read a line include a key word from the define file                        */
/*  both:    fp   = NuSDaS define file pointer                                */
/*  input:   key  = key word                                                  */
/*  output:  line = line string include the key word                          */
/*  return:  n    = item number                                               */
/*           -1   = error                                                     */
/* -------------------------------------------------------------------------- */
{
    char key_f[80];
    N_SI4 n;
    long offset = 0;

    fseek(fp, offset, SEEK_SET);
    while (1) {
	if ((n = n_getline(fp, line)) < 0) {
	    n = -1;
	    break;
	}
	sscanf(line, "%s", key_f);
	n_char_upper(key_f);
	if (strcmp(key_f, key) == 0)
	    break;
    }

    return (n);
}


/* -------------------------------------------------------------------------- */
FILE *n_open_def_main(const char *nrd,
		      const char *type1, const char *type2,
		      const char *type3)
/* -------------------------------------------------------------------------- */
/* open the targetted NuSDaS define file (main subroutine)                    */
/*  input:  nrd    = NuSDaS root directory                                    */
/*          type1  = target type1 string ( model necessary )                  */
/*          type2  = target type2 string ( attribute necessary )              */
/*          type3  = target type3 string ( type name )                        */
/*  return: fp     = file pointer to the target NuSDaS define file            */
/*          NULL   = open error                                               */
/* -------------------------------------------------------------------------- */
{
    FILE *fp;
    DIR *dp;
    struct dirent *de;
    char defpath[256], fn[256], line1[N_LINE_MAX], line2[N_LINE_MAX];
    char w[5][80], ftype[N_LINE_MAX], line3[N_LINE_MAX];
    N_SI4 i, ff;
    strcpy(n_routine_name[n_routine_d++], "open_def_main");

    strcpy(defpath, nrd);
    strcat(defpath, "/NUSDAS_DEF");
    if ((dp = opendir(defpath)) == NULL) {
	strcpy(defpath, nrd);
	strcat(defpath, "/nusdas_def");
	if ((dp = opendir(defpath)) == NULL) {
	    N_ERTN(NULL);
	}
    }
    ff = 0;
    fp = NULL;
    while (1) {
	if ((de = readdir(dp)) == NULL)
	    break;
	if (de->d_name[0] == '.')
	    continue;
	if ((i = strlen(de->d_name)) <= 4)
	    continue;
	if (memcmp(&de->d_name[i - 4], ".def", 4) != 0 &&
	    memcmp(&de->d_name[i - 4], ".DEF", 4) != 0) {
	    continue;
	}
	sprintf(fn, "%-s%c%-s", defpath, '/', de->d_name);
	if ((fp = fopen(fn, "r")) == NULL)
	    continue;
	/* --- ȉt@C fp ̓ǂ݂Ƃ --- */
	if (n_def_read_key(fp, "TYPE1", line1) > 0 &&
	    sscanf(line1, "%s %s %s %s %s", w[0], w[1], w[2], w[3]) == 4) {
	    strcpy(ftype, w[1]);
	    for (i = 2; i < 4; i++)
		strcat(ftype, w[i]);
	    if (n_char_set)
		n_char_upper(ftype);
	    if (strcmp(type1, ftype) == 0) {
		if (n_def_read_key(fp, "TYPE2", line2) >
		    0 && sscanf(line2, "%s %s %s", w[0], w[1], w[2]) == 3) {
		    strcpy(ftype, w[1]);
		    strcat(ftype, w[2]);
		    if (n_char_set)
			n_char_upper(ftype);
		    if (strcmp(type2, ftype) == 0) {
			if (n_def_read_key(fp, "TYPE3", line3) > 0 &&
			    sscanf(line3, "%s %s", w[0], w[1]) == 2) {
			    strcpy(ftype, w[1]);
			    if (n_char_set)
				n_char_upper(ftype);
			    if (strcmp(type3, ftype) == 0)
				ff = 1;
			}
		    }
		}
	    }
	}
	if (ff == 1)
	    break;
	fclose(fp);
	fp = NULL;
    }
    closedir(dp);
    n_routine_d--;

    return (fp);
}


/* -------------------------------------------------------------------------- */
FILE *n_open_def(N_SI4 nrd_num, char *nrd, N_SI4 * s_nid,
		 const char *type1, const char *type2, const char *type3)
/* -------------------------------------------------------------------------- */
/* open the targetted NuSDaS define file (entrance subroutine)                */
/*  input:  nrd_num  = NuSDaS file number (NUSDASxx)                          */
/*  output: nrd   = NuSDaS root directory path                                */
/*          s_nid = NuSDaS root directory ID                                  */
/*  input:  type1 = target type1 string ( model necessary )                   */
/*          type2 = target type2 string ( attribute necessary )               */
/*          type3 = target type3 string ( type name )                         */
/*  return: fp    = file pointer to the target NuSDaS define file             */
/*          NULL  = open error                                                */
/* -------------------------------------------------------------------------- */
{
    char ename[256], *s;
    N_SI4 num, ns, ne;
    FILE *fp;

    fp = NULL;
    if (nrd_num == 0) {
	strcpy(nrd, ".");
	fp = n_open_def_main(nrd, type1, type2, type3);
	num = 0;
    } else {
	ns = (nrd_num == -1) ? 1 : nrd_num;
	ne = (nrd_num == -1) ? 99 : nrd_num;
	num = ns;
	while (num <= ne) {
	    /* environment variable support */
	    sprintf(ename, "NUSDAS%2.2d", num);
	    if ((s = getenv(ename)) != NULL) {
		if ((fp = n_open_def_main(s, type1, type2, type3)) != NULL) {
		    /*  nrd ݒ肵ȂƎȍ~Ɏs */
		    strcpy(nrd, s);
		    break;
		}
	    } else {
		sprintf(nrd, "./NUSDAS%2.2d", num);
		if ((fp = n_open_def_main(nrd, type1, type2, type3)) !=
		    NULL) break;
	    }
	    num++;
	}
    }
    if (fp == NULL) {
	N_ERP2( "Can not find NUSDAS root directory for selected type1-3\n");
	errprintf( "type1<%s> type2<%s> type3<%s>\n", type1, type2,
		type3);
    } else
	*s_nid = num;

    return (fp);
}


/* -------------------------------------------------------------------------- */
void
n_replace_string(char *dn, const char *type1, const char *type2,
		 const char *type3, N_SI4 basetime, char *member,
		 N_SI4 validtime1)
/* -------------------------------------------------------------------------- */
/* convert general directory name to real directory name                      */
/*  both:  dn      = each directory name in the path                          */
/*  input: type1   = type1 string                                             */
/*         type2   = type2 string                                             */
/*         type3   = type3 string                                             */
/*         basetime= basetime ( total minutes )                               */
/*         member  = member string                                            */
/*         validtime1= (start) validtime ( total minutes )                    */
/* -------------------------------------------------------------------------- */
{
    N_SI4 py, pm, pd, ph, pn, i;

    if (strcmp(dn, "_model") == 0 || strcmp(dn, "_MODEL") == 0) {
	for (i = 0; i < 4; i++)
	    *(dn + i) = *(type1 + i);
	*(dn + i) = '\0';
    }
	else if (strcmp(dn, "_attribute") == 0
		 || strcmp(dn, "_ATTRIBUTE") == 0) {
	for (i = 0; i < 2; i++)
	    *(dn + i) = *(type2 + i);
	*(dn + i) = '\0';
    } else if (strcmp(dn, "_space") == 0 || strcmp(dn, "_SPACE") == 0) {
	for (i = 0; i < 4; i++)
	    *(dn + i) = *(type1 + i + 4);
	*(dn + i) = '\0';
    } else if (strcmp(dn, "_2d") == 0 || strcmp(dn, "_2D") == 0) {
	for (i = 0; i < 2; i++)
	    *(dn + i) = *(type1 + i + 4);
	*(dn + i) = '\0';
    } else if (strcmp(dn, "_3d") == 0 || strcmp(dn, "_3D") == 0) {
	for (i = 0; i < 2; i++)
	    *(dn + i) = *(type1 + i + 6);
	*(dn + i) = '\0';
    } else if (strcmp(dn, "_time") == 0 || strcmp(dn, "_TIME") == 0) {
	for (i = 0; i < 2; i++)
	    *(dn + i) = *(type2 + i + 2);
	*(dn + i) = '\0';
    } else if (strcmp(dn, "_name") == 0 || strcmp(dn, "_NAME") == 0) {
	for (i = 0; i < 4; i++)
	    *(dn + i) = *(type3 + i);
	*(dn + i) = '\0';
    }
	else if (strcmp(dn, "_basetime") == 0
		 || strcmp(dn, "_BASETIME") == 0) {
	nwp_seq2ymdhm(&py, &pm, &pd, &ph, &pn, basetime);
	sprintf(dn, "%4.4d%2.2d%2.2d%2.2d%2.2d", py, pm, pd, ph, pn);
    } else if (strcmp(dn, "_member") == 0 || strcmp(dn, "_MEMBER") == 0) {
	for (i = 0; i < 4; i++)
	    *(dn + i) = *(member + i);
	*(dn + i) = '\0';
    }
	else if (strcmp(dn, "_validtime") == 0
		 || strcmp(dn, "_VALIDTIME") == 0) {
	nwp_seq2ymdhm(&py, &pm, &pd, &ph, &pn, validtime1);
	sprintf(dn, "%4.4d%2.2d%2.2d%2.2d%2.2d", py, pm, pd, ph, pn);
    }
}


/* -------------------------------------------------------------------------- */
N_SI4 n_keyword_set(char *kw)
/* -------------------------------------------------------------------------- */
{
    if (strcmp(kw, "NUSDAS") == 0)
	return (N_KW_NUSDAS);
    else if (strcmp(kw, "PATH") == 0)
	return (N_KW_PATH);
    else if (strcmp(kw, "FILENAME") == 0)
	return (N_KW_FILENAME);
    else if (strcmp(kw, "TYPE1") == 0)
	return (N_KW_TYPE1);
    else if (strcmp(kw, "TYPE2") == 0)
	return (N_KW_TYPE2);
    else if (strcmp(kw, "TYPE3") == 0)
	return (N_KW_TYPE3);
    else if (strcmp(kw, "MEMBER") == 0)
	return (N_KW_MEMBER);
    else if (strcmp(kw, "MEMBERLIST") == 0)
	return (N_KW_MEMBERLIST);
    else if (strcmp(kw, "VALIDTIME") == 0)
	return (N_KW_VALID_NUM);
    else if (strcmp(kw, "VALIDTIME1") == 0)
	return (N_KW_VALIDTIME1);
    else if (strcmp(kw, "VALIDTIME2") == 0)
	return (N_KW_VALIDTIME2);
    else if (strcmp(kw, "PLANE") == 0)
	return (N_KW_PLANE_NUM);
    else if (strcmp(kw, "PLANE1") == 0)
	return (N_KW_PLANE1);
    else if (strcmp(kw, "PLANE2") == 0)
	return (N_KW_PLANE2);
    else if (strcmp(kw, "ELEMENT") == 0)
	return (N_KW_ELEMENT_NUM);
    else if (strcmp(kw, "ELEMENTMAP") == 0)
	return (N_KW_ELEMENT_MAP);
    else if (strcmp(kw, "SIZE") == 0)
	return (N_KW_SIZE);
    else if (strcmp(kw, "BASEPOINT") == 0)
	return (N_KW_BASEPOINT);
    else if (strcmp(kw, "DISTANCE") == 0)
	return (N_KW_DISTANCE);
    else if (strcmp(kw, "STANDARD") == 0)
	return (N_KW_STANDARD);
    else if (strcmp(kw, "OTHERS") == 0)
	return (N_KW_OTHERS);
    else if (strcmp(kw, "VALUE") == 0)
	return (N_KW_VALUE);
    else if (strcmp(kw, "PACKING") == 0)
	return (N_KW_PACKING);
    else if (strcmp(kw, "MISSING") == 0)
	return (N_KW_MISSING);
    else if (strcmp(kw, "INFORMATION") == 0)
	return (N_KW_INFORMATION);
    else if (strcmp(kw, "SUBCNTL") == 0)
	return (N_KW_SUBCNTL);
    else if (strcmp(kw, "BASETIME") == 0)
	return (N_KW_BASETIME);
    else if (strcmp(kw, "CREATOR") == 0)
	return (N_KW_CREATOR);
    else if (strcmp(kw, "FORCEDRLEN") == 0)
	return (N_KW_FORCED_RLEN);
    else
	return (N_KW_CONTINUE);

}


/* -------------------------------------------------------------------------- */
void n_init_def(N_ST_DEF * def)
/* -------------------------------------------------------------------------- *//* initialize members belong to a struct def                                  *//* -------------------------------------------------------------------------- */
{
    FILE *fp;
    char *nus_sysdir, fname[N_LINE_MAX];

    strcpy(n_routine_name[n_routine_d++], "init_def");

    if ((nus_sysdir = getenv("NUSDAS_DIR")) == NULL) {
	sprintf(fname, "%s/version", N_SYSDEF_DIR);
    } else {
	sprintf(fname, "%s/Sysdef/version", nus_sysdir);
    }
    if ((fp = fopen(fname, "r")) == NULL) {
	def->version = 1;
	N_ERP1("open (%s) failed: version assumed to 1.\n", fname);
    } else {
	fscanf(fp, "%d", &(def->version));
	fclose(fp);
    }
    def->path = N_DEFAULT;
    def->pathname[0] = '\0';
    def->filename[0] = '\0';
    def->creator[0] = '\0';
    def->type1[0] = '\0';
    def->type2[0] = '\0';
    def->type3[0] = '\0';
    def->infile_dc = N_IN_FILE;
    def->infile_vt = N_IN_FILE;
    def->basetime_t = 0;
    def->vt1 = NULL;
    def->vt2 = NULL;
    def->lv1 = NULL;
    def->lv2 = NULL;
    def->el = NULL;
    def->elmap = NULL;
    def->n_infoc = 0;
    def->n_subc = 0;
    def->forced_rlen = 0;
    def->n_dc = 1;
    if ((def->dc = (N_UI4 *) malloc(sizeof(N_UI4) * def->n_dc)) == NULL) {
	N_ERP2("malloc failure for member <%d>\n", def->n_dc);
    } else
	memcpy(def->dc, "    ", 4);
    memcpy(&(def->miss), n_miss_string[0], 4);
    memcpy(&(def->gpv_mean), n_mean_string[0], 4);
    memcpy(&(def->pack), n_pack_string[0], 4);
    memcpy(&(def->time_unit), n_tunit_string[0], 4);
    def->x0 = 0;
    def->y0 = 0;
    def->xd = 0;
    def->yd = 0;
    def->lat_0 = 0;
    def->lon_0 = 0;
    def->lat_1 = 0;
    def->lon_1 = 0;
    def->lat_2 = 0;
    def->lon_2 = 0;
    def->lat_s1 = 0;
    def->lon_s1 = 0;
    def->lat_s2 = 0;
    def->lon_s2 = 0;
    n_routine_d--;

}


/* -------------------------------------------------------------------------- */
N_SI4
n_setpath(char flag, char *nrd, N_ST_DEF * def, char *type1, char *type2,
	  char *type3, N_SI4 * basetime, char *member, N_SI4 * validtime1,
	  char *path, char *filename)
/* -------------------------------------------------------------------------- */
/* create and set the NuSDaS file system based on the define file             */
/*  input:  flag      = action mode ('r':read only, 'c':create directory,     */
/*                                   'v':virtural struct)                     */
/*          nrd       = NuSDaS root directory                                 */
/*          def       = define item struct read from NuSDaS define file       */
/*          type1     = type1 string                                          */
/*          type2     = type2 string                                          */
/*          type3     = type3 string                                          */
/*          basetime  = basetime ( total minutes )                            */
/*          member    = member string                                         */
/*          validtime1= (start) validtime ( total minutes )                   */
/*  output: path      = set path name ( if necessary, created )               */
/*          filename  = set file name                                         */
/*  return: 1         = normal end (new create nusdas file system)            */
/*          0         = normal end (already created)                          */
/*         -1         = file system construct error                           */
/* -------------------------------------------------------------------------- */
{
    char line[N_LINE_MAX], path2[N_LINE_MAX], s1[256], s2[256];
    char dn[32][N_LINE_MAX], cc[3][N_LINE_MAX], dum[N_LINE_MAX],
	rule[N_LINE_MAX];
    N_SI4 i, j, k, py, pm, pd, ph, pn, item, layer, rt = 0;
    mode_t mode;
    DIR *dp;
    N_ST_CNTL dum_cntl;

    strcpy(n_routine_name[n_routine_d++], "setpath");

    mode = N_DIR_MODE;
    *path = '\0';
    path2[0] = '\0';

    if (def->path == N_NWP_ESF) {
	if (strlen(def->filename) > 0)
	    strcpy(filename, def->filename);
	else {
	    N_ERP2( "filename is not specified in define file for ESF use!\n");
	    N_ERTN(-2);
	}
	strcpy(path, "n_nwp_esf");
	n_routine_d--;
	return (0);
    }
	else if (def->path == N_NWP_PATH_S || def->path == N_NWP_PATH_M ||
		 def->path == N_NWP_PATH_BS || def->path == N_NWP_PATH_VM) {
	sscanf(def->type1, "%s %s %s %s", dum, dum, dum, cc[0]);
	sscanf(def->type3, "%s %s", dum, cc[1]);
	sprintf(dn[0], "%2.2s%4.4s", cc[0], cc[1]);
	layer = 1;
	if (def->path == N_NWP_PATH_M || def->path == N_NWP_PATH_VM) {
	    for (i = 0; i < def->n_dc; i++)
		if (memcmp(member, (char *) (def->dc + i), 4) == 0)
		    break;
	    if (i == def->n_dc) {
		N_ERP2( "member <%s> is not registered in define files\n",
		       member);
		N_ERTN(-1);
	    }
	    memcpy(dn[layer], member, 4);
	    dn[layer++][4] = '\0';
	}
	if (n_init_or_check_validtime(def, &dum_cntl, basetime,
				      validtime1, 0) < 0) {
	    n_timeprint(s1, *basetime);
	    n_timeprint(s2, *validtime1);
	    N_ERP2("basetime <%s> validtime <%s> is not found\n",
		   s1, s2);
	    N_ERTN(-1);
	}
	if (def->path == N_NWP_PATH_S || def->path == N_NWP_PATH_M) {
	    nwp_seq2ymdhm(&py, &pm, &pd, &ph, &pn, *validtime1);
	    sprintf(dn[layer++], "%4.4d%2.2d%2.2d%2.2d%2.2d",
		    py, pm, pd, ph, pn);
	} else if (def->path == N_NWP_PATH_BS) {
	    nwp_seq2ymdhm(&py, &pm, &pd, &ph, &pn, *basetime);
	    sprintf(dn[layer++], "%4.4d%2.2d%2.2d%2.2d%2.2d",
		    py, pm, pd, ph, pn);
	}
	strcpy(filename, dn[--layer]);
    } else {
	if (def->path == N_RELATIVE_PATH) {
	    i = 0;
	    layer = 0;
	    k = 0;
	    while (1) {
		if (def->pathname[i] == '\0')
		    break;
		else if (def->pathname[i] == '/') {
		    dn[layer++][k] = '\0';
		    k = 0;
		} else
		    dn[layer][k++] = def->pathname[i];
		i++;
	    }
	    if (def->pathname[i - 1] != '/')
		dn[layer++][k] = '\0';
	} else if (def->path == N_DEFAULT) {
	    sscanf(def->type1, "%s %s %s %s", dum, dn[0], cc[0], cc[1]);
	    strcpy(dn[2], cc[0]);
	    strcat(dn[2], cc[1]);
	    sscanf(def->type2, "%s %s %s", dum, dn[1], dn[3]);
	    sscanf(def->type3, "%s %s", dum, dn[4]);
	    nwp_seq2ymdhm(&py, &pm, &pd, &ph, &pn, *basetime);
	    sprintf(dn[5], "%4.4d%2.2d%2.2d%2.2d%2.2d", py, pm, pd, ph,
		    pn);
	    layer = 6;
	    if (def->infile_dc == N_OUT_FILE) {
		for (i = 0; i < def->n_dc; i++)
		    if (memcmp(member, (char *) (def->dc + i), 4) == 0)
			break;
		if (i == def->n_dc) {
		    N_ERP2( "member <%s> is not registered in define files\n",
			   member);
		    N_ERTN(-1);
		}
		memcpy(dn[layer], member, 4);
		dn[layer++][4] = '\0';
	    }
	    if (def->infile_vt == N_OUT_FILE) {
		if (n_init_or_check_validtime(def, &dum_cntl,
					      basetime, validtime1, 0) < 0) {
		    n_timeprint(s1, *basetime);
		    n_timeprint(s2, *validtime1);
		    N_ERP2( "basetime <%s> validtime <%s> is not found\n",
			   s1, s2);
		    N_ERTN(-1);
		}
		nwp_seq2ymdhm(&py, &pm, &pd, &ph, &pn, *validtime1);
		sprintf(dn[layer++], "%4.4d%2.2d%2.2d%2.2d%2.2d",
			py, pm, pd, ph, pn);
	    }
	}
	for (i = 0; i < layer; i++)
	    n_replace_string(dn[i], type1, type2, type3,
			     *basetime, member, *validtime1);
	if (strlen(def->filename) > 0)
	    strcpy(filename, def->filename);
	else
	    strcpy(filename, dn[--layer]);
	n_replace_string(filename, type1, type2, type3,
			 *basetime, member, *validtime1);
    }
    n_space_string(filename, "_");

    /* construct nusdas file system */
    strcpy(path2, nrd);
    for (i = 0; i < layer; i++) {
	strcpy(path2, nrd);
	for (j = 0; j <= i; j++) {
	    strcat(path2, "/");
	    strcat(path2, dn[j]);
	}
	n_space_string(path2, "_");
	if (n_char_set)
	    n_char_upper(path2);
	if (flag != 'v' && flag != 'V') {
	    if ((dp = opendir(path2)) == NULL) {
		if (errno == ENOENT) {
		    if (flag == 'c' || flag == 'C') {
			if (mkdir(path2, mode) != 0) {
			    errsys("");
			    N_ERP2( "Can not create directory (%s)\n",
				   path2);
			} else {
			    rt = 1;
			    continue;
			}
		    } else {
			N_ERP2("Can not find path (%s)\n", path2);
		    }
		} else {
		    N_ERP2("Can not open path (%s)\n", path2);
		}
		*filename = '\0';
		N_ERTN(-2);
	    }
	    closedir(dp);
	} else {
	    rt = 0;
	}
    }
    strcpy(path, path2);
    n_routine_d--;

    return (rt);
}
