#define N_DEF_GETDATA_SOURCE
#include "nusdas_sys.h"

/* -------------------------------------------------------------------------- */
N_SI4
n_get_data(FILE * fp, long offset, N_ST_CNTL * s_cntl,
	   const char *descriptor, N_SI4 * validtime1, N_SI4 * validtime2,
	   const char *level1, const char *level2, const char *element,
	   void *data, const char *dtype, const N_SI4 * dnum)
/* -------------------------------------------------------------------------- */
/* read the DATA record and set it to a user array                            */
/*  input: data  = user data array                                            */
/*         dtype = data variable type                                         */
/*         dnum  = allocated item number of user data array                   */
/*  return:  >=0 = data item number of setting to user array                  */
/*            -3 = kind on DATA rec. is not consistent with that on CNTL rec. */
/*            -4 = allocated user array is insufficient size                  */
/*            -5 = specified dtype is not supported for this data packing     */
/*            -6 = data is bad in runlength                                   */
/*           -10 = malloc error                                               */
/*           -99 = file IO error                                              */
/* -------------------------------------------------------------------------- */
{
    char s[] = "DATA", s1[256], s2[256];
    N_ST_DATA s_data;
    float *d_f;
    N_SI4 i, j, ng, tbyte, dbyte, mbyte, rt, maxv, nbit, num, *d_i;
    long offset_b2;
    N_UI2 n_ui2;
    N_UI1 dum = 0, p_p, *p_miss, *p_mask, *mb, *wp;
    char	format[256];

    s_data.d = NULL;
    if (fseek(fp, offset, SEEK_SET) != 0)
	return (-99);
    if ((rt = fread_fmt(&s_data, N_ST_DATA_FORMAT, fp)) != N_SIZE_STDATA) {
	return (-99);
    }
    if (s_data.d) errprintf( "#n_get_data: s_data.d contaminated\n");

    if (memcmp(&s_data.dc, descriptor, 4) != 0 || s_data.vt1 != *validtime1
	|| s_data.vt2 != *validtime2 || memcmp(&s_data.lvel[0], level1, 6)
	!= 0 || memcmp(&s_data.lvel[6], level2, 6) != 0
	|| memcmp(&s_data.lvel[12], element, 6) != 0) {
	N_ERP2("parameters in DATA record is not consistent\n");
	errprintf( "<compare>   DATA rec <-> your request\n");
	errprintf( "member    :   %4.4s   <->   %4.4s\n",
		(char *) (&s_data.dc), descriptor);
	n_timeprint(s1, s_data.vt1);
	n_timeprint(s2, *validtime1);
	errprintf( "validtime1: %s <-> %s\n", s1, s2);
	n_timeprint(s1, s_data.vt2);
	n_timeprint(s2, *validtime2);
	errprintf( "validtime2: %s <-> %s\n", s1, s2);
	errprintf( "level1    :  %6.6s  <->  %6.6s\n",
		(char *) (&s_data.lvel[0]), level1);
	errprintf( "level2    :  %6.6s  <->  %6.6s\n",
		(char *) (&s_data.lvel[6]), level2);
	errprintf( "element   :  %6.6s  <->  %6.6s\n",
		(char *) (&s_data.lvel[12]), element);
	return (-3);
    }
    /* for beta02 version dataset */
    for (i = 0; i < n_miss_num; i++)
	if (memcmp(&(s_data.miss), n_miss_string[i], 4) == 0)
	    break;
    if (i == n_miss_num) {
	ng = s_cntl->f.xs * s_cntl->f.ys;
	memcpy(&s_data.pack, &s_data.xs, 4);
	memcpy(&s_data.miss, &s_data.ys, 4);
	offset_b2 = (long) sizeof(int) * (-2);
	fseek(fp, offset_b2, SEEK_CUR);
    } else {

	ng = s_data.xs * s_data.ys;

    }
    /* for beta02 version dataset */

    if (*dnum < ng) {
	N_ERP2("number of your data array <%d> is small.\n",
	       *dnum);
	errprintf( "necessary number is %d.\n", ng);
	return (-4);
    }

    p_miss = NULL;
    p_mask = NULL;
    if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	mbyte = n_mbyte_set(&s_data, &p_p, NULL, &ng);
	if (n_alloc_dp(&p_mask, mbyte) < 0)
	    return (-10);
	fread(p_mask, 1, mbyte, fp);
    } else if (memcmp(&s_data.miss, n_miss_string[1], 4) == 0) {
	mbyte = n_mbyte_set(&s_data, NULL, &p_p, &ng);
	if (n_alloc_dp(&p_miss, mbyte) < 0)
	    return (-10);
	fread_native(p_miss, mbyte, 1, fp);
    }


    rt = 0;
    if (memcmp(&s_data.pack, N_P_1PAC, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	    i = s_data.h.available - 15 * 4 - mbyte;
	    sprintf(format, "%ldb", (long)i);
	} else {
	    i = sizeof(N_UI1) * ng + sizeof(float) * 2;
	    sprintf(format, "2f%ldb", (long)ng);
	}
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (fread_fmt(s_data.d, format, fp) != i)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_1pac_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_1pac_i4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    n_decode_1pac_i2(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
    } else if (memcmp(&s_data.pack, N_P_2PAC, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	    i = s_data.h.available - 15 * 4 - mbyte;
	    sprintf(format, "%ldh", (long)i / 2);
	} else {
	    i = sizeof(N_SI2) * ng + sizeof(float) * 2;
	    sprintf(format, "2f%ldh", (long)ng);
	}
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (fread_fmt(s_data.d, format, fp) != i)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_2pac_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R8, 2) == 0)
	    n_decode_2pac_r8(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_2pac_i4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_NC, 2) == 0)
	    n_decode_2pac_nc(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
    } else if (memcmp(&s_data.pack, N_P_2UPAC, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	    i = s_data.h.available - 15 * 4 - mbyte;
	    sprintf(format, "%ldh", (long)i / 2);
	} else {
	    i = sizeof(N_SI2) * ng + sizeof(float) * 2;
	    sprintf(format, "2f%ldh", (long)ng);
	}
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (fread_fmt(s_data.d, format, fp) != i)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_2upac_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R8, 2) == 0)
	    n_decode_2upac_r8(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
    } else if (memcmp(&s_data.pack, N_P_4PAC, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	    i = s_data.h.available - 15 * 4 - mbyte;
	    sprintf(format, "%ldi", (long)i / 4);
	} else {
	    i = sizeof(N_SI4) * ng + sizeof(double) * 2;
	    sprintf(format, "2d%ldi", (long)ng);
	}
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (fread_fmt(s_data.d, format, fp) != i)
	    rt = -99;
	else if (strncmp(dtype, N_R8, 2) == 0)
	    n_decode_4pac_r8(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_4pac_r4(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
    } else if (memcmp(&s_data.pack, N_P_R4, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	    i = s_data.h.available - 15 * 4 - mbyte;
	    sprintf(format, "%ldf", (long)i / 4);
	} else {
	    i = sizeof(float) * ng;
	    sprintf(format, "%ldf", (long)ng);
	}
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (fread_fmt(s_data.d, format, fp) != i)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_r4_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R8, 2) == 0)
	    n_decode_r4_r8(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
    } else if (memcmp(&s_data.pack, N_P_R8, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	    i = s_data.h.available - 15 * 4 - mbyte;
	    sprintf(format, "%ldd", (long)i / 8);
	} else {
	    i = sizeof(double) * ng;
	    sprintf(format, "%ldd", (long)ng);
	}
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (fread_fmt(s_data.d, format, fp) != i)
	    rt = -99;
	else if (strncmp(dtype, N_R8, 2) == 0)
	    n_decode_r8_r8(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_r8_r4(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
    } else if (memcmp(&s_data.pack, N_P_I1, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	    i = s_data.h.available - 15 * 4 - mbyte;
	    sprintf(format, "%ldb", (long)i);
	} else {
	    i = sizeof(N_UI1) * ng;
	    sprintf(format, "%ldb", (long)ng);
	}
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (fread_fmt(s_data.d, format, fp) != i)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_i1_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_i1_i4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    n_decode_i1_i2(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I1, 2) == 0)
	    n_decode_i1_i1(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
    } else if (memcmp(&s_data.pack, N_P_I2, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	    i = s_data.h.available - 15 * 4 - mbyte;
	    sprintf(format, "%ldh", (long)i / 2);
	} else {
	    i = sizeof(N_SI2) * ng;
	    sprintf(format, "%ldh", (long)ng);
	}
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (fread_fmt(s_data.d, format, fp) != i)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_i2_r4(data, s_data.d, ng, p_miss, p_mask, 1);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_i2_i4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    n_decode_i2_i2(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
    } else if (memcmp(&s_data.pack, N_P_N1I2, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	    i = s_data.h.available - 15 * 4 - mbyte;
	    sprintf(format, "%ldh", (long)i / 2);
	} else {
	    i = sizeof(N_SI2) * ng;
	    sprintf(format, "%ldh", (long)ng);
	}
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (fread_fmt(s_data.d, format, fp) != i)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_i2_r4(data, s_data.d, ng, p_miss, p_mask, 10);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_i2_i4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    n_decode_i2_i2(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
    } else if (memcmp(&s_data.pack, N_P_I4, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	    i = s_data.h.available - 15 * 4 - mbyte;
	    sprintf(format, "%ldi", (long)i / 4);
	} else {
	    i = sizeof(N_SI4) * ng;
	    sprintf(format, "%ldi", (long)ng);
	}
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (fread_fmt(s_data.d, format, fp) != i)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_i4_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_i4_i4(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
    } else if (memcmp(&s_data.pack, N_P_RLEN, 4) == 0) {
	if (fread_native(&nbit, sizeof(N_SI4), 1, fp) != 1)
	    rt = -99;
	else if (fread_native(&maxv, sizeof(N_SI4), 1, fp) != 1)
	    rt = -99;
	else if (fread_native(&num, sizeof(N_SI4), 1, fp) != 1)
	    rt = -99;
	else if (n_alloc_dp(&s_data.d, sizeof(N_UI1) * num) < 0)
	    rt = -10;
	else if (fread(s_data.d, sizeof(N_UI1) * num, 1, fp) != 1)
	    rt = -99;
	else if (strncmp(dtype, N_I4, 2) == 0) {
	    if ((i = n_decode_rlen_8bit(data, s_data.d, num, ng, maxv)) !=
		ng) rt = i;
	}else if (strncmp(dtype, N_I1, 2) == 0) {
	    if ((i = n_decode_rlen_8bit_I1(data, s_data.d, num, ng, maxv)) !=
		ng) rt = i;
	} else if (strncmp(dtype, N_R4, 2) == 0) {
	    if ((i = n_decode_rlen_8bit(data, s_data.d, num, ng, maxv)) !=
		ng) rt = i;
	    d_f = (float *) data;
	    d_i = (N_SI4 *) data;
	    for (i = 0; i < ng; i++) {
		j = d_i[i];
		d_f[i] = (float) j;
	    }
	} else if (strncmp(dtype, N_NC, 2) == 0) {
	    wp = data;
	    memcpy((wp + 0), &nbit, 4);
	    memcpy((wp + 4), &maxv, 4);
	    memcpy((wp + 8), &num, 4);
	    memcpy((wp + 12), s_data.d, num * sizeof(N_UI1));
	    rt = num + 4 * 3;
	} else
	    rt = -5;
    } else {
	rt = -5;
    }

    if (s_data.d != NULL)
	free(s_data.d);
    if (p_mask != NULL)
	free(p_mask);
    if (p_miss != NULL)
	free(p_miss);

    if (rt == 0)
	rt = ng;
    else if (rt == -5) {
	N_ERP2( "your data type <%s> is not supported for <%4.4s> decoding\n",
	       dtype, (char *) &s_data.pack);
    }
    return (rt);
}
