#define N_DEF_GETESF_SOURCE
#include "nusdas_sys.h"

/* -------------------------------------------------------------------------- */
N_SI4
n_get_esf(N_UI1 * sd, N_ST_CNTL * s_cntl,
	  const char *descriptor, N_SI4 * validtime1, N_SI4 * validtime2,
	  const char *level1, const char *level2, const char *element,
	  void *data, const char *dtype, const N_SI4 * dnum)
/* -------------------------------------------------------------------------- */
/* read the DATA record and set it to a user array                            */
/*  input: data  = user data array                                            */
/*         dtype = data variable type                                         */
/*         dnum  = allocated item number of user data array                   */
/*  return:  >=0 = data item number of setting to user array                  */
/*            -3 = kind on DATA rec. is not consistent with that on CNTL rec. */
/*            -4 = allocated user array is insufficient size                  */
/*            -5 = specified dtype is not supported for this data packing     */
/*            -6 = data is bad in runlength                                   */
/*           -10 = malloc error                                               */
/*           -99 = memory IO error                                            */
/* -------------------------------------------------------------------------- */
{
    char s[] = "DATA";
    N_ST_DATA s_data;
    float *d_f;
    N_SI4 i, ng, tbyte, dbyte, mbyte, rt, maxv, nbit, num, sp = 0;
    long offset_b2;
    N_UI2 n_ui2;
    N_UI1 dum = 0, p_p, *p_miss, *p_mask, *mb;

    memcpy(&s_data, (N_UI1 *) (sd + sp), N_SIZE_STDATA);
    sp += N_SIZE_STDATA;
    if (memcmp(&s_data.h.recname, s, 4) != 0) {
	return (0);
    }
    if (memcmp(&s_data.dc, descriptor, 4) != 0 || s_data.vt1 != *validtime1
	|| s_data.vt2 != *validtime2 || memcmp(&s_data.lvel[0], level1, 6)
	!= 0 || memcmp(&s_data.lvel[6], level2, 6) != 0
	|| memcmp(&s_data.lvel[12], element, 6) != 0) {
	N_ERP2("parameters in DATA record is not consistent\n");
	errprintf( "<compare>   DATA rec <-> your request\n");
	errprintf( "member    :   %4.4s   <->   %4.4s\n",
		(char *) (&s_data.dc), descriptor);
	errprintf( "validtime1: %d <-> %d\n", s_data.vt1,
		*validtime1);
	errprintf( "validtime2: %d <-> %d\n", s_data.vt2,
		*validtime2);
	errprintf( "level1    :  %6.6s  <->  %6.6s\n",
		(char *) (&s_data.lvel[0]), level1);
	errprintf( "level2    :  %6.6s  <->  %6.6s\n",
		(char *) (&s_data.lvel[6]), level2);
	errprintf( "element   :  %6.6s  <->  %6.6s\n",
		(char *) (&s_data.lvel[12]), element);
	return (-3);
    }

    ng = s_data.xs * s_data.ys;

    if (*dnum < ng) {
	N_ERP2("number of your data array <%d> is small.\n",
	       *dnum);
	errprintf( "necessary number is %d.\n", ng);
	return (-4);
    }

    p_miss = NULL;
    p_mask = NULL;
    if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0) {
	mbyte = n_mbyte_set(&s_data, &p_p, NULL, &ng);
	if (n_alloc_dp(&p_mask, mbyte) < 0)
	    return (-10);
	memcpy(p_mask, (N_UI1 *) (sd + sp), mbyte);
	sp += mbyte;
    } else if (memcmp(&s_data.miss, n_miss_string[1], 4) == 0) {
	mbyte = n_mbyte_set(&s_data, NULL, &p_p, &ng);
	if (n_alloc_dp(&p_miss, mbyte) < 0)
	    return (-10);
	memcpy(p_miss, (N_UI1 *) (sd + sp), mbyte);
	sp += mbyte;
    }

    rt = 0;
    if (memcmp(&s_data.pack, N_P_1PAC, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0)
	    i = s_data.h.available - 15 * 4 - mbyte;
	else
	    i = sizeof(N_UI1) * ng + sizeof(float) * 2;
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp), i) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_1pac_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_1pac_i4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    n_decode_1pac_i2(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
	sp += i;
    } else if (memcmp(&s_data.pack, N_P_2PAC, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0)
	    i = s_data.h.available - 15 * 4 - mbyte;
	else
	    i = sizeof(N_SI2) * ng + sizeof(float) * 2;
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp), i) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_2pac_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R8, 2) == 0)
	    n_decode_2pac_r8(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_2pac_i4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_NC, 2) == 0)
	    n_decode_2pac_nc(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
	sp += i;
    } else if (memcmp(&s_data.pack, N_P_2UPAC, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0)
	    i = s_data.h.available - 15 * 4 - mbyte;
	else
	    i = sizeof(N_SI2) * ng + sizeof(float) * 2;
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp), i) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_2upac_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R8, 2) == 0)
	    n_decode_2upac_r8(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
	sp += i;
    } else if (memcmp(&s_data.pack, N_P_4PAC, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0)
	    i = s_data.h.available - 15 * 4 - mbyte;
	else
	    i = sizeof(N_SI4) * ng + sizeof(double) * 2;
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp), i) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_R8, 2) == 0)
	    n_decode_4pac_r8(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
	sp += i;
    } else if (memcmp(&s_data.pack, N_P_R4, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0)
	    i = s_data.h.available - 15 * 4 - mbyte;
	else
	    i = sizeof(float) * ng;
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp), i) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_r4_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R8, 2) == 0)
	    n_decode_r4_r8(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
	sp += i;
    } else if (memcmp(&s_data.pack, N_P_R8, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0)
	    i = s_data.h.available - 15 * 4 - mbyte;
	else
	    i = sizeof(double) * ng;
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp), i) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_R8, 2) == 0)
	    n_decode_r8_r8(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
	sp += i;
    } else if (memcmp(&s_data.pack, N_P_I1, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0)
	    i = s_data.h.available - 15 * 4 - mbyte;
	else
	    i = sizeof(N_UI1) * ng;
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp), i) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_i1_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_i1_i4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    n_decode_i1_i2(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I1, 2) == 0)
	    n_decode_i1_i1(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
	sp += i;
    } else if (memcmp(&s_data.pack, N_P_I2, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0)
	    i = s_data.h.available - 15 * 4 - mbyte;
	else
	    i = sizeof(N_SI2) * ng;
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp), i) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_i2_r4(data, s_data.d, ng, p_miss, p_mask, 1);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_i2_i4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    n_decode_i2_i2(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
	sp += i;
    } else if (memcmp(&s_data.pack, N_P_N1I2, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0)
	    i = s_data.h.available - 15 * 4 - mbyte;
	else
	    i = sizeof(N_SI2) * ng;
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp), i) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_i2_r4(data, s_data.d, ng, p_miss, p_mask, 10);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_i2_i4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    n_decode_i2_i2(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
	sp += i;
    } else if (memcmp(&s_data.pack, N_P_I4, 4) == 0) {
	if (memcmp(&s_data.miss, n_miss_string[2], 4) == 0)
	    i = s_data.h.available - 15 * 4 - mbyte;
	else
	    i = sizeof(N_SI4) * ng;
	if (n_alloc_dp(&s_data.d, i) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp), i) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_R4, 2) == 0)
	    n_decode_i4_r4(data, s_data.d, ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    n_decode_i4_i4(data, s_data.d, ng, p_miss, p_mask);
	else
	    rt = -5;
	sp += i;
    } else if (memcmp(&s_data.pack, N_P_RLEN, 4) == 0) {
	if (memcpy(&nbit, (N_UI1 *) (sd + sp), sizeof(N_SI4)) == NULL)
	    rt = -99;
	else if (memcpy(&maxv, (N_UI1 *) (sd + sp + 4), sizeof(N_SI4))
		 == NULL)
	    rt = -99;
	else if (memcpy(&num, (N_UI1 *) (sd + sp + 8), sizeof(N_SI4)) ==
		 NULL) rt = -99;
	else if (n_alloc_dp(&s_data.d, sizeof(N_UI1) * num) < 0)
	    rt = -10;
	else if (memcpy(s_data.d, (N_UI1 *) (sd + sp + 12),
			sizeof(N_UI1) * num) == NULL)
	    rt = -99;
	else if (strncmp(dtype, N_I4, 2) == 0) {
	    if ((i = n_decode_rlen_8bit(data, s_data.d, num, ng, maxv)) !=
		ng) rt = i;
	} else
	    rt = -5;
	sp += (12 + num);
    }

    if (s_data.d != NULL)
	free(s_data.d);
    if (p_mask != NULL)
	free(p_mask);
    if (p_miss != NULL)
	free(p_miss);

    if (rt == 0)
	rt = ng;
    else if (rt == -5) {
	N_ERP2( "your data type <%s> is not supported for <%4.4s> decoding\n",
	       dtype, (char *) &s_data.pack);
    }

    return (rt);
}
