#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <ghttp.h>

#include  "nusdas.h"


static int debug =1;
typedef struct pandora_serv{
    char type1[9];
    char type2[5];
    char type3[5];
    char *host; 
       /* include NRD  ex: http://localhost:8080/NUSDAS/cpd_ll.nus */
} pandora_serv;

static pandora_serv *pserv;
static int serv_num=0;
static int read_file_flag=0;


int set_pdnus_debug_mode() { return debug = 1; }
int reset_pdnus_debug_mode() { return debug = 0; }



/*--------------------------------------------------------------------------*/
int read_server_table()
/*
return value:
   >=0 : Item number of server list (Normal End).
  -1: "PANDORA_SERVER_LIST" is not set;
  -99 : I/O error
  -10 : malloc error;
*/

{
    char *fname;
    char buf[1024];
    int i = 0;
    FILE *fp;
    int array_num = 10;

    if(read_file_flag == 1){
        return serv_num;
    }

    fname = getenv("PANDORA_SERVER_LIST");
    if(fname == NULL){
        fprintf(stderr,"Not specify PANDORA_SERVER_LIST\n");
        return -1;
    }
    if((fp=fopen(fname,"r")) == NULL){
        fprintf(stderr,"Cannot open PANDORA_SERVER_LIST\n");
        return -99;
    }

    pserv=(pandora_serv*)malloc(array_num*sizeof(pserv));
    if(pserv == NULL){
        fprintf(stderr,"Cannot allocate <pserv>\n");
        return -10;
    }

    i = 0;
    while(fgets(buf,1024,fp)){
        char type1[1024],type2[1024],type3[1024],data_type[1024],host[1024];
        char *p;
        int len;
        int n;

        p=buf;
        while(*p != '\0'){
            if(*p == '#' || *p == '\n'){
                *p='\0';
                break;
            }
            p++;
        }
        n=sscanf(buf,"%8s.%4s.%4s%s",type1, type2, type3, host);
        if(n==4){
            strncpy(pserv[i].type1,type1,8);
            strncpy(pserv[i].type2,type2,4);
            strncpy(pserv[i].type3,type3,4);

            len = strlen(host);
            pserv[i].host = (char*)malloc(len+1);
            if(pserv[i].host == NULL){
                return -10;
            }
            strncpy(pserv[i].host,host,len);
            i++;
            if(debug){
                fprintf(stderr,"Server-type: %s,%s,%s,%s\n",
                        type1,type2,type3,host);
            }
        }
        if(i==array_num){
            pandora_serv *p_tmp;
            array_num+=10;
            p_tmp=(pandora_serv*)realloc(pserv,array_num*sizeof(pserv));
            if(p_tmp == NULL){
                fprintf(stderr,"realloc <pserv> failed\n");
                return -10;
            }
            else{
                pserv=p_tmp;
            }
        }

    }
    fclose(fp);
    read_file_flag = 1;
    serv_num = i;
    return i;
}
/*--------------------------------------------------------------------------*/
N_SI4 nusdas_read_new(const char *type1, const char *type2, const char *type3,
                N_SI4 *base, const char *kind, N_SI4 *valid, 
                const char *layer,const char *physical, void *data,
                const char *fmt, const N_SI4 *size){
    int rt,rt2,rt3;

    rt=nusdas_read(type1,type2,type3,base,kind,valid,layer,physical,
                         data,fmt,size);

    if(rt>0){
        return rt;
    }
    if(debug){
        if(getenv("PANDORA_SERVER_LIST")!=NULL){
            fprintf(stderr,"Then, try to get data from Pandora Server.\n");
        }
    }
    rt2=read_server_table();
    if(rt2 <= 0){
        return rt;
    }
    rt3=pandora_nusdas_read(type1,type2,type3,base,kind,valid,layer,physical,
                         data,fmt,size);
    if( rt3 < 0){
        return rt + rt3*100;
    }
    return rt3;
}
/*--------------------------------------------------------------------------*/
N_SI4 nusdas_subc_srf_new(const char *type1, const char *type2, 
                          const char *type3,
                N_SI4 *base, const char *kind, N_SI4 *valid, 
                const char *layer,const char *physical, const char *gname,
                N_SI4 *data, char *op){

    int rt,rt2,rt3;
    
    rt = nusdas_subc_srf(type1,type2,type3,base,kind,valid,layer,physical,
                         gname,data,op);

    if(rt == 0){
        return rt;
    }
    if(debug){
        if(getenv("PANDORA_SERVER_LIST")!=NULL){
            fprintf(stderr,"Then, try to get data from Pandora Server.\n");
        }
    }
    rt2 = read_server_table();
    if(rt2 <= 0){
        return rt;
    }

    rt3 = pandora_nusdas_subc_srf(type1,type2,type3,
                                  base,kind,valid,layer,physical,
                                  gname,data,op);
    if(rt3 < 0){
        return rt+rt3*100;
    }
    return rt3;
}

/*--------------------------------------------------------------------------*/
typedef struct gheader_s { char *label, *value; } GHEADER;
/*------------------------------------------------------------
    phT[oɃf[^v
 *------------------------------------------------------------*/
static N_SI4          /*  Return code > 0 :               */
get_body_ghttp (      /*                                      */
/*-------------------------------------------------------------*/
        char     *uri,              /*(in)  http://.../URI    */
        GHEADER  *ghead,            /*(in)  http header lists */
        void     *data              /*(out) http body         */
) {
    ghttp_request *request = NULL;
    int            len, rt;


    request = ghttp_request_new ();

    rt = ghttp_set_uri (request, uri);
    if (debug)printf ("ghttp_set_uri() = %d\n", rt);


    if (ghead != NULL) {
        GHEADER *p = ghead;
        while (p->label && p->value) {
            ghttp_set_header (request, p->label, p->value);
            if (debug) printf ("ghttp_set_header(%s,%s)\n", p->label, p->value);
            ++p;
        }
    }

    rt = ghttp_prepare (request);
    if (debug) printf ("ghttp_prepare() = %d\n", rt);

    rt = ghttp_process (request);    /*  ۂ̏  */
    if (debug) printf ("ghttp_process() = %d\n", rt);
    if (debug) printf ("    status code = %d\n", ghttp_status_code(request));
    if (rt < 0) return rt;

    if ((rt = ghttp_status_code(request)) != 200) {
        if (debug) printf("%s\n", ghttp_get_error(request));
        return -rt;
    } else {
        if (debug) printf("%s\n", ghttp_reason_phrase(request));
    }

    len = ghttp_get_body_len(request);
    if (debug) printf ("ghttp_get_body_len() = %d\n", len);

    memcpy (data, ghttp_get_body(request), len);

    ghttp_request_destroy (request);
    if (debug) printf ("ghttp_request_destroy() \n");

    return  len;
}


/*----------------------------------------------------------------------*/
static struct nus_dims_t *
set_nus_dims (const char  *type1, const char *type2, const char  *type3,
	N_SI4 *base, const char *kind,  N_SI4 *valid,  N_SI4 *valid2,
        const char  *layer, const char *physical, struct nus_dims_t *dims)
/*-----------------------------------------------------------------------*/
{
    static struct  nus_dims_t   d;

    memset (&d, 0, sizeof(d));
    strncpy (d.type.type1, type1, sizeof(d.type.type1));
    strncpy (d.type.type2, type2, sizeof(d.type.type2));
    strncpy (d.type.type3, type3, sizeof(d.type.type3));
    d.base = *base;
    strncpy(d.member,kind,sizeof(d.member));
    d.valid  = *valid;
    d.valid2 = *valid2;
    strncpy (d.plane, layer, sizeof(d.plane));
    strncpy (d.plane2, layer, sizeof(d.plane2));
    strncpy (d.element, physical, sizeof(d.element));

    if (dims != NULL) *dims = d;

    return  &d;
}
/*------------------------------------------------------------------------*/
static N_SI4
nusdas_read_net (char *url, void *data, const char *fmt, const N_SI4 *size)
/*------------------------------------------------------------------------*/
{
    char    uri[256];
    int     i, rt;
    int     items = 0;

    static GHEADER ghead[] = {       {http_hdr_Connection, "keep-alive"}
                                     /*  ,{http_hdr_Accept,"text/html"}*/  
                                    ,{NULL,NULL}
                             };


    if (debug) printf("request URL:%s \n", url);


    if (strcmp(fmt,N_R4) == 0) {
        sprintf (uri,"%s/data.f32", url);
        if (debug) printf ("URI %s.\n", uri);

        items = get_body_ghttp (uri, ghead, data) / 4;
        if(items>0){
            endian_swab4(data, items);
        }
    } else if (strcmp(fmt,N_I1) == 0) {
        sprintf (uri,"%s/data.bin", url);
        if (debug) printf ("URI %s.\n", uri);

        items = get_body_ghttp (uri, ghead, data);

    }

    return  items;
}


/*-----------------------------------------------------------------------*/
static N_SI4
nusdas_subc_srf_net (char *url, const char *gname, N_SI4 *data, char *get)
/*-----------------------------------------------------------------------*/
{
    char    uri[256];
    int     i;
    int     rt = -1;

    static GHEADER ghead[] = {       {http_hdr_Connection, "keep-alive"}
                                /*  ,{http_hdr_Accept,"text/html"}  */
                                    ,{NULL,NULL}
                             };


    if (debug) printf("nus_path:%s \n", url);

    if (strcmp(get,N_IO_GET) != 0) {
        fprintf (stderr, "W! nusdas_subc_srf() support only N_IO_GET\n");
        return -5;
    }


    if (strcmp(gname,"ISPC") == 0) {

        sprintf (uri,"%s/data.i32", url);
        if (debug) printf ("URI %s.\n", uri);

        rt = get_body_ghttp (uri, ghead, data) == 512 ? 0 : -2;
        if(rt==0){
            endian_swab4(data,512/4);
        }

    } else {
        fprintf (stderr, "W! nusdas_subc_srf() not support %s.\n", gname);
    }

    return  rt;
}


#ifdef DUMMY_FUNCTION
/*--------------------------------------------------------------------------*/
static N_SI4
nusdas_subc_srf_local (const char  *type1, const char *type2, const char *type3,
	N_SI4 *base, const char *kind,  N_SI4 *valid, const char  *layer, 
	const char *physical, const char *gname, N_SI4 *data, char *get)
/*--------------------------------------------------------------------------*/
{ return -1; }
/*--------------------------------------------------------------------------*/
static N_SI4
nusdas_read_local (const char  *type1, const char *type2, const char  *type3,
	N_SI4 *base, const char *kind,  N_SI4 *valid, const char  *layer, 
	const char *physical, void  *data,  const char *fmt, const N_SI4 *size)
/*--------------------------------------------------------------------------*/
{ return -1; }

#endif

/*--------------------------------------------------------------------------*/
int
pandora_nusdas_subc_srf (const char  *type1, const char *type2, 
                         const char  *type3,
	N_SI4 *base, const char *kind,  N_SI4 *valid, const char  *layer, 
	const char *physical, const char *gname, N_SI4 *data, char *get)
/*--------------------------------------------------------------------------*/
/*
returned value : 0 : Normal end
                -1 : Cannot find server matched type1.type2.type3
                -2 : failed to get ispec;
 */
{
    int     i;
    int     k = -1;
    N_SI4   rt = -1;
    struct nus_dims_t d;
    char path[256],url[512];
    
    for(i = 0; i < serv_num; i++){
        if(strcmp(pserv[i].type1,type1)==0 &&
           strcmp(pserv[i].type2,type2)==0 &&
           strcmp(pserv[i].type3,type3)==0){
            k = i;
        }
        else{
            continue;
        }
        set_nus_dims (type1, type2, type3, base, kind,
                      valid, valid, layer, physical, &d);
        strcpy  (path, nusdims_to_path(&d));


        sprintf (url, "%s/%s", pserv[k].host,path+1);
        rt = nusdas_subc_srf_net (url, gname, data, get);
        if (rt == 0){
            return rt;
        };
    }
    if(k==-1){
        return -1;
    }
    else{
        return -2;
    }
}
/*--------------------------------------------------------------------------*/
int
pandora_nusdas_read (const char  *type1, const char *type2, const char  *type3,
	N_SI4 *base, const char *kind,  N_SI4 *valid, const char  *layer, 
	const char *physical, void  *data,  const char *fmt, const N_SI4 *size)
/*--------------------------------------------------------------------------*/
/*
  returned value: >0 : number of items
                  =0: No data;
                  -1: Cannot find server matched type1.type2.type3
 */
{
    int     i;
    int     k = -1;
    N_SI4   rt = -1;
    struct nus_dims_t d;
    char path[256],url[512];
    
    for(i=0;i<serv_num;i++){
        if(strcmp(pserv[i].type1,type1)==0 &&
           strcmp(pserv[i].type2,type2)==0 &&
           strcmp(pserv[i].type3,type3)==0 ){
           k = i;
        }
        else{
            continue;
        }
        set_nus_dims (type1, type2, type3, base, kind,
                      valid, valid, layer, physical, &d);
        strcpy  (path, nusdims_to_path(&d));


        sprintf (url, "%s/%s", pserv[k].host,path+1);
        rt = nusdas_read_net (url, data, fmt, size);
        if (rt > 0){
            return rt;
        };
    }
    if(k==-1){
        return -1;
    }
    else{
        return 0;
    }
        
}

