#define N_DEF_PUTESF_SOURCE
#include "nusdas_sys.h"

/* -------------------------------------------------------------------------- */
N_SI4
n_put_esf(N_UI1 * esf, N_SI4 * sinfo, long o_offset, N_SI4 n_offset,
	  N_ST_CNTL * s_cntl, const char *descriptor, N_SI4 * validtime1,
	  N_SI4 * validtime2, const char *level1, const char *level2,
	  const char *element, void *data, const char *dtype,
	  const N_SI4 * dnum, N_UI4 s_pack, N_UI4 s_miss,
	  N_SI4 s_frlen, N_SI4 * ng)
/* -------------------------------------------------------------------------- */
/*  input:  esf         = memory pointer for write DATA record                */
/*  output: sinfo       = 0: use old_offset, 1: use next_offset               */
/*          o_offset    = current offset in INDX                              */
/*          n_offset    = next offset from SEEK_SET                           */
/*          s_cntl      = structure for CNTL                                  */
/*  input:  descriptor, validtime1, validtime2, level1, level2, element       */
/*                      = target parameters                                   */
/*          data        = user data pointer for write the nusdas file         */
/*          dtype       = variable type for user data pointer                 */
/*          dnum        = item number of user data array (not byte)           */
/*          s_pack      = packing parameter                                   */
/*          s_miss      = missing value parameter                             */
/*          s_frlen     = forced record length                                */
/*  output: ng          = item number of recorded data array (not byte)       */
/*  return:             = record length of this writing record                */
/*              -1      = file read error etc.                                */
/*              -3      = reserved array size is insufficient                 */
/*              -4      = input data type is not supported                    */
/*              -5      = record length exceed s_frlen                        */
/*              -6      = specified UDFV is not supported                     */
/*              -7      = mask bit is not allocated                           */
/*              -8      = input data range is large for specified byte encode */
/*              -9      = specified packing is not supported                  */
/*             -10      = malloc error                                        */
/* -------------------------------------------------------------------------- */
{
    char s[] = "DATA";
    N_ST_DATA s_data;
    float *d_f;
    N_SI4 i, rt, tbyte, dbyte, mbyte, maxv, nbit, rlen, sps = 0;
    long offset;
    N_UI2 n_ui2;
    N_UI1 dum = 0, *p_miss, *p_mask, *ui;

    time(&n_systime);
    memcpy(&s_data.h.recname, s, 4);
    s_data.h.creattime = n_systime;
    memcpy(&s_data.dc, descriptor, 4);
    s_data.vt1 = *validtime1;
    s_data.vt2 = *validtime2;
    memcpy(&s_data.lvel[0], level1, 6);
    memcpy(&s_data.lvel[6], level2, 6);
    memcpy(&s_data.lvel[12], element, 6);
    s_data.lvel[18] = 0;
    s_data.lvel[19] = 0;
    s_data.pack = s_pack;
    s_data.miss = s_miss;
    s_data.xs =
	(n_sizex_override == N_OFF) ? s_cntl->f.xs : n_sizex_override;
    s_data.ys =
	(n_sizey_override == N_OFF) ? s_cntl->f.ys : n_sizey_override;

    *ng = s_data.xs * s_data.ys;
    if (*dnum < *ng) {
	N_ERP2("number of your data array is insufficient.\n");
	errprintf( "data array <%d>, grid size <%d>.\n", *dnum, *ng);
	return (-3);
    } else if (*dnum > *ng) {
	N_ERP1(
	       "number of your data array is not consistent with set size\n");
    }

    if ((rt = n_miss_mask_set(&s_data, &p_miss, &p_mask)) < 0)
	return (rt);

    if (memcmp(&s_data.pack, N_P_1PAC, 4) == 0) {
	if (n_alloc_dp
	    (&s_data.d, sizeof(N_UI1) * (*ng) + sizeof(float) * 2) < 0)
	    return (-10);
	if (strncmp(dtype, N_R4, 2) == 0)
	    dbyte = n_encode_1pac_r4(data, s_data.d, *ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    dbyte = n_encode_1pac_i4(data, s_data.d, *ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    dbyte = n_encode_1pac_i2(data, s_data.d, *ng, p_miss, p_mask);
	else {
	    N_ERP2(
		   "your data type <%s> is not supported for 1pac encoding.\n",
		   dtype);
	    free(s_data.d);
	    return (-4);
	}
    } else if (memcmp(&s_data.pack, N_P_2PAC, 4) == 0) {
	if (n_alloc_dp
	    (&s_data.d, sizeof(N_SI2) * (*ng) + sizeof(float) * 2) < 0)
	    return (-10);
	if (strncmp(dtype, N_R4, 2) == 0)
	    dbyte = n_encode_2pac_r4(data, s_data.d, *ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R8, 2) == 0)
	    dbyte = n_encode_2pac_r8(data, s_data.d, *ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I4, 2) == 0)
	    dbyte = n_encode_2pac_i4(data, s_data.d, *ng, p_miss, p_mask);
	else if (strncmp(dtype, N_NC, 2) == 0)
	    dbyte = n_encode_2pac_nc(data, s_data.d, *ng, p_miss, p_mask);
	else {
	    N_ERP2(
		   "your data type <%s> is not supported for 2pac encoding.\n",
		   dtype);
	    free(s_data.d);
	    return (-4);
	}
    } else if (memcmp(&s_data.pack, N_P_2UPAC, 4) == 0) {
	if (n_alloc_dp
	    (&s_data.d, sizeof(N_UI2) * (*ng) + sizeof(float) * 2) < 0)
	    return (-10);
	if (strncmp(dtype, N_R4, 2) == 0)
	    dbyte = n_encode_2upac_r4(data, s_data.d, *ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R8, 2) == 0)
	    dbyte = n_encode_2upac_r8(data, s_data.d, *ng, p_miss, p_mask);
	else {
	    N_ERP2(
		   "your data type <%s> is not supported for 2upac encoding.\n",
		   dtype);
	    free(s_data.d);
	    return (-4);
	}
    } else if (memcmp(&s_data.pack, N_P_4PAC, 4) == 0) {
	if (n_alloc_dp
	    (&s_data.d, sizeof(N_SI4) * (*ng) + sizeof(double) * 2) < 0)
	    return (-10);
	if (strncmp(dtype, N_R8, 2) == 0)
	    dbyte = n_encode_4pac_r8(data, s_data.d, *ng, p_miss, p_mask);
	else {
	    N_ERP2(
		   "your data type <%s> is not supported for 4pac encoding.\n",
		   dtype);
	    free(s_data.d);
	    return (-4);
	}
    } else if (memcmp(&s_data.pack, N_P_R4, 4) == 0) {
	if (n_alloc_dp(&s_data.d, sizeof(float) * (*ng)) < 0)
	    return (-10);
	if (strncmp(dtype, N_R4, 2) == 0)
	    dbyte = n_encode_r4_r4(data, s_data.d, *ng, p_miss, p_mask);
	else if (strncmp(dtype, N_R8, 2) == 0)
	    dbyte = n_encode_r4_r8(data, s_data.d, *ng, p_miss, p_mask);
	else {
	    N_ERP2(
		   "your data type <%s> is not supported for 4byte real encoding.\n",
		   dtype);
	    free(s_data.d);
	    return (-4);
	}
    } else if (memcmp(&s_data.pack, N_P_R8, 4) == 0) {
	if (n_alloc_dp(&s_data.d, sizeof(double) * (*ng)) < 0)
	    return (-10);
	if (strncmp(dtype, N_R8, 2) == 0)
	    dbyte = n_encode_r8_r8(data, s_data.d, *ng, p_miss, p_mask);
	else {
	    N_ERP2(
		   "your data type <%s> is not supported for 8byte real encoding.\n",
		   dtype);
	    free(s_data.d);
	    return (-4);
	}
    } else if (memcmp(&s_data.pack, N_P_I1, 4) == 0) {
	if (n_alloc_dp(&s_data.d, sizeof(N_UI1) * (*ng)) < 0)
	    return (-10);
	if (strncmp(dtype, N_I4, 2) == 0)
	    dbyte = n_encode_i1_i4(data, s_data.d, *ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    dbyte = n_encode_i1_i2(data, s_data.d, *ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I1, 2) == 0)
	    dbyte = n_encode_i1_i1(data, s_data.d, *ng, p_miss, p_mask);
	else {
	    N_ERP2(
		   "your data type <%s> is not supported for 1byte integer encoding.\n",
		   dtype);
	    free(s_data.d);
	    return (-4);
	}
	if (dbyte < 0) {
	    N_ERP2(
		   "your data range is large for 1byte integer encoding.\n");
	    free(s_data.d);
	    return (-8);
	}
    } else if (memcmp(&s_data.pack, N_P_I2, 4) == 0) {
	if (n_alloc_dp(&s_data.d, sizeof(N_SI2) * (*ng)) < 0)
	    return (-10);
	if (strncmp(dtype, N_I4, 2) == 0)
	    dbyte = n_encode_i2_i4(data, s_data.d, *ng, p_miss, p_mask);
	else if (strncmp(dtype, N_I2, 2) == 0)
	    dbyte = n_encode_i2_i2(data, s_data.d, *ng, p_miss, p_mask);
	else {
	    N_ERP2(
		   "your data type <%s> is not supported for 2byte integer encoding.\n",
		   dtype);
	    free(s_data.d);
	    return (-4);
	}
	if (dbyte < 0) {
	    N_ERP2(
		   "your data range is large for 2byte integer encoding.\n");
	    free(s_data.d);
	    return (-8);
	}
    } else if (memcmp(&s_data.pack, N_P_N1I2, 4) == 0) {
	if (n_alloc_dp(&s_data.d, sizeof(N_SI2) * (*ng)) < 0)
	    return (-10);
	if (strncmp(dtype, N_R4, 2) == 0) {
	    dbyte =
		n_encode_i2_r4(data, s_data.d, *ng, p_miss, p_mask, 10);
	} else if (strncmp(dtype, N_I4, 2) == 0) {
	    dbyte = n_encode_i2_i4(data, s_data.d, *ng, p_miss, p_mask);
	} else if (strncmp(dtype, N_I2, 2) == 0) {
	    dbyte = n_encode_i2_i2(data, s_data.d, *ng, p_miss, p_mask);
	} else {
	    N_ERP2(
		   "your data type <%s> is not supported for 2byte integer(x10) encoding.\n",
		   dtype);
	    free(s_data.d);
	    return (-4);
	}
	if (dbyte < 0) {
	    N_ERP2(
		   "your data range is large for 2byte integer(x10) encoding.\n");
	    free(s_data.d);
	    return (-8);
	}
    } else if (memcmp(&s_data.pack, N_P_I4, 4) == 0) {
	if (n_alloc_dp(&s_data.d, sizeof(N_SI4) * (*ng)) < 0)
	    return (-10);
	if (strncmp(dtype, N_I4, 2) == 0)
	    dbyte = n_encode_i4_i4(data, s_data.d, *ng, p_miss, p_mask);
	else {
	    N_ERP2(
		   "your data type <%s> is not supported for 4byte integer encoding.\n",
		   dtype);
	    free(s_data.d);
	    return (-4);
	}
    } else if (memcmp(&s_data.pack, N_P_RLEN, 4) == 0) {
	if (n_alloc_dp(&ui, sizeof(N_UI1) * (*ng)) < 0)
	    return (-10);
	maxv = -1;
	nbit = 8;
	if ((dbyte = n_encode_rlen_8bit(data, ui, *ng, *ng, &maxv)) < 0) {
	    N_ERP2(
		   "your data is out of range for 8bit runlength compress\n");
	    free(ui);
	    return (-4);
	}
	if (n_alloc_dp(&s_data.d, sizeof(N_UI1) * dbyte + sizeof(N_SI4)
		       * 3) < 0)
	    return (-10);
	memcpy((N_UI1 *) (s_data.d), &nbit, 4);
	memcpy((N_UI1 *) (s_data.d + sizeof(N_SI4)), &maxv, 4);
	memcpy((N_UI1 *) (s_data.d + sizeof(N_SI4) * 2), &dbyte, 4);
	memcpy((N_UI1 *) (s_data.d + sizeof(N_SI4) * 3), ui, dbyte);
	dbyte += sizeof(N_SI4) * 3;
	free(ui);
    } else {
	N_ERP2("specified packing <%4s> is not supported.\n",
	       (char *) (&s_data.pack));
	return (-9);
    }

    mbyte = n_mbyte_set(&s_data, p_mask, p_miss, ng);
    tbyte = mbyte + dbyte + N_SIZE_STDATA + sizeof(N_SI4);
    if (s_frlen != 0 && tbyte > s_frlen) {
	N_ERP2(
	       "DATA record length <%d> exceeded forced record length <%d>.\n",
	       tbyte, s_frlen);
	free(s_data.d);
	return (-5);
    }
    s_data.h.length_s = s_frlen;
    s_data.h.available = tbyte - sizeof(N_SI4) * 2;
    s_data.length_e = s_data.h.length_s;

    memcpy((N_UI1 *) (esf + sps), &s_data, N_SIZE_STDATA);
    sps += N_SIZE_STDATA;
    if (p_mask != NULL) {
	memcpy((N_UI1 *) (esf + sps), p_mask, mbyte);
	sps += mbyte;
    } else if (p_miss != NULL) {
	memcpy((N_UI1 *) (esf + sps), p_miss, mbyte);
	sps += mbyte;
    }
    memcpy((N_UI1 *) (esf + sps), s_data.d, dbyte);
    sps += dbyte;

    free(s_data.d);

    for (i = s_data.h.available + sizeof(N_SI4); i < s_data.length_e
	 - sizeof(N_SI4); i++)
	memcpy((N_UI1 *) (esf + sps++), &dum, sizeof(N_UI1));
    memcpy((N_UI1 *) (esf + sps), &s_data.length_e, sizeof(N_SI4));
    sps += 4;

    return (s_data.length_e);
}
