#define N_DEF_RECORD_SOURCE
#include "nusdas_sys.h"
#include <pwd.h>

/* -------------------------------------------------------------------------- */
void n_timeprint(char *timestring, N_SI4 nwp_tmin)
/* -------------------------------------------------------------------------- */
/* print time on the string                                                   */
/*  both:  timestring = pointer for the string                                */
/*  input: nwp_tmin   = total minutes in nwp routine system                   */
/*                                 ( 0 :use system clock time )               */
/* -------------------------------------------------------------------------- */
{
    if (nwp_tmin == 0) {
	struct tm *tp;
	tp = gmtime(&n_systime);
	sprintf(timestring, "%4.4d%2.2d%2.2d%2.2d%2.2d", tp->tm_year
		+ 1900, tp->tm_mon + 1, tp->tm_mday, tp->tm_hour,
		tp->tm_min);
    } else {
	N_SI4 y, m, d, h, f;
	nwp_seq2ymdhm(&y, &m, &d, &h, &f, nwp_tmin);
	sprintf(timestring, "%4.4d%2.2d%2.2d%2.2d%2.2d", y, m, d, h, f);
    }
}


/* -------------------------------------------------------------------------- */
N_SI4 n_calc_addday(N_SI4 py, N_SI4 pp, N_SI4 vt, N_SI4 id)
/* -------------------------------------------------------------------------- */
/* calculate total day correspond to validtime in each time unit              */
/*  input:    py  = year                                                      */
/*            pp  = number of total time unit in the year                     */
/*            vt  = validtime in time unit                                    */
/*            id  = id for time unit                                          */
/*  return:       = total day                                                 */
/* -------------------------------------------------------------------------- */
{
    N_SI4 l, ky, kp, kd;

    if (id == N_TU_PENT) {
	for (l = 0, kd = 0, kp = pp, ky = py; l < vt; l++) {
	    kd += (kp == 12 && ky % 4 == 0 && !(ky % 100 ==
						0
						&& ky % 400 != 0)) ? 6 : 5;
	    if (++kp > 73) {
		kp -= 73;
		ky++;
	    }
	}
    } else if (id == N_TU_MONT) {
	for (l = 0, kd = 0, kp = pp, ky = py; l < vt; l++) {
	    kd += n_mday[kp - 1];
	    if (kp == 2 && ky % 4 == 0
		&& !(ky % 100 == 0 && ky % 400 != 0)) kd++;
	    if (++kp > 12) {
		kp -= 12;
		ky++;
	    }
	}
    } else if (id == N_TU_JUN) {
	for (l = 0, kd = 0, kp = pp, ky = py; l < vt; l++) {
	    kd += (kp % 3 != 0) ? 10 : n_mday[(kp - 1) / 3] - 20;
	    if (kp == 6 && ky % 4 == 0
		&& !(ky % 100 == 0 && ky % 400 != 0)) kd++;
	    if (++kp > 36) {
		kp -= 36;
		ky++;
	    }
	}
    }

    return (kd);
}


/* -------------------------------------------------------------------------- */
N_SI4 n_calc_pent(N_SI4 pm, N_SI4 pd)
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, tday;

    for (i = 1, tday = pd; i < pm; i++)
	tday += n_mday[i - 1];

    return (tday / 5 + 1);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_init_nusd(FILE * fp, N_ST_NUSD * s_nusd, N_ST_DEF * s_def, N_SI4 tbyte,
	    N_SI4 trec, N_SI4 irec, N_SI4 srec)
/* -------------------------------------------------------------------------- */
/* initialize NUSD record                                                     */
/*  input:  fp      = file pointer                                            */
/*          s_nusd  = NUSD struct pointer                                     */
/*          s_def   = DEF struct pointer                                      */
/*          tbyte   = total byte in this file                                 */
/*          trec    = total number of record in this file                     */
/*          irec    = total number of information record in this file         */
/*          srec    = total number of sub-cntl record in this file            */
/*  return:         = this NUSD record size (byte)                            */
/* -------------------------------------------------------------------------- */
{
    N_UI1 uc = 0;
    N_SI4 i, j;
    char hostname[256], username[256], s[256];

    memcpy(&(s_nusd->h.recname), "NUSD", 4);
    s_nusd->h.creattime = n_systime;
    gethostname(hostname, 256);
    strcpy(username, getpwuid(getuid())->pw_name);
    sprintf(s, "<%s@%s>", hostname, username);
    i = strlen(s);
    if (i > 79)
	i = 79;
    j = strlen(s_def->creator);
    if (j > 79)
	j = 79;
    memcpy(s_nusd->creator, s_def->creator, j);
    s_nusd->creator[j] = '\0';
    j = strlen(s_nusd->creator);
    if (i + j <= 79)
	strcat(s_nusd->creator, s);
    else {
	strncpy(&s_nusd->creator[79 - j], s, j);
	s_nusd->creator[79] = '\0';
    }
    s_nusd->version = s_def->version;
    s_nusd->filesize = tbyte;
    s_nusd->irec_num = irec;
    s_nusd->srec_num = srec;
    s_nusd->rec_num = trec;
    s_nusd->h.length_s =
	(s_def->forced_rlen ==
	 0) ? (N_SI4) sizeof(N_ST_NUSD) : s_def->forced_rlen;
    s_nusd->h.available = (N_SI4) (sizeof(N_ST_NUSD) - 2 * sizeof(N_SI4));
    s_nusd->length_e = s_nusd->h.length_s;

    if (fp != NULL) {
	j = sizeof(N_ST_NUSD) - sizeof(N_SI4);
	if (fwrite_fmt(s_nusd, N_ST_NUSD_FORMAT_WITHOUTLEN, fp)
	    != j) {
	    return -1;
	}
	for (i = j; i < s_nusd->h.length_s - sizeof(N_SI4); i++)
	    fwrite(&uc, 1, 1, fp);
	fwrite_native(&s_nusd->length_e, sizeof(N_SI4), 1, fp);
    }

    return (s_nusd->h.length_s);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_init_or_check_validtime(N_ST_DEF * s_def, N_ST_CNTL * s_cntl,
			  N_SI4 * basetime, N_SI4 * validtime1, N_SI4 flag)
/* -------------------------------------------------------------------------- */
/* change validtime from relative to absolute                                 */
/*  input:  flag  = 1: set s_cntl->vt1,2 2: not set                           */
/*  return: j     = sequential number of registered validtime                 */
/*          -1    = not registered                                            */
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, k, l;
    N_SI4 py, pm, pd, ph, pn, pp, ky, kp, kd, vvt;

    for (i = 0; i < n_tunit_num; i++)
	if (memcmp(&s_def->time_unit, n_tunit_string[i], 4) == 0)
	    break;
    if (i == N_TU_HOUR || i == N_TU_MIN || i == N_TU_DAY || i == N_TU_WEEK
	|| i == n_tunit_num) {
	if (*validtime1 < 0) {	/* vt in file */
	    for (j = 0; j < s_def->n_vt; j++) {
		vvt = *basetime + *(s_def->vt1 + j) * n_tunit_min[i];
		if (flag == N_ON) {
		    *(s_cntl->vt1 + j) = vvt;
		    *(s_cntl->vt2 + j) = *(s_def->vt2 + j);
		} else if (vvt == *validtime1)
		    break;
	    }
	    if (flag == N_OFF && j == s_def->n_vt)
		return (-1);
	} else {		/* vt out file */
	    for (j = 0; j < s_def->n_vt; j++) {
		vvt = *basetime + *(s_def->vt1 + j) * n_tunit_min[i];
		if (vvt == *validtime1)
		    break;
	    }
	    if (j == s_def->n_vt)
		return (-1);
	    else if (flag) {
		*(s_cntl->vt1) = vvt;
		*(s_cntl->vt2) = *(s_def->vt2 + j);
	    }
	}
    } else if (i == N_TU_PENT || i == N_TU_MONT || i == N_TU_JUN) {
	nwp_seq2ymdhm(&py, &pm, &pd, &ph, &pn, *basetime);
	pp = (i == N_TU_PENT) ? n_calc_pent(pm, pd)
	    : (i == N_TU_MONT) ? pm : (pm - 1) * 3 + (pd - 1) / 10 + 1;
	if (*validtime1 < 0) {
	    for (j = 0; j < s_def->n_vt; j++) {
		kd = n_calc_addday(py, pp, *(s_def->vt1 + j), i);
		vvt = *basetime + kd * 60 * 24;
		if (flag == N_ON) {
		    *(s_cntl->vt1 + j) = vvt;
		    *(s_cntl->vt2 + j) = *(s_def->vt2 + j);
		} else if (vvt == *validtime1)
		    break;
	    }
	    if (flag == N_OFF && j == s_def->n_vt)
		return (-1);
	} else {
	    for (j = 0; j < s_def->n_vt; j++) {
		kd = n_calc_addday(py, pp, *(s_def->vt1 + j), i);
		vvt = *basetime + kd * 60 * 24;
		if (vvt == *validtime1)
		    break;
	    }
	    if (j == s_def->n_vt)
		return (-1);
	    else if (flag) {
		*(s_cntl->vt1) = vvt;
		*(s_cntl->vt2) = *(s_def->vt2 + j);
	    }
	}
    }

    return (j);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_init_cntl(FILE * fp, N_ST_CNTL * s_cntl, N_ST_DEF * s_def, char *type,
	    N_SI4 * basetime, N_UI4 * descriptor, N_SI4 * validtime1)
/* -------------------------------------------------------------------------- */
/* initialize CNTL record                                                     */
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, len = sizeof(N_ST_CF), kp, kd, ky, l;
    N_UI1 uc = 0;
    char *c;

    strcpy(n_routine_name[n_routine_d++], "init_cntl");

    memcpy(&(s_cntl->f.h.recname), "CNTL", 4);
    s_cntl->f.h.creattime = n_systime;
    memcpy(s_cntl->f.type, type, 16);
    n_timeprint((char *) s_cntl->f.basetime, *basetime);
    s_cntl->f.basetime_t = *basetime;
    s_cntl->f.time_unit = s_def->time_unit;
    s_cntl->f.n_dc = (s_def->infile_dc == N_IN_FILE) ? s_def->n_dc : 1;
    s_cntl->f.n_vt = (s_def->infile_vt == N_IN_FILE) ? s_def->n_vt : 1;
    s_cntl->f.n_lv = s_def->n_lv;
    s_cntl->f.n_el = s_def->n_el;
    s_cntl->f.projection = s_def->projection;
    s_cntl->f.xs = s_def->xs;
    s_cntl->f.ys = s_def->ys;
    s_cntl->f.x0 = s_def->x0;
    s_cntl->f.y0 = s_def->y0;
    s_cntl->f.xd = s_def->xd;
    s_cntl->f.yd = s_def->yd;
    s_cntl->f.lat_0 = s_def->lat_0;
    s_cntl->f.lon_0 = s_def->lon_0;
    s_cntl->f.lat_1 = s_def->lat_1;
    s_cntl->f.lon_1 = s_def->lon_1;
    s_cntl->f.lat_2 = s_def->lat_2;
    s_cntl->f.lon_2 = s_def->lon_2;
    s_cntl->f.lat_s1 = s_def->lat_s1;
    s_cntl->f.lon_s1 = s_def->lon_s1;
    s_cntl->f.lat_s2 = s_def->lat_s2;
    s_cntl->f.lon_s2 = s_def->lon_s2;
    s_cntl->f.gpv_mean = s_def->gpv_mean;
    for (i = 0; i < 2; i++)
	s_cntl->f.reserved_1[i] = 0;
    for (i = 0; i < 6; i++)
	s_cntl->f.reserved_2[i] = 0;

    if ((s_cntl->dc = (N_UI4 *) malloc(sizeof(N_UI4) * s_cntl->f.n_dc))
	== NULL) {
	N_ERP2("malloc failed <member:%d>\n", s_cntl->f.n_dc);
	N_ERTN(-2);
    } else {
	if (s_def->infile_dc == N_IN_FILE) {
	    for (i = 0; i < s_cntl->f.n_dc; i++)
		*(s_cntl->dc + i) = *(s_def->dc + i);
	} else {
	    for (i = 0; i < s_def->n_dc; i++)
		if (memcmp(descriptor, (char *) (s_def->dc + i), 4) == 0)
		    break;
	    if (i == s_def->n_dc) {
		N_ERP2("member <%4.4s> is not registered\n",
		       descriptor);
		N_ERTN(-3);
	    } else
		*(s_cntl->dc) = *(s_def->dc + i);
	}
	len += sizeof(N_UI4) * s_cntl->f.n_dc;
    }
    if ((s_cntl->vt1 = (N_SI4 *) malloc(sizeof(N_SI4) * s_cntl->f.n_vt))
	== NULL ||
	(s_cntl->vt2 = (N_SI4 *) malloc(sizeof(N_SI4) * s_cntl->f.n_vt))
	== NULL) {
	N_ERP2("malloc failed <vtime:%d>\n", s_cntl->f.n_vt);
	N_ERTN(-2);
    } else {
	i = (s_def->infile_vt == N_IN_FILE) ? -1 : *validtime1;
	if (n_init_or_check_validtime(s_def, s_cntl, basetime, &i, 1) < 0) {
	    N_ERP2("validtime1 <%d> is not registered\n",
		   *validtime1);
	    N_ERTN(-3);
	}
	len += 2 * sizeof(N_SI4) * s_cntl->f.n_vt;
    }
    if ((s_cntl->lv1 = (N_UI8 *) malloc(sizeof(N_UI8) * s_cntl->f.n_lv))
	== NULL ||
	(s_cntl->lv2 = (N_UI8 *) malloc(sizeof(N_UI8) * s_cntl->f.n_lv))
	== NULL) {
	N_ERP2("malloc failed <plane:%d>\n", s_cntl->f.n_vt);
	N_ERTN(-2);
    } else {
	for (i = 0; i < s_cntl->f.n_lv; i++)
	    memcpy((N_UI8 *) (s_cntl->lv1 + i), (N_UI8 *) (s_def->lv1 + i),
		   8);
	for (i = 0; i < s_cntl->f.n_lv; i++)
	    memcpy((N_UI8 *) (s_cntl->lv2 + i), (N_UI8 *) (s_def->lv2 + i),
		   8);
	len += 2 * 6 * s_cntl->f.n_lv;
    }
    if ((s_cntl->el = (N_UI8 *) malloc(sizeof(N_UI8) * s_cntl->f.n_el))
	== NULL) {
	N_ERP2("malloc failed <element:%d>\n", s_cntl->f.n_el);
	N_ERTN(-2);
    } else {
	for (i = 0; i < s_cntl->f.n_el; i++)
	    memcpy((N_UI8 *) (s_cntl->el + i), (N_UI8 *) (s_def->el + i),
		   8);
	len += 6 * s_cntl->f.n_el;
    }
    len += sizeof(N_SI4);

    if (s_def->forced_rlen == 0)
	s_cntl->f.h.length_s = len;
    else if (s_def->forced_rlen >= len)
	s_cntl->f.h.length_s = s_def->forced_rlen;
    else {
	N_ERP2(
	       "Forced record length (%d) is less than necessary length (%d) <CNTL RECORD>",
	       s_def->forced_rlen, len);
	N_ERTN(-4);
    }
    s_cntl->f.h.available = len - 2 * sizeof(N_SI4);
    s_cntl->length_e = s_cntl->f.h.length_s;

    if (fp != NULL) {
	if (fwrite_fmt(&s_cntl->f, N_ST_CF_ALLFORMAT, fp) != sizeof(N_ST_CF)) {
	    N_ERTN(-1);
	}
	fwrite((N_UI4 *) (s_cntl->dc), sizeof(N_UI4), s_cntl->f.n_dc, fp);
	fwrite_native((N_SI4 *) (s_cntl->vt1), sizeof(N_SI4), s_cntl->f.n_vt, fp);
	fwrite_native((N_SI4 *) (s_cntl->vt2), sizeof(N_SI4), s_cntl->f.n_vt, fp);
	for (i = 0; i < s_cntl->f.n_lv; i++)
	    fwrite((N_UI8 *) (s_cntl->lv1 + i), 6, 1, fp);
	for (i = 0; i < s_cntl->f.n_lv; i++)
	    fwrite((N_UI8 *) (s_cntl->lv2 + i), 6, 1, fp);
	for (i = 0; i < s_cntl->f.n_el; i++)
	    fwrite((N_UI8 *) (s_cntl->el + i), 6, 1, fp);
	j = s_cntl->f.h.available + sizeof(N_SI4);
	for (i = j; i < s_cntl->f.h.length_s - sizeof(N_SI4); i++)
	    fwrite(&uc, 1, 1, fp);
	fwrite_native(&s_cntl->length_e, sizeof(N_SI4), 1, fp);
    }
    n_routine_d--;
    return (s_cntl->f.h.length_s);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_init_indx(FILE * fp, N_ST_INDX * s_indx, N_ST_CNTL * s_cntl,
	    N_ST_DEF * s_def, N_SI4 * basetime, N_SI4 * validtime1)
/* -------------------------------------------------------------------------- */
/* initialize INDX record                                                     */
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, k, ks, ke, l, ls, le, m, n, num, len;
    N_UI1 uc = 0;
    N_ST_CNTL dum_cntl;

    strcpy(n_routine_name[n_routine_d++], "init_indx");

    memcpy(&(s_indx->h.recname), "INDX", 4);
    s_indx->h.creattime = n_systime;
    num =
	s_cntl->f.n_dc * s_cntl->f.n_vt * s_cntl->f.n_lv * s_cntl->f.n_el;
    if ((s_indx->offset = (N_SI4 *) malloc(sizeof(N_SI4) * num)) == NULL) {
	N_ERP2("malloc failed <index:%d>\n", num);
	N_ERTN(-2);
    } else {
	if (s_def->infile_dc == N_IN_FILE) {
	    ls = 0;
	    le = s_cntl->f.n_dc;
	} else {
	    for (ls = 0; ls < s_def->n_dc; ls++)
		if (memcmp((char *) s_cntl->dc, (char *) (s_def->dc
							  + ls), 4) == 0)
		    break;
	    le = ls + 1;
	}
	if (s_def->infile_vt == N_IN_FILE) {
	    ks = 0;
	    ke = s_cntl->f.n_vt;
	} else {
	    ks = n_init_or_check_validtime(s_def, &dum_cntl,
					   basetime, validtime1, 0);
	    ke = ks + 1;
	}
	for (l = ls, m = 0; l < le; l++)
	    for (k = ks; k < ke; k++)
		for (j = 0; j < s_cntl->f.n_lv; j++)
		    for (i = 0; i < s_cntl->f.n_el; i++, m++) {
			n = i + s_def->n_el * (j
					       + s_def->n_lv * (k +
								s_def->n_vt
								* l));
			*(s_indx->offset + m) =
			    (*(s_def->elmap + n) == 1) ? 0 : -1;
		    }
    }
    len = num * sizeof(N_SI4) + sizeof(N_ST_RI) + sizeof(N_SI4);
    if (s_def->forced_rlen == 0)
	s_indx->h.length_s = len;
    else if (s_def->forced_rlen >= len)
	s_indx->h.length_s = s_def->forced_rlen;
    else {
	N_ERP2(
	       "Forced record length (%d) is less than necessary length (%d) <INDX RECORD>",
	       s_def->forced_rlen, len);
	N_ERTN(-4);
    }
    s_indx->h.available = len - 2 * sizeof(N_SI4);
    s_indx->length_e = s_indx->h.length_s;

    if (fp != NULL) {
	if (fwrite_fmt(&s_indx->h, N_ST_RI_FORMAT, fp) != sizeof(N_ST_RI)) {
	    N_ERTN(-1);
	}
	fwrite_native((N_SI4 *) (s_indx->offset), sizeof(N_SI4), num, fp);
	j = s_indx->h.available + sizeof(N_SI4);
	for (i = j; i < s_indx->h.length_s - sizeof(N_SI4); i++)
	    fwrite(&uc, 1, 1, fp);
	fwrite_native(&s_indx->length_e, sizeof(N_SI4), 1, fp);
    }
    n_routine_d--;
    return (s_indx->h.length_s);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_init_info(FILE * fp, N_ST_INFO * s_info, N_ST_DEF * s_def,
	    N_UI4 * igname, char *info_file)
/* -------------------------------------------------------------------------- */
/* initialize INFO record (read information from info_file and write it)      */
/* -------------------------------------------------------------------------- */
{
    FILE *fpi;
    long offset1, offset2;
    N_SI4 c, l;
    char cc;

    strcpy(n_routine_name[n_routine_d++], "init_indx");

    if ((fpi = fopen(info_file, "r")) == NULL) {
	N_ERP2("Information file <%s> is not found\n", info_file);
	N_ERTN(-5);
    }
    memcpy(&(s_info->h.recname), "INFO", 4);
    s_info->h.creattime = n_systime;
    s_info->h.length_s = sizeof(N_ST_RI) + sizeof(N_SI4) + sizeof(N_UI4);
    s_info->h.available = s_info->h.length_s - 2 * sizeof(N_SI4);
    memcpy(&(s_info->gname), igname, 4);

    if (fp != NULL) {
	offset1 = ftell(fp);
	if (fwrite_fmt(s_info, N_ST_RI_FORMAT, fp) != sizeof(N_ST_RI)) {
	    N_ERTN(-1);
	}
	fwrite_native(&(s_info->gname), sizeof(N_UI4), 1, fp);
    }
    l = 0;
    while ((c = fgetc(fpi)) != EOF) {
	if (fp != NULL)
	    fputc(c, fp);
	l++;
	if (s_def->forced_rlen != 0 && l >= s_def->forced_rlen -
	    sizeof(N_ST_RI) - sizeof(N_SI4) * 2) {
	    N_ERP1(
		   "Information record <file:%s> length is over s_def record length!\nCharacters over limit are deleted\n",
		   info_file);
	    break;
	}
    }
    s_info->h.available += l;
    s_info->h.length_s += l;
    if (s_def->forced_rlen != 0)
	s_info->h.length_s = s_def->forced_rlen;
    s_info->length_e = s_info->h.length_s;
    s_info->size = l;
    if (fp == NULL) {
	s_info->d = (N_UI1 *) malloc(sizeof(N_UI1) * s_info->size);
	offset1 = 0;
	fseek(fpi, offset1, SEEK_SET);
	l = 0;
	while ((c = fgetc(fpi)) != EOF) {
	    cc = (char) c;
	    memcpy((N_UI1 *) (s_info->d + l++), &cc, 1);
	    if (l >= s_info->size)
		break;
	}
    }
    fclose(fpi);

    if (fp != NULL) {
	fwrite_native(&s_info->h.length_s, sizeof(N_SI4), 1, fp);
	offset2 = ftell(fp);
	fseek(fp, offset1, SEEK_SET);
	fwrite_fmt(s_info, N_ST_RI_FORMAT, fp);
	fseek(fp, offset2, SEEK_SET);
    }

    n_routine_d--;
    return (s_info->h.length_s);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_init_subc_v(FILE * fp, N_SI4 defn, N_SI4 ns, N_ST_CNTL * s_cntl,
	      N_ST_DEF * s_def, N_SI4 * basetime, N_SI4 * validtime1)
/* -------------------------------------------------------------------------- */
/* initialize SUBC record                                                     */
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, k, ks, ke, l, ls, le, m, n, num, len;
    N_UI1 uc = 0;
    N_ST_SUBC s_subc;
    N_ST_CNTL dum_cntl;

    strcpy(n_routine_name[n_routine_d++], "init_subc");

    memcpy(&(s_subc.h.recname), "SUBC", 4);
    s_subc.h.creattime = n_systime;
    s_subc.gname = s_def->subc[ns];
    num = s_cntl->f.n_dc * s_cntl->f.n_vt * s_def->subc_size[ns];
    if ((s_subc.d = (N_UI1 *) malloc(sizeof(N_UI1) * num)) == NULL) {
	N_ERP2("malloc failed <subc:%d>\n", num);
	N_ERTN(-2);
    }
    s_subc.size = num;
    for (i = 0; i < n_n_subc[defn]; i++)
	if (s_subc.gname == n_w_subc[defn][i].gname)
	    break;
    if (i == n_n_subc[defn] || s_subc.size != n_w_subc[defn][i].size) {
	for (l = 0; l < s_cntl->f.n_dc; l++)
	    for (k = 0; k < s_cntl->f.n_vt; k++)
		for (j = 0; j < s_def->subc_size[ns]; j++)
		    *(s_subc.d + j + s_def->subc_size[ns]
		      * (k + l * s_cntl->f.n_vt)) = 0;
    } else {
	memcpy(s_subc.d, n_w_subc[defn][i].d, s_subc.size);
    }
    len =
	sizeof(N_UI4) + num * sizeof(N_UI1) + sizeof(N_ST_RI) +
	sizeof(N_SI4);
    if (s_def->forced_rlen == 0)
	s_subc.h.length_s = len;
    else if (s_def->forced_rlen >= len)
	s_subc.h.length_s = s_def->forced_rlen;
    else {
	N_ERP2(
	       "Forced record length (%d) is less than necessary length (%d) <SUBC RECORD>",
	       s_def->forced_rlen, len);
	free(s_subc.d);
	N_ERTN(-4);
    }
    s_subc.h.available = len - 2 * sizeof(N_SI4);
    s_subc.length_e = s_subc.h.length_s;

    if (fp != NULL) {
	if (fwrite_fmt(&s_subc.h, N_ST_RI_FORMAT, fp) != sizeof(N_ST_RI)) {
	    N_ERTN(-1);
	}
        /* 2003.2.4 tabito */
	/*fwrite_native(&s_subc.gname, sizeof(N_UI4), 1, fp);*/ 
        fwrite_native(&s_subc.gname, 1, sizeof(N_UI4), fp);
	fwrite((N_UI1 *) (s_subc.d), sizeof(N_UI1), s_subc.size, fp);
	j = s_subc.h.available + sizeof(N_SI4);
	for (i = j; i < s_subc.h.length_s - sizeof(N_SI4); i++)
	    fwrite(&uc, 1, 1, fp);
	fwrite_native(&s_subc.length_e, sizeof(N_SI4), 1, fp);
	free(s_subc.d);
    }

    n_routine_d--;
    return (s_subc.h.length_s);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_init_subc_c(FILE * fp, N_SI4 defn, N_ST_CNTL * s_cntl,
	      N_ST_DEF * s_def, N_ST_SUBC * s_subc, N_SI4 k)
/* -------------------------------------------------------------------------- */
{
    N_UI1 uc = 0;
    N_SI4 i, j;

    strcpy(n_routine_name[n_routine_d++], "init_subc");

    memcpy(&(s_subc->h.recname), "SUBC", 4);
    s_subc->h.creattime = n_systime;
    s_subc->h.length_s =
	(N_SI4) sizeof(N_ST_RI) + 4 + s_def->subc_size[k] + 4;
    s_subc->h.available = s_subc->h.length_s - 2 * sizeof(N_SI4);
    s_subc->gname = s_def->subc[k];
    s_subc->size = s_def->subc_size[k];
    if (s_def->forced_rlen != 0) {
	if (s_subc->h.length_s > s_def->forced_rlen) {
	    N_ERP2(
		   "SUBC record length <%d> exceed forced record length <%d>!\n",
		   s_subc->h.length_s, s_def->forced_rlen);
	    N_ERTN(-4);
	}
	s_subc->h.length_s = s_def->forced_rlen;
    }
    s_subc->length_e = s_subc->h.length_s;

    if (fp != NULL) {		/* disk file case */
	j = sizeof(N_ST_RI) + 4;
	if (fwrite_fmt(s_subc, N_ST_RI_FORMAT "c4", fp) != j) {
	    N_ERTN(-1);
	}
	for (i = 0; i < n_n_subc[defn]; i++)
	    if (s_subc->gname == n_w_subc[defn][i].gname)
		break;
	if (i == n_n_subc[defn] || s_subc->size != n_w_subc[defn][i].size) {
	    if (s_subc->size != n_w_subc[defn][i].size
		&& n_n_subc[defn] != 0) {
		N_ERP2(
		       "your set SUBC size for <%4s> is not consistent with def-file.\n",
		       (char *) (&s_subc->gname));
		N_ERP2("SUBC data is set to NULL.\n");
	    }
	    for (i = j; i < s_subc->h.length_s - sizeof(N_SI4); i++)
		fwrite(&uc, 1, 1, fp);
	} else {
	    fwrite(n_w_subc[defn][i].d, s_subc->size, 1, fp);
	    j += s_subc->size;
	    for (i = j; i < s_subc->h.length_s - sizeof(N_SI4); i++)
		fwrite(&uc, 1, 1, fp);
	}
	fwrite_native(&s_subc->length_e, sizeof(N_SI4), 1, fp);
    } else {			/* ESF case */
	for (i = 0; i < n_n_subc[defn]; i++)
	    if (s_subc->gname == n_w_subc[defn][i].gname)
		break;
	if (i == n_n_subc[defn] || s_subc->size != n_w_subc[defn][i].size) {
	    if (s_subc->size != n_w_subc[defn][i].size) {
		N_ERP2(
		       "your set SUBC size for <%4s> is not consistent with def-file.\n",
		       (char *) (&s_subc->gname));
		N_ERP2("SUBC data is set to NULL.\n");
	    }
	    s_subc->d = (N_UI1 *) malloc(sizeof(N_UI1));
	} else {
	    s_subc->d = (N_UI1 *) malloc(sizeof(N_UI1) * s_subc->size);
	    memcpy(s_subc->d, n_w_subc[defn][i].d, s_subc->size);
	}
    }
    n_routine_d--;
    return (s_subc->h.length_s);
}


/* -------------------------------------------------------------------------- */
void n_init_end(FILE * fp, N_ST_END * s_end, N_SI4 filesize, N_SI4 rec_num)
/* -------------------------------------------------------------------------- *//* initialize END record                                                      *//* -------------------------------------------------------------------------- */
{
    N_SI4 i, j;
    N_UI1 uc = 0;

    memcpy(&(s_end->h.recname), "END ", 4);
    s_end->h.available = sizeof(N_ST_END) - 2 * sizeof(N_SI4);
    s_end->h.creattime = n_systime;
    s_end->filesize = filesize;
    s_end->rec_num = rec_num;

    if (fp != NULL) {
	fwrite_fmt(s_end, N_ST_END_FORMAT, fp);
	j = s_end->h.available + sizeof(N_SI4);
	for (i = j; i < s_end->h.length_s - sizeof(N_SI4); i++)
	    fwrite(&uc, 1, 1, fp);
	fwrite_native(&s_end->length_e, sizeof(N_SI4), 1, fp);
    }

}


/* -------------------------------------------------------------------------- */
N_SI4
init_nusdas_file(FILE * fp, N_ST_DEF * s_def, N_SI4 defn, char *type,
		 N_SI4 * basetime, N_UI4 * descriptor, N_SI4 * validtime1,
		 N_ST_NUSD * s_nusd, N_ST_CNTL * s_cntl,
		 N_ST_INDX * s_indx, N_ST_END * s_end, N_ST_SUBC * s_subc,
		 N_ST_INFO * s_info)
/* -------------------------------------------------------------------------- */
{
    N_SI4 trec, srec, irec, tbyte, rt, i;
    long offset;
    trec = srec = irec = tbyte = 0;

    time(&n_systime);
    if ((rt = n_init_nusd(fp, s_nusd, s_def, tbyte, trec, irec, srec)) <=
	0) return (rt);
    tbyte += rt;
    trec++;
    if ((rt = n_init_cntl(fp, s_cntl, s_def, type, basetime,
			  descriptor, validtime1)) <= 0)
	return (rt);
    tbyte += rt;
    trec++;
    if ((rt = n_init_indx(fp, s_indx, s_cntl, s_def,
			  basetime, validtime1)) <= 0)
	return (rt);
    tbyte += rt;
    trec++;
    for (i = 0; i < s_def->n_subc; i++) {
	if (memcmp(&s_def->subc[i], "ETA ", 4) == 0 ||
	    memcmp(&s_def->subc[i], "SIGM", 4) == 0 ||
	    memcmp(&s_def->subc[i], "Z*  ", 4) == 0) {
	    if ((rt = n_init_subc_c(fp, defn, s_cntl, s_def, s_subc, i)) <=
		0) return (rt);
	} else {
	    if ((rt = n_init_subc_v(fp, defn, i, s_cntl, s_def,
				    basetime, validtime1)) <= 0)
		return (rt);
	}
	tbyte += rt;
	trec++;
	srec++;
    }
    for (i = 0; i < s_def->n_infoc; i++) {
	if ((rt = n_init_info(fp, s_info, s_def, &s_def->info[i],
			      s_def->info_file[i])) <= 0)
	    return (rt);
	tbyte += rt;
	trec++;
	irec++;
    }
    s_end->h.length_s = (s_def->forced_rlen == 0) ? sizeof(N_ST_END)
	: s_def->forced_rlen;
    s_end->length_e = s_end->h.length_s;
    tbyte += s_end->h.length_s;
    n_init_end(fp, s_end, tbyte, ++trec);
    offset = 0;
    if (fp != NULL)
	fseek(fp, offset, SEEK_SET);
    if ((rt = n_init_nusd(fp, s_nusd, s_def, tbyte, trec, irec, srec)) <=
	0) return (rt);
    if (fp != NULL)
	fflush(fp);

    return (0);
}
