#define N_DEF_RLENCMP_SOURCE
#include "nusdas_sys.h"

/* -------------------------------------------------------------------------- */
N_SI4
n_decode_rlen_8bit(N_UI1 * udata, N_UI1 * din,
		   N_SI4 nin, N_SI4 nout, N_SI4 maxv)
/* -------------------------------------------------------------------------- */
/* decode ranlength compress ( 8bit data version )                            */
/*   output:   dout   = user data for put(N_SI4)                                     */
/*   input:    din    = compressed data                                       */
/*             nin    = item number of compressed data                        */
/*             nout   = allocated item number for user data                   */
/*             maxv   = maximum value of user data                            */
/*   return:   >=0    = item number of compressed data                        */
/*             -4     = uncompressed data size exceeds nout                   */
/*             -6     = first user data is out of the data range              */
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, k, l, m, n, v, p, w, nbit = 8, *dout;
    N_UI1 *d;

    dout = (N_SI4 *) udata;
    d = din;
    l = n_ipow(2, nbit) - 1 - maxv;
    v = (N_SI4) (*d);
    if (v < 0 || v > maxv)
	return (-6);

    i = 0;
    k = 0;
    p = -1;
    while (i < nin) {
	v = (N_SI4) (*(d + i++));
	if (v <= maxv) {
	    if (p >= 0) {
		for (j = 0; j < m; j++) {
		    if (k == nout)
			return (-4);
		    *(dout + k++) = p;
		}
	    }
	    p = v;
	    m = 1;
	    n = 0;
	} else {
	    m += n_ipow(l, n++) * (v - maxv - 1);
	}
    }
    for (j = 0; j < m; j++) {
	if (k == nout)
	    return (-4);
	*(dout + k++) = p;
    }

    return (k);
}

/* 2003.2.5 tabito added the function which decodes RLEN to I1*/
/*----------------------------------------------------------------------------*/
N_SI4
n_decode_rlen_8bit_I1(N_UI1 * udata, N_UI1 * din,
		   N_SI4 nin, N_SI4 nout, N_SI4 maxv)
/* -------------------------------------------------------------------------- */
/* decode ranlength compress ( 8bit data version, output type is N_UI1)       */
/*   output:   dout   = user data for put (N_UI1)                             */
/*   input:    din    = compressed data                                       */
/*             nin    = item number of compressed data                        */
/*             nout   = allocated item number for user data                   */
/*             maxv   = maximum value of user data                            */
/*   return:   >=0    = item number of compressed data                        */
/*             -4     = uncompressed data size exceeds nout                   */
/*             -6     = first user data is out of the data range              */
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, k, l, m, n, v, p, w, nbit = 8;
    N_UI1 *d,*dout;

    dout = (N_UI1*)udata;
    d = din;
    l = n_ipow(2, nbit) - 1 - maxv;
    v = (N_SI4) (*d);
    if (v < 0 || v > maxv)
	return (-6);

    i = 0;
    k = 0;
    p = -1;
    while (i < nin) {
	v = (N_SI4) (*(d + i++));
	if (v <= maxv) {
	    if (p >= 0) {
		for (j = 0; j < m; j++) {
		    if (k == nout)
			return (-4);
		    *(dout + k++) = p;
		}
	    }
	    p = v;
	    m = 1;
	    n = 0;
	} else {
	    m += n_ipow(l, n++) * (v - maxv - 1);
	}
    }
    for (j = 0; j < m; j++) {
	if (k == nout)
	    return (-4);
	*(dout + k++) = p;
    }

    return (k);
}


/* -------------------------------------------------------------------------- */
N_SI4
n_encode_rlen_8bit(N_UI1 * udata, N_UI1 * dout,
		   N_SI4 nin, N_SI4 nout, N_SI4 * maxv)
/* -------------------------------------------------------------------------- */
/* encode ranlength compress ( 8bit data version )                            */
/*   input:    din    = original user data                                    */
/*   output:   dout   = compressed data                                       */
/*   input:    nin    = item number of original data                          */
/*             nout   = allocated item number for compressed data             */
/*             maxv   = maximum value of user data                            */
/*   return:   >=0    = item number of compressed data                        */
/*             -1     = compressed data size exceeds nout                     */
/*             -2     = some user data is out of the range                    */
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, k, l, v, w, x, nbit = 8, *din;
    N_UI1 *d;

    din = (N_SI4 *) udata;
    d = dout;
    /*x = n_ipow(2, nbit) - 16;*/ /* (2003.2.5 tabito)*/
    x = n_ipow(2, nbit) - 4;
    if (*maxv == -1) {
	for (i = 0, *maxv = LONG_MIN; i < nin; i++)
	    if (*(din + i) < 0 || *(din + i) > x)
		return (-2);
	    else if (*(din + i) > *maxv)
		*maxv = *(din + i);
    } else {
	for (i = 0; i < nin; i++)
	    if (*(din + i) < 0 || *(din + i) > *maxv)
		return (-2);
    }
    l = n_ipow(2, nbit) - 1 - *maxv;

    i = 0;
    k = 0;
    while (i < nin) {
	v = *(din + i++);
	if (k == nout)
	    return (-1);
	*(d + k++) = (N_UI1) v;
	if (i == nin)
	    break;
	j = 0;
	while (1) {
	    w = *(din + i);
	    if (v != w)
		break;
	    j++;
	    if (++i == nin)
		break;
	}
	if (j == 0)
	    continue;
	while (j >= 1) {
	    if (k == nout)
		return (-1);
	    *(d + k++) = (N_UI1) (j % l + *maxv + 1);
	    j /= l;
	}
    }

    return (k);
}
/* 2003.2.5 tabito added the function which encodes I1 user date to RLEN      */
/* -------------------------------------------------------------------------- */
N_SI4
n_encode_rlen_8bit_I1(N_UI1 * udata, N_UI1 * dout,
		   N_SI4 nin, N_SI4 nout, N_SI4 * maxv)
/* -------------------------------------------------------------------------- */
/* encode ranlength compress ( 8bit data version, user data type is N_UI1)    */
/*   input:    din    = original user data (N_UI1)                            */
/*   output:   dout   = compressed data                                       */
/*   input:    nin    = item number of original data                          */
/*             nout   = allocated item number for compressed data             */
/*             maxv   = maximum value of user data                            */
/*   return:   >=0    = item number of compressed data                        */
/*             -1     = compressed data size exceeds nout                     */
/*             -2     = some user data is out of the range                    */
/* -------------------------------------------------------------------------- */
{
    N_SI4 i, j, k, l, v, w, x, nbit = 8;
    N_UI1 *d,*din;

    din = (N_UI1*)udata;
    d = dout;
    /*x = n_ipow(2, nbit) - 16;*/ /* 2003.2.5 tabito */
    x = n_ipow(2, nbit) - 4;
    if (*maxv == -1) {
	for (i = 0, *maxv = LONG_MIN; i < nin; i++)
	    if (*(din + i) > x)
		return (-2);
	    else if (*(din + i) > *maxv)
		*maxv = *(din + i);
    } else {
	for (i = 0; i < nin; i++)
	    if (*(din + i) > *maxv)
		return (-2);
    }
    l = n_ipow(2, nbit) - 1 - *maxv;

    i = 0;
    k = 0;
    while (i < nin) {
	v = *(din + i++);
	if (k == nout)
	    return (-1);
	*(d + k++) = (N_UI1) v;
	if (i == nin)
	    break;
	j = 0;
	while (1) {
	    w = *(din + i);
	    if (v != w)
		break;
	    j++;
	    if (++i == nin)
		break;
	}
	if (j == 0)
	    continue;
	while (j >= 1) {
	    if (k == nout)
		return (-1);
	    *(d + k++) = (N_UI1) (j % l + *maxv + 1);
	    j /= l;
	}
    }

    return (k);
}
