/*
**	systime.c
**
**	99. 5.20-	NPD/JMA
**	00.10.11	debug : init str[] and get only 40 chars at fgets()
**      00.11.08        debug : (NWP_GETTIME) (NWP_GETTIMEENV)
**                              add "\0" at the tail of tcname,envname
**	01.10.10	use stat(), free()
**
** iseq = nwp_systime (        py,pm,pd,ph,pmn,ps)
** iseq = nwp_gettime (tcname, py,pm,pd,ph,pmn)
*/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "nwpl_timecnv.h"

#define PATH_MAX 1023

int
nwp_gettime (char *tcname, int *py, int *pm, int *pd, int *ph, int *pmn)
{
  FILE *fp;
  char str[80], *xstr, *xname;
  int  i;

  *py = *pm = *pd = *ph = *pmn = -1;
  for (i=0; i<80; ++i) str[i]='\0';
  xname=NULL;

  if (tcname)
    {
      xname=strtok(strdup(tcname)," ");	/* remove space */

      if (! (fp=fopen(xname,"r")) )
        { fprintf (stderr,"  file open error [%s] \n",xname);
          free(xname);
          return(-1); }
    }
  else
    { fp=stdin; }	/* if tcname==NULL */

  if (! fgets(str,40,fp))
    { fprintf (stderr,"  file read error [%s] \n",xname);
      if (tcname) fclose(fp);
      if (xname)  free(xname);
      return(-1); }		/* size zero file returns here */

  if (tcname) fclose(fp);

                     xstr=strdup(strcat(str," 0 0 0 0 0"));
  *py  = atoi(strtok(xstr," ")); /* no-field or illegal char become 0 */
  *pm  = atoi(strtok(NULL," "));
  *pd  = atoi(strtok(NULL," "));
  *ph  = atoi(strtok(NULL," "));
  *pmn = atoi(strtok(NULL," "));

  fprintf(stderr,"  file [%s] says : %d %d %d %2.2d %2.2d \n",
          xname, *py,*pm,*pd,*ph,*pmn);

             free(xstr);
  if (xname) free(xname);

  if (*py<1801) { fprintf(stderr,"  ... year<1801 ??? strange !! \n");
                  return(-1); }
  else          { return(nwp_ymdhm2seq(*py,*pm,*pd,*ph,*pmn)); }
}

/**/

static int
nwp_gettimeenv (char *envname, int *py, int *pm, int *pd, int *ph, int *pmn)
{
  char *xstr;
  char *val;

  val=getenv(envname);
  if(val == NULL) { 
    fprintf(stderr,"Environment variable \"%s\" is not defined.\n",envname);
    return -1;
  }

  xstr=strdup(strcat(getenv(envname)," 0 0 0 0 0"));

  *py  = atoi(strtok(xstr," ")); /* no-field or illegal char become 0 */
  *pm  = atoi(strtok(NULL," "));
  *pd  = atoi(strtok(NULL," "));
  *ph  = atoi(strtok(NULL," "));
  *pmn = atoi(strtok(NULL," "));

  free(xstr);

  fprintf(stderr,"  environment [%s] says : %d %d %d %2.2d %2.2d \n",
          envname, *py,*pm,*pd,*ph,*pmn);

  if (*py<1801) { fprintf(stderr,"  ... year<1801 ??? strange !! \n");
                  return(-1); }
  else          { return(nwp_ymdhm2seq(*py,*pm,*pd,*ph,*pmn)); }
}

/**/

int
nwp_systime (int *py, int *pm, int *pd, int *ph, int *pmn, int *ps)
{
  int    iseq,i70seq;
  time_t tloc;
  struct stat buf;

  fprintf (stderr,"\n--- You requested system clock ...\n");

  *ps=0; /* default is zero */
  if ( getenv("SYSTIME") ) /* if exist not-NULL */
    {
      fprintf(stderr,"  environment SYSTIME exists ... \n");
      if ( (iseq=nwp_gettimeenv("SYSTIME", py,pm,pd,ph,pmn)) > 0 )  return(iseq);
    }

  if ( ! stat("SYSTIME",&buf) ) /* stat() returns 0 if file exist */
    {
      fprintf(stderr,"  file SYSTIME exists ... \n");
      if ( (iseq=nwp_gettime("SYSTIME", py,pm,pd,ph,pmn)) > 0 )  return(iseq);
    }

  fprintf(stderr,"  read system-clock ... \n");
  time (&tloc);  /* fprintf(stderr,"  tloc is : %d \n",tloc); */

  i70seq=nwp_ymdhm2seq(1970,1,1,00,00); /* unix standard time */
  *ps  = tloc%60;
  iseq = tloc/60 + i70seq;
  nwp_seq2ymdhm(py,pm,pd,ph,pmn, iseq);

  fprintf(stderr,"  system-clock says : %d %d %d %2.2d %2.2d %2.2d \n",
                  *py,*pm,*pd,*ph,*pmn,*ps);
  
  return(iseq);
}

/**/

/* #define DEBUG */
#ifdef  DEBUG

void
main()
{
  int iy,im,id,ih,imn,is;

  nwp_systime (&iy,&im,&id,&ih,&imn,&is);

  fprintf(stderr,"\nRESULT : %d %d %d %2.2d %2.2d %2.2d \n",iy,im,id,ih,imn,is);
}

#endif

